/** @license AmauiStyleReact v1.0.11
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */
  (function (global, factory) {
            typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('react')) :
            typeof define === 'function' && define.amd ? define(['exports', 'react'], factory) :
            (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global.AmauiStyleReact = {}, global.React));
})(this, (function (exports, React) { 'use strict';

            function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

            var React__default = /*#__PURE__*/_interopDefaultLegacy(React);

            var global$1 = (typeof global !== "undefined" ? global :
                        typeof self !== "undefined" ? self :
                        typeof window !== "undefined" ? window : {});

            var REGEX = /^(?:[0-9a-f]{8}-[0-9a-f]{4}-[1-5][0-9a-f]{3}-[89ab][0-9a-f]{3}-[0-9a-f]{12}|00000000-0000-0000-0000-000000000000)$/i;

            function validate(uuid) {
              return typeof uuid === 'string' && REGEX.test(uuid);
            }

            const isObjectLike = value => typeof value === 'object' && value !== null && !Array.isArray(value);

            const equalDeep = (valueA, valueB) => {
              if (valueA === valueB) return true;
              if (Number.isNaN(valueA) && Number.isNaN(valueB)) return true;
              if (typeof valueA !== typeof valueB && !(isObjectLike(valueA) && isObjectLike(valueB))) return false;
              if (Array.isArray(valueA) && valueA.length === valueB.length) return valueA.every((item, index) => equalDeep(item, valueB[index]));

              if (isObjectLike(valueA)) {
                const valueA_ = { ...valueA
                };
                const valueB_ = { ...valueB
                };
                return Object.keys(valueA_).every(key => equalDeep(valueA_[key], valueB_[key]));
              }

              return false;
            };

            var equalDeep$1 = equalDeep;

            const optionsDefault$l = {}; // Multiple is methods instead of one,
            // so it's lighter for tree shaking usability reasons

            const is$1 = function (type) {
              var _value$constructor;

              let value = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : undefined;
              let options_ = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : optionsDefault$l;
              const options = { ...optionsDefault$l,
                ...options_
              };
              const {
                variant
              } = options;
              const prototype = value && typeof value === 'object' && Object.getPrototypeOf(value);

              switch (type) {
                case 'string':
                  return typeof value === 'string';

                case 'number':
                  return typeof value === 'number' && !Number.isNaN(value);

                case 'boolean':
                  return typeof value === 'boolean';

                case 'array':
                  return Array.isArray(value);

                case 'object':
                  const isObject = typeof value === 'object' && !!value && value.constructor === Object;
                  return isObject;
                // Map, null, WeakMap, Date, etc.

                case 'object-like':
                  return typeof value === 'object' && (value === null || value.constructor !== Object);

                case 'class':
                  return (typeof value === 'object' || typeof value === 'function') && (/class/gi.test(String(value)) || /class/gi.test(String(value === null || value === void 0 ? void 0 : value.constructor)));

                case 'function':
                  return !!(value && value instanceof Function);

                case 'async':
                  // If it's browser avoid calling the method
                  // to see if it's async func or not,
                  // where as in nodejs we have no other choice
                  // that i know of when using transpilation
                  // And also it might not be always correct, as
                  // a method that returns a promise is also async
                  // but we can't know that until the method is called and
                  // we inspect the method's return value
                  return !!(is$1('function', value) && (isEnvironment('browser') ? value.constructor.name === 'AsyncFunction' : value() instanceof Promise));

                case 'map':
                  return !!(prototype === Map.prototype);

                case 'weakmap':
                  return !!(prototype === WeakMap.prototype);

                case 'set':
                  return !!(prototype === Set.prototype);

                case 'weakset':
                  return !!(prototype === WeakSet.prototype);

                case 'promise':
                  return !!(prototype === Promise.prototype);

                case 'int8array':
                  return !!(prototype === Int8Array.prototype);

                case 'uint8array':
                  return !!(prototype === Uint8Array.prototype);

                case 'uint8clampedarray':
                  return !!(prototype === Uint8ClampedArray.prototype);

                case 'int16array':
                  return !!(prototype === Int16Array.prototype);

                case 'uint16array':
                  return !!(prototype === Uint16Array.prototype);

                case 'int32array':
                  return !!(prototype === Int32Array.prototype);

                case 'uint32array':
                  return !!(prototype === Uint32Array.prototype);

                case 'float32array':
                  return !!(prototype === Float32Array.prototype);

                case 'float64array':
                  return !!(prototype === Float64Array.prototype);

                case 'bigint64array':
                  return !!(prototype === BigInt64Array.prototype);

                case 'biguint64array':
                  return !!(prototype === BigUint64Array.prototype);

                case 'typedarray':
                  return is$1('int8array', value) || is$1('uint8array', value) || is$1('uint8clampedarray', value) || is$1('int16array', value) || is$1('uint16array', value) || is$1('int32array', value) || is$1('uint32array', value) || is$1('float32array', value) || is$1('float64array', value) || is$1('bigint64array', value) || is$1('biguint64array', value);

                case 'dataview':
                  return !!(prototype === DataView.prototype);

                case 'arraybuffer':
                  return !!(prototype === ArrayBuffer.prototype);

                case 'sharedarraybuffer':
                  return typeof SharedArrayBuffer !== 'undefined' && !!(prototype === SharedArrayBuffer.prototype);

                case 'symbol':
                  return !!(typeof value === 'symbol');

                case 'error':
                  return !!(value && value instanceof Error);

                case 'date':
                  return !!(value && value instanceof Date);

                case 'regexp':
                  return !!(value && value instanceof RegExp);

                case 'arguments':
                  return !!(value && value.toString() === '[object Arguments]');

                case 'null':
                  return value === null;

                case 'undefined':
                  return value === undefined;

                case 'blob':
                  return isEnvironment('browser') && value instanceof Blob;

                case 'buffer':
                  return !!(isEnvironment('nodejs') && typeof (value === null || value === void 0 ? void 0 : (_value$constructor = value.constructor) === null || _value$constructor === void 0 ? void 0 : _value$constructor.isBuffer) === 'function' && value.constructor.isBuffer(value));

                case 'element':
                  if (value) {
                    switch (variant) {
                      case undefined:
                      case 'html':
                      case 'element':
                        return isEnvironment('browser') && (typeof HTMLElement === 'object' ? value instanceof HTMLElement : value && typeof value === 'object' && value !== null && value.nodeType === 1 && typeof value.nodeName === 'string');

                      case 'node':
                        return isEnvironment('browser') && (typeof Node === 'object' ? value instanceof Node : value && typeof value === 'object' && value !== null && typeof value.nodeType === 'number' && typeof value.nodeName === 'string');

                      case 'react':
                        return value.elementType || value.hasOwnProperty('$$typeof');

                      default:
                        return false;
                    }
                  }

                  return false;

                case 'simple':
                  return is$1('string', value, options) || is$1('number', value, options) || is$1('boolean', value, options) || is$1('undefined', value, options) || is$1('null', value, options);

                case 'not-array-object':
                  return !is$1('array', value, options) && !is$1('object', value, options);

                default:
                  return false;
              }
            };
            const isValid = function (type) {
              var _value_;

              let value = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : undefined;
              let options_ = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : optionsDefault$l;
              const options = { ...optionsDefault$l,
                ...options_
              };
              let valueA;
              let valueB;
              let operator;
              let operators;
              let pattern;
              let value_;

              switch (type) {
                case 'date':
                  return isValid('timestamp', new Date(value).getTime());

                case 'uuid':
                  return validate(value);

                case 'binary-string':
                  value_ = ['0', '1'];
                  return is$1('string', value) && [...value].every(item => value_.indexOf(item) > -1);

                case 'hexadecimal-string':
                  value_ = ['0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f'];
                  return is$1('string', value) && [...value].every(item => value_.indexOf(item) > -1);

                case 'url':
                  pattern = /^(?:http(s)?:\/\/)?[\w.-]+(?:\.[\w\.-]+)+[\w\-\._~:/?#[\]@!\$&'\(\)\*\+,;=.]+$/;
                  return pattern.test(value);

                case 'url-path':
                  pattern = /^\/([^\/][A-Za-z0-9\-._~!$&'()*+,;=:@\/?#%]*)?$/;
                  return pattern.test(value);

                case 'compare':
                  ({
                    valueA,
                    valueB,
                    operator
                  } = options);
                  operators = {
                    'less-than': valueA < valueB,
                    'less-than-equal': valueA <= valueB,
                    'equal': equalDeep$1(valueA, valueB),
                    'not-equal': !equalDeep$1(valueA, valueB),
                    'greater-than-equal': valueA >= valueB,
                    'greater-than': valueA > valueB,
                    'array-all': is$1('array', valueA) && is$1('array', valueB) && valueA.every((_, index) => equalDeep$1(valueA[index], valueB[index])),
                    'array-some': is$1('array', valueA) && is$1('array', valueB) && valueA.some((_, index) => equalDeep$1(valueA[index], valueB[index])),
                    'starts-with': is$1('string', valueA) && valueA.indexOf(valueB) === 0,
                    'contains': is$1('string', valueA) && valueA.indexOf(valueB) > -1
                  };
                  return operators[operator];

                case 'semver':
                  pattern = /^(0|[1-9]\d*)\.(0|[1-9]\d*)\.(0|[1-9]\d*)(?:-((?:0|[1-9]\d*|\d*[a-zA-Z-][0-9a-zA-Z-]*)(?:\.(?:0|[1-9]\d*|\d*[a-zA-Z-][0-9a-zA-Z-]*))*))?(?:\+([0-9a-zA-Z-]+(?:\.[0-9a-zA-Z-]+)*))?$/;
                  return pattern.test(value);

                case 'semver-compare':
                  ({
                    valueA,
                    valueB,
                    operator
                  } = options);
                  if (!(isValid('semver', valueA) && isValid('semver', valueB))) return false;
                  valueA = (valueA.match(/\d+(\.|\-|\+){0,1}/g) || []).map(item => item.replace(/[,\-\+\.]/g, ''));
                  valueB = (valueB.match(/\d+(\.|\-|\+){0,1}/g) || []).map(item => item.replace(/[,\-\+\.]/g, ''));
                  operators = {
                    'less-than': false,
                    'less-than-equal': false,
                    'equal': valueA.every((item, index) => item === valueB[index]),
                    'greater-than-equal': false,
                    'greater-than': false
                  }; // Less then

                  valueA.forEach((item, index) => {
                    if (!operators['less-than']) operators['less-than'] = item < valueB[index];
                  }); // Greater then

                  valueA.forEach((item, index) => {
                    if (!operators['greater-than']) operators['greater-than'] = item > valueB[index];
                  }); // Other or operator values

                  operators['less-than-equal'] = operators['less-than'] || operators['equal'];
                  operators['greater-than-equal'] = operators['greater-than'] || operators['equal'];
                  return operators[operator];

                case 'timestamp':
                  return Number.isInteger(value) && String(value).length >= 10 && (new Date(value).getTime() > 0 || new Date(value * 1000).getTime() > 0);

                case 'mobile':
                  pattern = /^(\+\d{1,2}\s?)?1?-?\.?\s?\(?\d{3}\)?[\s.-]?\d{3}[\s.-]?\d{4}$/;
                  return pattern.test(value) && !Number.isInteger(value);

                case 'email':
                  pattern = /\S+@\S+\.\S+/;
                  return pattern.test(value);

                case 'hash':
                  pattern = /^(0x)?[a-f0-9]{64}$/gi;
                  return is$1('string', value) && pattern.test(value);

                case 'color-rgb':
                  // Matches rgb() and rgba(), with values divided with ',' and spaces (optionaly)
                  pattern = /rgb(a)?\((([0-9]|[1-9][0-9]|1[0-9][0-9]|2[0-4][0-9]|25[0-5]))(\.\d+)?,\s*(([0-9]|[1-9][0-9]|1[0-9][0-9]|2[0-4][0-9]|25[0-5]))(\.\d+)?,\s*(([0-9]|[1-9][0-9]|1[0-9][0-9]|2[0-4][0-9]|25[0-5]))(\.\d+)?(,\s*(?:0(?:\.[0-9]{1,2})?|1(?:\.00?)?))?\)/;
                  return pattern.test(value);

                case 'color-hex':
                  // Matches #nnn, #nnnnnn and #nnnnnnnn (where last nn's are for alpha (optionaly))
                  pattern = /^#((?:[0-9a-fA-F]){3}|(?:[0-9a-fA-F]){6}|(?:[0-9a-fA-F]){8})$/;
                  return pattern.test(value);

                case 'color-hsl':
                  // Matches hsl() and hsla(), with values divided with ',' and spaces (optionaly)
                  pattern = /hsl(a)?\((0|[1-9][0-9]?|[12][0-9][0-9]|3[0-5][0-9])(\.\d+)?,\s*([0-9]|[1-9][0-9]|100)(\.\d+)?%,\s*([0-9]|[1-9][0-9]|100)(\.\d+)?%(,\s*(?:0(?:\.[0-9]{1,2})?|1(?:\.00?)?))?\)/;
                  return pattern.test(value);

                case 'json':
                  try {
                    value_ = JSON.parse(value);
                  } catch (error) {
                    return false;
                  }

                  return is$1('object', value_, options) || is$1('array', value_, options);

                case 'min':
                  return value >= options.min;

                case 'max':
                  return value <= options.max;

                case 'min-max':
                  return isValid('min', value, options) && isValid('max', value, options);

                case 'same-origin':
                  try {
                    value_ = new URL(value);
                  } catch (error) {}

                  return isEnvironment('browser') && (isValid('url-path', value, options) || window.location.hostname === ((_value_ = value_) === null || _value_ === void 0 ? void 0 : _value_.hostname));

                case 'js-chunk':
                  return is$1('object', value, options) && !!value.__esModule && (value.default instanceof Function || value.default instanceof Object);

                case 'http-method':
                  const methods = ['GET', 'POST', 'PUT', 'DELETE', 'HEAD', 'OPTIONS', 'PATCH'];
                  return is$1('string', value, options) && methods.indexOf(value.toUpperCase()) > -1;

                case 'base64':
                  value_ = typeof value === 'string' ? value.trim() : value;
                  return is$1('string', value_, options) && value_.length >= 1 && /^(?:[A-Za-z0-9+\\/]{4})*(?:[A-Za-z0-9+\\/]{2}(==)?|[A-Za-z0-9+\\/]{3}=?)?$/gi.test(value_);

                case 'datauri':
                  value_ = typeof value === 'string' ? value.trim() : value;
                  return is$1('string', value_, options) && /^data:\w+\/[-+.\w]+;base64,(?:[A-Za-z0-9+\\/]{4})*(?:[A-Za-z0-9+\\/]{2}(==)?|[A-Za-z0-9+\\/]{3}=?)?$/gi.test(value_) || /^data:(\w+\/[-+.\w]+)?(;charset=[\w-]+)?,(.*)?/gi.test(value_);

                default:
                  return false;
              }
            };
            const isEnvironment = function (type) {
              let value = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : undefined;
              let value_;

              switch (type) {
                case 'browser':
                  return typeof window !== 'undefined' && typeof window.document !== 'undefined';

                case 'worker':
                  return typeof WorkerGlobalScope !== 'undefined' && self instanceof WorkerGlobalScope;

                case 'nodejs':
                  return !!(typeof global$1 !== 'undefined' && typeof module !== 'undefined' && module.exports);

                case 'localhost':
                  value_ = value !== undefined ? value : isEnvironment('browser') && window.location.hostname;
                  return is$1('string', value_) && ['localhost', '127.0.0.1'].some(value__ => value_.indexOf(value__) > -1);

                default:
                  return false;
              }
            };

            // unlike JSON.stringify usually used for deep simple copy

            const copy = value => {
              if (is$1('array', value)) return value.map(item => copy(item));

              if (is$1('object', value)) {
                const newValue = {};
                Object.keys(value).forEach(key => newValue[key] = copy(value[key]));
                return newValue;
              }

              return value;
            };

            var copy$1 = copy;

            const optionsDefault$k = {
              copy: false,
              merge: {
                array: false
              }
            };

            const merge = function (target, source) {
              let options_ = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : optionsDefault$k;
              const options = { ...optionsDefault$k,
                ...options_
              };

              if (options.merge.array && is$1('array', target) && is$1('array', source)) {
                const length = Math.max(target.length, source.length);

                for (let i = 0; i < length; i++) {
                  if (target[i] === undefined) target[i] = source[i];
                  if (is$1('object', target[i]) && is$1('object', source[i]) || is$1('array', target[i]) && is$1('array', source[i])) target[i] = merge(target[i], source[i], options);
                }
              }

              if (is$1('object', target) && is$1('object', source)) {
                Object.keys(source).forEach(key => {
                  // We only care about direct target object properties
                  // not about inherited properties from a prototype chain
                  if (target.hasOwnProperty(key)) {
                    if (is$1('object', target[key]) && is$1('object', source[key])) target[key] = merge(target[key], source[key], options);
                  } else target[key] = options.copy ? copy$1(source[key]) : source[key];
                });
              }

              return target;
            };

            var merge$1 = merge;

            const optionsDefault$j = {
              decode: false,
              decodeMethod: decodeURIComponent
            };

            const castParam = function (value) {
              let options_ = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : optionsDefault$j;
              const options = merge$1(options_, optionsDefault$j);
              let newValue = value;

              try {
                if (is$1('string', value) && options.decode && is$1('function', options.decodeMethod)) newValue = options.decodeMethod(value);
              } catch (error) {}

              try {
                if (is$1('string', newValue)) {
                  if ('undefined' === newValue) return undefined;
                  if ('NaN' === newValue) return NaN;
                  return JSON.parse(newValue);
                }

                return newValue;
              } catch (error) {}

              return newValue;
            };

            var castParam$1 = castParam;

            const resolve = value => {
              if (value === undefined) return 'undefined';
              if (value instanceof Function || value instanceof Object) return value.toString();
              return value;
            };

            const clean = value => {
              if (is$1('string', value)) return value.replace(/(\s|\r|\n)+/, ' ');
              return value;
            };

            const serializeValue = (value_, method) => {
              let value = method(value_); // Ref circular value

              if (value === undefined) return ''; // Is object-like make into an object

              try {
                if (is$1('object-like', value) && is$1('not-array-object', value) && value !== null) value = { ...value
                };
              } catch (error) {}

              if (is$1('object', value)) return "{".concat(Object.keys(value).sort().map(key => "\"".concat(key, "\":").concat(serializeValue(value[key], method))).filter(item => item.slice(-1) !== ':').join(','), "}");
              if (is$1('array', value)) return "[".concat(value.map(value__ => serializeValue(value__, method)).filter(Boolean).join(','), "]");
              if (is$1('string', value)) return "\"".concat(value, "\"");
              return clean(JSON.stringify(resolve(value)));
            };

            const serialize = value => {
              const values = new WeakSet();

              const getValue = value_ => {
                if (typeof value_ === 'object' && value_ !== null) {
                  if (values.has(value_)) return;
                  values.add(value_);
                }

                return value_;
              };

              return serializeValue(value, getValue);
            };

            var serialize$1 = serialize;

            const capitalize$1 = value => {
              if (is$1('string', value)) return "".concat(value.charAt(0).toUpperCase()).concat(value.slice(1));
              return value;
            };

            var capitalize$2 = capitalize$1;

            function commonjsRequire (path) {
            	throw new Error('Could not dynamically require "' + path + '". Please configure the dynamicRequireTargets or/and ignoreDynamicRequires option of @rollup/plugin-commonjs appropriately for this require call to work.');
            }

            var sha256 = {exports: {}};

            var core = {exports: {}};

            (function (module, exports) {
            (function (root, factory) {
            	{
            		// CommonJS
            		module.exports = factory();
            	}
            }(this, function () {

            	/*globals window, global, require*/

            	/**
            	 * CryptoJS core components.
            	 */
            	var CryptoJS = CryptoJS || (function (Math, undefined$1) {

            	    var crypto;

            	    // Native crypto from window (Browser)
            	    if (typeof window !== 'undefined' && window.crypto) {
            	        crypto = window.crypto;
            	    }

            	    // Native crypto in web worker (Browser)
            	    if (typeof self !== 'undefined' && self.crypto) {
            	        crypto = self.crypto;
            	    }

            	    // Native crypto from worker
            	    if (typeof globalThis !== 'undefined' && globalThis.crypto) {
            	        crypto = globalThis.crypto;
            	    }

            	    // Native (experimental IE 11) crypto from window (Browser)
            	    if (!crypto && typeof window !== 'undefined' && window.msCrypto) {
            	        crypto = window.msCrypto;
            	    }

            	    // Native crypto from global (NodeJS)
            	    if (!crypto && typeof global$1 !== 'undefined' && global$1.crypto) {
            	        crypto = global$1.crypto;
            	    }

            	    // Native crypto import via require (NodeJS)
            	    if (!crypto && typeof commonjsRequire === 'function') {
            	        try {
            	            crypto = require('crypto');
            	        } catch (err) {}
            	    }

            	    /*
            	     * Cryptographically secure pseudorandom number generator
            	     *
            	     * As Math.random() is cryptographically not safe to use
            	     */
            	    var cryptoSecureRandomInt = function () {
            	        if (crypto) {
            	            // Use getRandomValues method (Browser)
            	            if (typeof crypto.getRandomValues === 'function') {
            	                try {
            	                    return crypto.getRandomValues(new Uint32Array(1))[0];
            	                } catch (err) {}
            	            }

            	            // Use randomBytes method (NodeJS)
            	            if (typeof crypto.randomBytes === 'function') {
            	                try {
            	                    return crypto.randomBytes(4).readInt32LE();
            	                } catch (err) {}
            	            }
            	        }

            	        throw new Error('Native crypto module could not be used to get secure random number.');
            	    };

            	    /*
            	     * Local polyfill of Object.create

            	     */
            	    var create = Object.create || (function () {
            	        function F() {}

            	        return function (obj) {
            	            var subtype;

            	            F.prototype = obj;

            	            subtype = new F();

            	            F.prototype = null;

            	            return subtype;
            	        };
            	    }());

            	    /**
            	     * CryptoJS namespace.
            	     */
            	    var C = {};

            	    /**
            	     * Library namespace.
            	     */
            	    var C_lib = C.lib = {};

            	    /**
            	     * Base object for prototypal inheritance.
            	     */
            	    var Base = C_lib.Base = (function () {


            	        return {
            	            /**
            	             * Creates a new object that inherits from this object.
            	             *
            	             * @param {Object} overrides Properties to copy into the new object.
            	             *
            	             * @return {Object} The new object.
            	             *
            	             * @static
            	             *
            	             * @example
            	             *
            	             *     var MyType = CryptoJS.lib.Base.extend({
            	             *         field: 'value',
            	             *
            	             *         method: function () {
            	             *         }
            	             *     });
            	             */
            	            extend: function (overrides) {
            	                // Spawn
            	                var subtype = create(this);

            	                // Augment
            	                if (overrides) {
            	                    subtype.mixIn(overrides);
            	                }

            	                // Create default initializer
            	                if (!subtype.hasOwnProperty('init') || this.init === subtype.init) {
            	                    subtype.init = function () {
            	                        subtype.$super.init.apply(this, arguments);
            	                    };
            	                }

            	                // Initializer's prototype is the subtype object
            	                subtype.init.prototype = subtype;

            	                // Reference supertype
            	                subtype.$super = this;

            	                return subtype;
            	            },

            	            /**
            	             * Extends this object and runs the init method.
            	             * Arguments to create() will be passed to init().
            	             *
            	             * @return {Object} The new object.
            	             *
            	             * @static
            	             *
            	             * @example
            	             *
            	             *     var instance = MyType.create();
            	             */
            	            create: function () {
            	                var instance = this.extend();
            	                instance.init.apply(instance, arguments);

            	                return instance;
            	            },

            	            /**
            	             * Initializes a newly created object.
            	             * Override this method to add some logic when your objects are created.
            	             *
            	             * @example
            	             *
            	             *     var MyType = CryptoJS.lib.Base.extend({
            	             *         init: function () {
            	             *             // ...
            	             *         }
            	             *     });
            	             */
            	            init: function () {
            	            },

            	            /**
            	             * Copies properties into this object.
            	             *
            	             * @param {Object} properties The properties to mix in.
            	             *
            	             * @example
            	             *
            	             *     MyType.mixIn({
            	             *         field: 'value'
            	             *     });
            	             */
            	            mixIn: function (properties) {
            	                for (var propertyName in properties) {
            	                    if (properties.hasOwnProperty(propertyName)) {
            	                        this[propertyName] = properties[propertyName];
            	                    }
            	                }

            	                // IE won't copy toString using the loop above
            	                if (properties.hasOwnProperty('toString')) {
            	                    this.toString = properties.toString;
            	                }
            	            },

            	            /**
            	             * Creates a copy of this object.
            	             *
            	             * @return {Object} The clone.
            	             *
            	             * @example
            	             *
            	             *     var clone = instance.clone();
            	             */
            	            clone: function () {
            	                return this.init.prototype.extend(this);
            	            }
            	        };
            	    }());

            	    /**
            	     * An array of 32-bit words.
            	     *
            	     * @property {Array} words The array of 32-bit words.
            	     * @property {number} sigBytes The number of significant bytes in this word array.
            	     */
            	    var WordArray = C_lib.WordArray = Base.extend({
            	        /**
            	         * Initializes a newly created word array.
            	         *
            	         * @param {Array} words (Optional) An array of 32-bit words.
            	         * @param {number} sigBytes (Optional) The number of significant bytes in the words.
            	         *
            	         * @example
            	         *
            	         *     var wordArray = CryptoJS.lib.WordArray.create();
            	         *     var wordArray = CryptoJS.lib.WordArray.create([0x00010203, 0x04050607]);
            	         *     var wordArray = CryptoJS.lib.WordArray.create([0x00010203, 0x04050607], 6);
            	         */
            	        init: function (words, sigBytes) {
            	            words = this.words = words || [];

            	            if (sigBytes != undefined$1) {
            	                this.sigBytes = sigBytes;
            	            } else {
            	                this.sigBytes = words.length * 4;
            	            }
            	        },

            	        /**
            	         * Converts this word array to a string.
            	         *
            	         * @param {Encoder} encoder (Optional) The encoding strategy to use. Default: CryptoJS.enc.Hex
            	         *
            	         * @return {string} The stringified word array.
            	         *
            	         * @example
            	         *
            	         *     var string = wordArray + '';
            	         *     var string = wordArray.toString();
            	         *     var string = wordArray.toString(CryptoJS.enc.Utf8);
            	         */
            	        toString: function (encoder) {
            	            return (encoder || Hex).stringify(this);
            	        },

            	        /**
            	         * Concatenates a word array to this word array.
            	         *
            	         * @param {WordArray} wordArray The word array to append.
            	         *
            	         * @return {WordArray} This word array.
            	         *
            	         * @example
            	         *
            	         *     wordArray1.concat(wordArray2);
            	         */
            	        concat: function (wordArray) {
            	            // Shortcuts
            	            var thisWords = this.words;
            	            var thatWords = wordArray.words;
            	            var thisSigBytes = this.sigBytes;
            	            var thatSigBytes = wordArray.sigBytes;

            	            // Clamp excess bits
            	            this.clamp();

            	            // Concat
            	            if (thisSigBytes % 4) {
            	                // Copy one byte at a time
            	                for (var i = 0; i < thatSigBytes; i++) {
            	                    var thatByte = (thatWords[i >>> 2] >>> (24 - (i % 4) * 8)) & 0xff;
            	                    thisWords[(thisSigBytes + i) >>> 2] |= thatByte << (24 - ((thisSigBytes + i) % 4) * 8);
            	                }
            	            } else {
            	                // Copy one word at a time
            	                for (var j = 0; j < thatSigBytes; j += 4) {
            	                    thisWords[(thisSigBytes + j) >>> 2] = thatWords[j >>> 2];
            	                }
            	            }
            	            this.sigBytes += thatSigBytes;

            	            // Chainable
            	            return this;
            	        },

            	        /**
            	         * Removes insignificant bits.
            	         *
            	         * @example
            	         *
            	         *     wordArray.clamp();
            	         */
            	        clamp: function () {
            	            // Shortcuts
            	            var words = this.words;
            	            var sigBytes = this.sigBytes;

            	            // Clamp
            	            words[sigBytes >>> 2] &= 0xffffffff << (32 - (sigBytes % 4) * 8);
            	            words.length = Math.ceil(sigBytes / 4);
            	        },

            	        /**
            	         * Creates a copy of this word array.
            	         *
            	         * @return {WordArray} The clone.
            	         *
            	         * @example
            	         *
            	         *     var clone = wordArray.clone();
            	         */
            	        clone: function () {
            	            var clone = Base.clone.call(this);
            	            clone.words = this.words.slice(0);

            	            return clone;
            	        },

            	        /**
            	         * Creates a word array filled with random bytes.
            	         *
            	         * @param {number} nBytes The number of random bytes to generate.
            	         *
            	         * @return {WordArray} The random word array.
            	         *
            	         * @static
            	         *
            	         * @example
            	         *
            	         *     var wordArray = CryptoJS.lib.WordArray.random(16);
            	         */
            	        random: function (nBytes) {
            	            var words = [];

            	            for (var i = 0; i < nBytes; i += 4) {
            	                words.push(cryptoSecureRandomInt());
            	            }

            	            return new WordArray.init(words, nBytes);
            	        }
            	    });

            	    /**
            	     * Encoder namespace.
            	     */
            	    var C_enc = C.enc = {};

            	    /**
            	     * Hex encoding strategy.
            	     */
            	    var Hex = C_enc.Hex = {
            	        /**
            	         * Converts a word array to a hex string.
            	         *
            	         * @param {WordArray} wordArray The word array.
            	         *
            	         * @return {string} The hex string.
            	         *
            	         * @static
            	         *
            	         * @example
            	         *
            	         *     var hexString = CryptoJS.enc.Hex.stringify(wordArray);
            	         */
            	        stringify: function (wordArray) {
            	            // Shortcuts
            	            var words = wordArray.words;
            	            var sigBytes = wordArray.sigBytes;

            	            // Convert
            	            var hexChars = [];
            	            for (var i = 0; i < sigBytes; i++) {
            	                var bite = (words[i >>> 2] >>> (24 - (i % 4) * 8)) & 0xff;
            	                hexChars.push((bite >>> 4).toString(16));
            	                hexChars.push((bite & 0x0f).toString(16));
            	            }

            	            return hexChars.join('');
            	        },

            	        /**
            	         * Converts a hex string to a word array.
            	         *
            	         * @param {string} hexStr The hex string.
            	         *
            	         * @return {WordArray} The word array.
            	         *
            	         * @static
            	         *
            	         * @example
            	         *
            	         *     var wordArray = CryptoJS.enc.Hex.parse(hexString);
            	         */
            	        parse: function (hexStr) {
            	            // Shortcut
            	            var hexStrLength = hexStr.length;

            	            // Convert
            	            var words = [];
            	            for (var i = 0; i < hexStrLength; i += 2) {
            	                words[i >>> 3] |= parseInt(hexStr.substr(i, 2), 16) << (24 - (i % 8) * 4);
            	            }

            	            return new WordArray.init(words, hexStrLength / 2);
            	        }
            	    };

            	    /**
            	     * Latin1 encoding strategy.
            	     */
            	    var Latin1 = C_enc.Latin1 = {
            	        /**
            	         * Converts a word array to a Latin1 string.
            	         *
            	         * @param {WordArray} wordArray The word array.
            	         *
            	         * @return {string} The Latin1 string.
            	         *
            	         * @static
            	         *
            	         * @example
            	         *
            	         *     var latin1String = CryptoJS.enc.Latin1.stringify(wordArray);
            	         */
            	        stringify: function (wordArray) {
            	            // Shortcuts
            	            var words = wordArray.words;
            	            var sigBytes = wordArray.sigBytes;

            	            // Convert
            	            var latin1Chars = [];
            	            for (var i = 0; i < sigBytes; i++) {
            	                var bite = (words[i >>> 2] >>> (24 - (i % 4) * 8)) & 0xff;
            	                latin1Chars.push(String.fromCharCode(bite));
            	            }

            	            return latin1Chars.join('');
            	        },

            	        /**
            	         * Converts a Latin1 string to a word array.
            	         *
            	         * @param {string} latin1Str The Latin1 string.
            	         *
            	         * @return {WordArray} The word array.
            	         *
            	         * @static
            	         *
            	         * @example
            	         *
            	         *     var wordArray = CryptoJS.enc.Latin1.parse(latin1String);
            	         */
            	        parse: function (latin1Str) {
            	            // Shortcut
            	            var latin1StrLength = latin1Str.length;

            	            // Convert
            	            var words = [];
            	            for (var i = 0; i < latin1StrLength; i++) {
            	                words[i >>> 2] |= (latin1Str.charCodeAt(i) & 0xff) << (24 - (i % 4) * 8);
            	            }

            	            return new WordArray.init(words, latin1StrLength);
            	        }
            	    };

            	    /**
            	     * UTF-8 encoding strategy.
            	     */
            	    var Utf8 = C_enc.Utf8 = {
            	        /**
            	         * Converts a word array to a UTF-8 string.
            	         *
            	         * @param {WordArray} wordArray The word array.
            	         *
            	         * @return {string} The UTF-8 string.
            	         *
            	         * @static
            	         *
            	         * @example
            	         *
            	         *     var utf8String = CryptoJS.enc.Utf8.stringify(wordArray);
            	         */
            	        stringify: function (wordArray) {
            	            try {
            	                return decodeURIComponent(escape(Latin1.stringify(wordArray)));
            	            } catch (e) {
            	                throw new Error('Malformed UTF-8 data');
            	            }
            	        },

            	        /**
            	         * Converts a UTF-8 string to a word array.
            	         *
            	         * @param {string} utf8Str The UTF-8 string.
            	         *
            	         * @return {WordArray} The word array.
            	         *
            	         * @static
            	         *
            	         * @example
            	         *
            	         *     var wordArray = CryptoJS.enc.Utf8.parse(utf8String);
            	         */
            	        parse: function (utf8Str) {
            	            return Latin1.parse(unescape(encodeURIComponent(utf8Str)));
            	        }
            	    };

            	    /**
            	     * Abstract buffered block algorithm template.
            	     *
            	     * The property blockSize must be implemented in a concrete subtype.
            	     *
            	     * @property {number} _minBufferSize The number of blocks that should be kept unprocessed in the buffer. Default: 0
            	     */
            	    var BufferedBlockAlgorithm = C_lib.BufferedBlockAlgorithm = Base.extend({
            	        /**
            	         * Resets this block algorithm's data buffer to its initial state.
            	         *
            	         * @example
            	         *
            	         *     bufferedBlockAlgorithm.reset();
            	         */
            	        reset: function () {
            	            // Initial values
            	            this._data = new WordArray.init();
            	            this._nDataBytes = 0;
            	        },

            	        /**
            	         * Adds new data to this block algorithm's buffer.
            	         *
            	         * @param {WordArray|string} data The data to append. Strings are converted to a WordArray using UTF-8.
            	         *
            	         * @example
            	         *
            	         *     bufferedBlockAlgorithm._append('data');
            	         *     bufferedBlockAlgorithm._append(wordArray);
            	         */
            	        _append: function (data) {
            	            // Convert string to WordArray, else assume WordArray already
            	            if (typeof data == 'string') {
            	                data = Utf8.parse(data);
            	            }

            	            // Append
            	            this._data.concat(data);
            	            this._nDataBytes += data.sigBytes;
            	        },

            	        /**
            	         * Processes available data blocks.
            	         *
            	         * This method invokes _doProcessBlock(offset), which must be implemented by a concrete subtype.
            	         *
            	         * @param {boolean} doFlush Whether all blocks and partial blocks should be processed.
            	         *
            	         * @return {WordArray} The processed data.
            	         *
            	         * @example
            	         *
            	         *     var processedData = bufferedBlockAlgorithm._process();
            	         *     var processedData = bufferedBlockAlgorithm._process(!!'flush');
            	         */
            	        _process: function (doFlush) {
            	            var processedWords;

            	            // Shortcuts
            	            var data = this._data;
            	            var dataWords = data.words;
            	            var dataSigBytes = data.sigBytes;
            	            var blockSize = this.blockSize;
            	            var blockSizeBytes = blockSize * 4;

            	            // Count blocks ready
            	            var nBlocksReady = dataSigBytes / blockSizeBytes;
            	            if (doFlush) {
            	                // Round up to include partial blocks
            	                nBlocksReady = Math.ceil(nBlocksReady);
            	            } else {
            	                // Round down to include only full blocks,
            	                // less the number of blocks that must remain in the buffer
            	                nBlocksReady = Math.max((nBlocksReady | 0) - this._minBufferSize, 0);
            	            }

            	            // Count words ready
            	            var nWordsReady = nBlocksReady * blockSize;

            	            // Count bytes ready
            	            var nBytesReady = Math.min(nWordsReady * 4, dataSigBytes);

            	            // Process blocks
            	            if (nWordsReady) {
            	                for (var offset = 0; offset < nWordsReady; offset += blockSize) {
            	                    // Perform concrete-algorithm logic
            	                    this._doProcessBlock(dataWords, offset);
            	                }

            	                // Remove processed words
            	                processedWords = dataWords.splice(0, nWordsReady);
            	                data.sigBytes -= nBytesReady;
            	            }

            	            // Return processed words
            	            return new WordArray.init(processedWords, nBytesReady);
            	        },

            	        /**
            	         * Creates a copy of this object.
            	         *
            	         * @return {Object} The clone.
            	         *
            	         * @example
            	         *
            	         *     var clone = bufferedBlockAlgorithm.clone();
            	         */
            	        clone: function () {
            	            var clone = Base.clone.call(this);
            	            clone._data = this._data.clone();

            	            return clone;
            	        },

            	        _minBufferSize: 0
            	    });

            	    /**
            	     * Abstract hasher template.
            	     *
            	     * @property {number} blockSize The number of 32-bit words this hasher operates on. Default: 16 (512 bits)
            	     */
            	    C_lib.Hasher = BufferedBlockAlgorithm.extend({
            	        /**
            	         * Configuration options.
            	         */
            	        cfg: Base.extend(),

            	        /**
            	         * Initializes a newly created hasher.
            	         *
            	         * @param {Object} cfg (Optional) The configuration options to use for this hash computation.
            	         *
            	         * @example
            	         *
            	         *     var hasher = CryptoJS.algo.SHA256.create();
            	         */
            	        init: function (cfg) {
            	            // Apply config defaults
            	            this.cfg = this.cfg.extend(cfg);

            	            // Set initial values
            	            this.reset();
            	        },

            	        /**
            	         * Resets this hasher to its initial state.
            	         *
            	         * @example
            	         *
            	         *     hasher.reset();
            	         */
            	        reset: function () {
            	            // Reset data buffer
            	            BufferedBlockAlgorithm.reset.call(this);

            	            // Perform concrete-hasher logic
            	            this._doReset();
            	        },

            	        /**
            	         * Updates this hasher with a message.
            	         *
            	         * @param {WordArray|string} messageUpdate The message to append.
            	         *
            	         * @return {Hasher} This hasher.
            	         *
            	         * @example
            	         *
            	         *     hasher.update('message');
            	         *     hasher.update(wordArray);
            	         */
            	        update: function (messageUpdate) {
            	            // Append
            	            this._append(messageUpdate);

            	            // Update the hash
            	            this._process();

            	            // Chainable
            	            return this;
            	        },

            	        /**
            	         * Finalizes the hash computation.
            	         * Note that the finalize operation is effectively a destructive, read-once operation.
            	         *
            	         * @param {WordArray|string} messageUpdate (Optional) A final message update.
            	         *
            	         * @return {WordArray} The hash.
            	         *
            	         * @example
            	         *
            	         *     var hash = hasher.finalize();
            	         *     var hash = hasher.finalize('message');
            	         *     var hash = hasher.finalize(wordArray);
            	         */
            	        finalize: function (messageUpdate) {
            	            // Final message update
            	            if (messageUpdate) {
            	                this._append(messageUpdate);
            	            }

            	            // Perform concrete-hasher logic
            	            var hash = this._doFinalize();

            	            return hash;
            	        },

            	        blockSize: 512/32,

            	        /**
            	         * Creates a shortcut function to a hasher's object interface.
            	         *
            	         * @param {Hasher} hasher The hasher to create a helper for.
            	         *
            	         * @return {Function} The shortcut function.
            	         *
            	         * @static
            	         *
            	         * @example
            	         *
            	         *     var SHA256 = CryptoJS.lib.Hasher._createHelper(CryptoJS.algo.SHA256);
            	         */
            	        _createHelper: function (hasher) {
            	            return function (message, cfg) {
            	                return new hasher.init(cfg).finalize(message);
            	            };
            	        },

            	        /**
            	         * Creates a shortcut function to the HMAC's object interface.
            	         *
            	         * @param {Hasher} hasher The hasher to use in this HMAC helper.
            	         *
            	         * @return {Function} The shortcut function.
            	         *
            	         * @static
            	         *
            	         * @example
            	         *
            	         *     var HmacSHA256 = CryptoJS.lib.Hasher._createHmacHelper(CryptoJS.algo.SHA256);
            	         */
            	        _createHmacHelper: function (hasher) {
            	            return function (message, key) {
            	                return new C_algo.HMAC.init(hasher, key).finalize(message);
            	            };
            	        }
            	    });

            	    /**
            	     * Algorithm namespace.
            	     */
            	    var C_algo = C.algo = {};

            	    return C;
            	}(Math));


            	return CryptoJS;

            }));
            }(core));

            (function (module, exports) {
            (function (root, factory) {
            	{
            		// CommonJS
            		module.exports = factory(core.exports);
            	}
            }(this, function (CryptoJS) {

            	(function (Math) {
            	    // Shortcuts
            	    var C = CryptoJS;
            	    var C_lib = C.lib;
            	    var WordArray = C_lib.WordArray;
            	    var Hasher = C_lib.Hasher;
            	    var C_algo = C.algo;

            	    // Initialization and round constants tables
            	    var H = [];
            	    var K = [];

            	    // Compute constants
            	    (function () {
            	        function isPrime(n) {
            	            var sqrtN = Math.sqrt(n);
            	            for (var factor = 2; factor <= sqrtN; factor++) {
            	                if (!(n % factor)) {
            	                    return false;
            	                }
            	            }

            	            return true;
            	        }

            	        function getFractionalBits(n) {
            	            return ((n - (n | 0)) * 0x100000000) | 0;
            	        }

            	        var n = 2;
            	        var nPrime = 0;
            	        while (nPrime < 64) {
            	            if (isPrime(n)) {
            	                if (nPrime < 8) {
            	                    H[nPrime] = getFractionalBits(Math.pow(n, 1 / 2));
            	                }
            	                K[nPrime] = getFractionalBits(Math.pow(n, 1 / 3));

            	                nPrime++;
            	            }

            	            n++;
            	        }
            	    }());

            	    // Reusable object
            	    var W = [];

            	    /**
            	     * SHA-256 hash algorithm.
            	     */
            	    var SHA256 = C_algo.SHA256 = Hasher.extend({
            	        _doReset: function () {
            	            this._hash = new WordArray.init(H.slice(0));
            	        },

            	        _doProcessBlock: function (M, offset) {
            	            // Shortcut
            	            var H = this._hash.words;

            	            // Working variables
            	            var a = H[0];
            	            var b = H[1];
            	            var c = H[2];
            	            var d = H[3];
            	            var e = H[4];
            	            var f = H[5];
            	            var g = H[6];
            	            var h = H[7];

            	            // Computation
            	            for (var i = 0; i < 64; i++) {
            	                if (i < 16) {
            	                    W[i] = M[offset + i] | 0;
            	                } else {
            	                    var gamma0x = W[i - 15];
            	                    var gamma0  = ((gamma0x << 25) | (gamma0x >>> 7))  ^
            	                                  ((gamma0x << 14) | (gamma0x >>> 18)) ^
            	                                   (gamma0x >>> 3);

            	                    var gamma1x = W[i - 2];
            	                    var gamma1  = ((gamma1x << 15) | (gamma1x >>> 17)) ^
            	                                  ((gamma1x << 13) | (gamma1x >>> 19)) ^
            	                                   (gamma1x >>> 10);

            	                    W[i] = gamma0 + W[i - 7] + gamma1 + W[i - 16];
            	                }

            	                var ch  = (e & f) ^ (~e & g);
            	                var maj = (a & b) ^ (a & c) ^ (b & c);

            	                var sigma0 = ((a << 30) | (a >>> 2)) ^ ((a << 19) | (a >>> 13)) ^ ((a << 10) | (a >>> 22));
            	                var sigma1 = ((e << 26) | (e >>> 6)) ^ ((e << 21) | (e >>> 11)) ^ ((e << 7)  | (e >>> 25));

            	                var t1 = h + sigma1 + ch + K[i] + W[i];
            	                var t2 = sigma0 + maj;

            	                h = g;
            	                g = f;
            	                f = e;
            	                e = (d + t1) | 0;
            	                d = c;
            	                c = b;
            	                b = a;
            	                a = (t1 + t2) | 0;
            	            }

            	            // Intermediate hash value
            	            H[0] = (H[0] + a) | 0;
            	            H[1] = (H[1] + b) | 0;
            	            H[2] = (H[2] + c) | 0;
            	            H[3] = (H[3] + d) | 0;
            	            H[4] = (H[4] + e) | 0;
            	            H[5] = (H[5] + f) | 0;
            	            H[6] = (H[6] + g) | 0;
            	            H[7] = (H[7] + h) | 0;
            	        },

            	        _doFinalize: function () {
            	            // Shortcuts
            	            var data = this._data;
            	            var dataWords = data.words;

            	            var nBitsTotal = this._nDataBytes * 8;
            	            var nBitsLeft = data.sigBytes * 8;

            	            // Add padding
            	            dataWords[nBitsLeft >>> 5] |= 0x80 << (24 - nBitsLeft % 32);
            	            dataWords[(((nBitsLeft + 64) >>> 9) << 4) + 14] = Math.floor(nBitsTotal / 0x100000000);
            	            dataWords[(((nBitsLeft + 64) >>> 9) << 4) + 15] = nBitsTotal;
            	            data.sigBytes = dataWords.length * 4;

            	            // Hash final blocks
            	            this._process();

            	            // Return final computed hash
            	            return this._hash;
            	        },

            	        clone: function () {
            	            var clone = Hasher.clone.call(this);
            	            clone._hash = this._hash.clone();

            	            return clone;
            	        }
            	    });

            	    /**
            	     * Shortcut function to the hasher's object interface.
            	     *
            	     * @param {WordArray|string} message The message to hash.
            	     *
            	     * @return {WordArray} The hash.
            	     *
            	     * @static
            	     *
            	     * @example
            	     *
            	     *     var hash = CryptoJS.SHA256('message');
            	     *     var hash = CryptoJS.SHA256(wordArray);
            	     */
            	    C.SHA256 = Hasher._createHelper(SHA256);

            	    /**
            	     * Shortcut function to the HMAC's object interface.
            	     *
            	     * @param {WordArray|string} message The message to hash.
            	     * @param {WordArray|string} key The secret key.
            	     *
            	     * @return {WordArray} The HMAC.
            	     *
            	     * @static
            	     *
            	     * @example
            	     *
            	     *     var hmac = CryptoJS.HmacSHA256(message, key);
            	     */
            	    C.HmacSHA256 = Hasher._createHmacHelper(SHA256);
            	}(Math));


            	return CryptoJS.SHA256;

            }));
            }(sha256));

            var SHA256 = sha256.exports;

            const optionsDefault$i = {
              serialize: true,
              withPrefix: true
            };

            const hash = function (value_) {
              let options_ = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : optionsDefault$i;
              const options = merge$1(options_, optionsDefault$i);
              let value = value_;
              if (options.serialize) value = serialize$1(value);
              value = SHA256(value).toString();
              return options.withPrefix ? "0x".concat(value) : value;
            };

            var hash$1 = hash;

            const optionsDefault$h = {
              filters: [',', '.', '-', '_', '\s+'],
              replaceWith: ' ',
              trim: true
            };

            class AmauiCache {
              static caches = {};

              static add(value) {
                for (var _len = arguments.length, args = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
                  args[_key - 1] = arguments[_key];
                }

                const key = hash$1(args);
                if (!this.caches[key]) this.caches[key] = value;
              }

              static get() {
                for (var _len2 = arguments.length, args = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
                  args[_key2] = arguments[_key2];
                }

                const key = hash$1(args);
                return this.caches[key];
              }

              static has() {
                for (var _len3 = arguments.length, args = new Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
                  args[_key3] = arguments[_key3];
                }

                const key = hash$1(args);
                return this.caches.hasOwnProperty(key);
              }

            }

            const cleanValue = function (value_) {
              let options_ = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : optionsDefault$h;

              try {
                const options = merge$1(options_, optionsDefault$h);
                if (AmauiCache.has(value_, options)) return AmauiCache.get(value_, options); // Few predefined options
                // for className cammel case to regular
                // css property names convert

                if (options.className) {
                  options.replaceWith = '-';
                  options.cammelCaseTransform = true;
                  options.lowercase = true;
                }

                const saveToCache = value => AmauiCache.add(value, value_, options_);

                if (is$1('string', value_)) {
                  let value = value_;

                  if (options.url) {
                    const parts = value.split('?').filter(Boolean);
                    let path = parts[0];
                    const query = parts[1];
                    if (path.slice(-1) === '/') path = path.slice(0, -1);
                    value = query ? [path, query].join('?') : path; // Save to cache

                    saveToCache(value);
                    return value;
                  }

                  if (options.cammelCaseTransform) value = value.split(/(?=[A-Z])/g).join(options.replaceWith || ' ');
                  options.filters.forEach(filter => {
                    const expression = "\\".concat(filter);
                    const regexp = new RegExp(expression, 'g');
                    value = value.replace(regexp, options.replaceWith || ' ');
                  });
                  if (options.trim) value = value.trim();
                  if (options.capitalize) value = capitalize$2(value);
                  if (options.lowercase) value = value.toLocaleLowerCase(); // Save to cache

                  saveToCache(value);
                  return value;
                } // Save to cache


                saveToCache(value_);
                return value_;
              } catch (error) {}

              return value_;
            };

            var cleanValue$1 = cleanValue;

            const getObjectPropertyValue = (object, keys) => {
              if (!object || !keys) return;

              if (is$1('string', keys)) {
                const keys_ = keys.split('.').filter(Boolean).map(key => castParam$1(key));
                return getObjectPropertyValue(object, keys_);
              }

              if (is$1('array', keys)) {
                const key = keys[0];
                if (keys.length === 1) return object[key];
                if (object.hasOwnProperty(key)) return getObjectPropertyValue(object[key], keys.slice(1));
              }
            };
            var getObjectPropertyValue$1 = getObjectPropertyValue;

            const getObjectValue = function (object) {
              for (var _len = arguments.length, args = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
                args[_key - 1] = arguments[_key];
              }

              if (!object || !args.length) return;
              let value;
              const keys = args.filter(Boolean);

              for (const key of keys) {
                value = getObjectPropertyValue$1(object, key);
                if (value !== undefined) return value;
              }
            };

            var getObjectValue$1 = getObjectValue;

            const optionsDefault$g = {
              valueOverride: false
            };

            const setObjectValue = function (object) {
              let keys = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '';
              let value = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : undefined;
              let options_ = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : optionsDefault$g;
              const options = merge$1(options_, optionsDefault$g);
              if (!(object || keys)) return object;

              if (is$1('string', keys)) {
                const keys_ = keys.split('.').filter(Boolean).map(key => castParam$1(key));
                return setObjectValue(object, keys_, value, options);
              }

              if (is$1('array', keys)) {
                const key = keys[0];
                const keyClean = cleanValue$1(String(key), {
                  filters: ['.', ','],
                  replaceWith: ''
                });

                if (keys.length === 1 && value !== undefined) {
                  if (is$1('array', object) && is$1('number', key) || is$1('object', object) && is$1('string', key)) {
                    object[is$1('string', key) ? keyClean : key] = value;
                  }
                } else {
                  // Add array or object as a value of the key, if that key doesn't exist atm
                  if (!object.hasOwnProperty(key) || options.valueOverride) object[keyClean] = is$1('number', keys[1]) ? [] : {};
                  const value_ = object[keyClean]; // If we are trying to set a deeply nested value on a
                  // simple value type, meaning if it's not an array or an object,
                  // To override existing value use valueOverride: true option

                  if (!(is$1('object', value_) || is$1('array', value_))) return object;
                  return setObjectValue(object[key], keys.slice(1), value, options);
                }
              }

              return object;
            };

            var setObjectValue$1 = setObjectValue;

            /**
             * It returns an array with unique simple values
             * and / or array and object values.
             *
             * Referenced values are only compared based on
             * values in those reference type values based on array
             * of keys provided in the second argument in the method.
             *
             * Uniqueness of array and object values is separatelly
             * evaluated based on keys value and returned in the result.
             */

            const unique = function (object) {
              for (var _len = arguments.length, args = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
                args[_key - 1] = arguments[_key];
              }

              const cache = {
                simple: [],
                array: [],
                object: []
              };
              const output = [];

              if (is$1('array', object)) {
                object.forEach(item => {
                  const isNotArrayObject = is$1('not-array-object', item);
                  const isArray = is$1('array', item);
                  const value = isNotArrayObject || !args.length ? item : getObjectValue$1(item, ...args);
                  const cacheArray = cache[isNotArrayObject ? 'simple' : isArray ? 'array' : 'object'];
                  const exists = cacheArray.find(cacheItem => value === cacheItem);

                  if (!exists && value !== undefined) {
                    output.push(item);
                    cache[isNotArrayObject ? 'simple' : isArray ? 'array' : 'object'].push(value);
                  }
                });
              }

              return output;
            };

            var unique$1 = unique;

            const getEnvironment = () => {
              if (typeof self !== 'undefined') return self;
              if (typeof window !== 'undefined') return window;
              if (typeof global$1 !== 'undefined') return global$1;
            };

            var getEnvironment$1 = getEnvironment;

            const optionsDefault$f = {};

            const Try = function (value) {
              let options_ = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : optionsDefault$f;
              const options = merge$1(options_, optionsDefault$f);

              try {
                return is$1('function', value) ? value() : undefined;
              } catch (error) {
                if (options.log) {
                  var _env$AMAUI;

                  console.error('Try: ', error);
                  const env = getEnvironment$1();

                  if (((_env$AMAUI = env.AMAUI) === null || _env$AMAUI === void 0 ? void 0 : _env$AMAUI.env) === 'test') {
                    var _env$AMAUI2, _env$AMAUI2$test, _env$AMAUI2$test$Try;

                    if (!((_env$AMAUI2 = env.AMAUI) !== null && _env$AMAUI2 !== void 0 && (_env$AMAUI2$test = _env$AMAUI2.test) !== null && _env$AMAUI2$test !== void 0 && (_env$AMAUI2$test$Try = _env$AMAUI2$test.Try) !== null && _env$AMAUI2$test$Try !== void 0 && _env$AMAUI2$test$Try.logs)) setObjectValue$1(env, 'AMAUI.test.Try.logs', []);
                    env.AMAUI.test.Try.logs.push(error);
                  }
                }
              }
            };

            var Try$1 = Try;

            function element(value) {
              const object = {};
              object.value = value;
              if (is$1('string', value)) object.value = window.document.querySelector(value);
              if (!is$1('element', object.value)) delete object.value;

              const matches = function () {
                let value_ = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : object.value;
                const method = is$1('element', value_) && (value_.matches || value_['webkitMatchesSelector'] || value_['mozMatchesSelector'] || value_['oMatchesSelector'] || value_['msMatchesSelector']);
                if (!method) return () => false;
                return method.bind(value_);
              }; // Parent


              object.parent = function () {
                if (this.value && isEnvironment('browser') && this.value.parentNode) return this.value.parentNode;
              }; // Parents


              object.parents = function (selectors) {
                let arrayMethod = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'some';
                const parents = [];
                let parent = this.value;

                while (parent && parent !== document) {
                  parent = element(parent).parent();
                  if (parent && (!(selectors !== null && selectors !== void 0 && selectors.length) || selectors[arrayMethod] && selectors[arrayMethod](item => Try$1(() => matches(parent)(item))))) parents.push(parent);
                }

                return parents;
              }; // Nearest


              object.nearest = function (selectors) {
                let arrayMethod = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'some';
                // Value matches
                // return itself in that use case
                if (!(selectors !== null && selectors !== void 0 && selectors.length) || selectors[arrayMethod] && selectors[arrayMethod](item => Try$1(() => matches(this.value)(item)))) return this.value;
                let parent = this.value;

                while (parent && parent !== document) {
                  parent = element(parent).parent();
                  if (parent && (!(selectors !== null && selectors !== void 0 && selectors.length) || selectors[arrayMethod] && selectors[arrayMethod](item => Try$1(() => matches(parent)(item))))) return parent;
                }
              }; // Furthest


              object.furthest = function (selectors) {
                let arrayMethod = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'some';
                const parents = this.parents(selectors, arrayMethod);
                return parents[parents.length - 1];
              }; // hasParent


              object.hasParent = function (selectors) {
                let grandparents = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;
                let arrayMethod = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 'some';
                let parent = this.value;
                if (!grandparents) return !(selectors !== null && selectors !== void 0 && selectors.length) || selectors[arrayMethod] && selectors[arrayMethod](item => Try$1(() => matches(this.parent())(item)));

                while (parent && parent !== document) {
                  parent = element(parent).parent();
                  if (parent && (!(selectors !== null && selectors !== void 0 && selectors.length) || selectors[arrayMethod] && selectors[arrayMethod](item => Try$1(() => matches(parent)(item))))) return true;
                }

                return false;
              }; // hasParents
              // If unique is true, sort selectors argument by most specifc first
              // and lowest specificty last for the most proper result


              object.hasParents = function (selectors) {
                let unique = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;
                let arrayMethod = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 'some';
                if (!(selectors !== null && selectors !== void 0 && selectors.length)) return !!this.parent();
                const parents = this.parents();
                return !!(this.value && selectors !== null && selectors !== void 0 && selectors.length && selectors[arrayMethod] && selectors[arrayMethod](selector => {
                  const index = parents.findIndex(item => is$1('string', selector) ? Try$1(() => matches(item)(selector)) : item === selector);

                  if (index > -1) {
                    if (unique) parents.splice(index, 1);
                    return true;
                  }

                  return false;
                }));
              };

              return object;
            }

            const clamp = function (value) {
              let min = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : Number.MIN_SAFE_INTEGER;
              let max = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : Number.MAX_SAFE_INTEGER;

              if (is$1('number', value)) {
                if (value < min) return min;
                if (value > max) return max;
                return value;
              }

              return value;
            };

            var clamp$1 = clamp;

            const optionsDefault$e = {
              response: 'array'
            }; // m - array
            // m! / a! * b! * c! ...

            function permutationWithRepetition(value_) {
              let options_ = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
              const options = merge$1(options_, optionsDefault$e, {
                copy: true
              });

              if (is$1('array', value_)) {
                const value = unique$1(value_);
                const length = value.length;
                const items = is$1('number', options.items) ? options.items : length;
                if (items < 1) return [value];
                if (items === 1) return value.map(item_ => [item_]);
                const item = new Array(items).fill(0);
                let index = items - 2;
                const response = []; // Starts with all 0 indexes for all items
                // it loops for the last position and makes each item for each index
                // then, it moves to the left and increments the index by 1
                // once item on left index increment moves over amount of items
                // it resets to 0 index, and then it moves left and increments that item
                // using same methodology, etc. until item at index 0 has index value === items.length

                if (options.response === 'array') {
                  while (index >= 0) {
                    // Reset
                    index = items - 2;

                    for (let i = 0; i < length; i++) {
                      item[items - 1] = i;
                      response.push(item.map(index_ => value[index_]));
                    } // Move to the left of the values


                    while (true) {
                      if (item[index] === length - 1) {
                        item[index] = 0;
                        index--;
                        if (index < 0) break;
                      } else {
                        item[index]++;
                        break;
                      }
                    }
                  }

                  return response;
                }

                if (options.response === 'yield') return function* () {
                  while (index >= 0) {
                    // Reset
                    index = items - 2;

                    for (let i = 0; i < length; i++) {
                      item[items - 1] = i;
                      const item_ = item.map(index_ => value[index_]);
                      yield item_;
                      response.push(item_);
                    } // Move to the left of the values


                    while (true) {
                      if (item[index] === length - 1) {
                        item[index] = 0;
                        index--;
                        if (index < 0) break;
                      } else {
                        item[index]++;
                        break;
                      }
                    }
                  }

                  return response;
                };
              }
            }

            // m ** n

            function variationWithRepetition(value_) {
              let items = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
              let options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
              return permutationWithRepetition(value_, { ...options,
                items
              });
            }

            const rgbToRgba = function (value) {
              let opacity = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : undefined;
              let array = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

              if (isValid('color-rgb', value)) {
                let values = value.replace(/rgb|a|\(|\s|\)/g, '').split(',').map((value_, index) => {
                  if (index < 3) return Math.round(castParam$1(value_));
                  return castParam$1(value_);
                });
                const a = opacity !== undefined ? +(opacity > 1 ? (opacity / 100).toFixed(2) : clamp$1(opacity, 0, 1)) : values[3];
                values = [...values.slice(0, 3).map(item => Math.round(castParam$1(item))), a && +a];
                const [r, g, b] = values;
                return array ? values.filter(value_ => is$1('number', value_)) : "rgb".concat(a ? 'a' : '', "(").concat(r, ", ").concat(g, ", ").concat(b).concat(a ? ", ".concat(a) : '', ")");
              }
            };

            var rgbToRgba$1 = rgbToRgba;

            const intToHex = value => {
              const hex = castParam$1(value).toString(16);
              return hex.length === 1 ? "0".concat(hex) : hex;
            };

            const rgbToHex = function (value) {
              let opacity_ = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : undefined;
              let array = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

              if (isValid('color-rgb', value)) {
                let values = value.replace(/rgb|a|\(|\s|\)/g, '').split(',').filter(Boolean); // If alpha value exists, multiply it by 255 rgb max range value

                if (values[3]) values[3] = Math.round(parseFloat(values[3]) * 255);
                const opacity = opacity_ !== undefined && (opacity_ > 1 ? +(opacity_ / 100).toFixed(2) : clamp$1(opacity_, 0, 1));
                if (opacity) values.push(Math.round(parseFloat(opacity) * 255));
                values = values.map(item => intToHex(item));
                const [r, g, b, a] = values;
                return array ? values.filter(Boolean) : "#".concat(r).concat(g).concat(b).concat(a ? a : '');
              }
            };

            var rgbToHex$1 = rgbToHex;

            const rgbToHsl = function (value) {
              let opacity = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : undefined;
              let array = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

              if (isValid('color-rgb', value)) {
                let values = value.replace(/rgb|a|\(|\s|\)/g, '').split(',').filter(Boolean);
                let [r, g, b, a] = values;
                r /= 255;
                g /= 255;
                b /= 255; // find greatest and smallest channel values

                const cmin = Math.min(r, g, b);
                const cmax = Math.max(r, g, b);
                const delta = cmax - cmin;
                let h = 0;
                let s = 0;
                let l = 0;
                if (delta === 0) h = 0;else if (cmax === r) h = (g - b) / delta % 6;else if (cmax === g) h = (b - r) / delta + 2;else h = (r - g) / delta + 4;
                h = Math.round(h * 60);
                if (h < 0) h += 360;
                l = (cmax + cmin) / 2;
                s = delta === 0 ? 0 : delta / (1 - Math.abs(2 * l - 1));
                s = +(s * 100).toFixed(0);
                l = +(l * 100).toFixed(0);
                a = opacity !== undefined ? +(opacity > 1 ? (opacity / 100).toFixed(2) : clamp$1(opacity, 0, 1)) : a;
                values = [...[h, s, l].map(value_ => Math.round(castParam$1(value_))), a && +a];
                return array ? values.filter(value_ => is$1('number', value_)) : "hsl".concat(a ? 'a' : '', "(").concat(h, ", ").concat(s, "%, ").concat(l, "%").concat(a ? ", ".concat(a) : '', ")");
              }
            };

            var rgbToHsl$1 = rgbToHsl;

            const hexToRgb = function (value) {
              let opacity = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : undefined;
              let array = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

              if (isValid('color-hex', value)) {
                const hex = value.slice(1);
                let r = parseInt(hex.length === 3 ? "".concat(hex[0]).concat(hex[0]) : hex.slice(0, 2), 16);
                let g = parseInt(hex.length === 3 ? "".concat(hex[1]).concat(hex[1]) : hex.slice(2, 4), 16);
                let b = parseInt(hex.length === 3 ? "".concat(hex[2]).concat(hex[2]) : hex.slice(4, 6), 16);
                let a = opacity !== undefined ? opacity > 1 ? (opacity / 100).toFixed(2) : clamp$1(opacity, 0, 1) : hex.length === 8 && (parseInt(hex.slice(6), 16) / 255).toFixed(2);
                const values = [...[r, g, b].map(item => Math.round(castParam$1(item))), a && +a];
                [r, g, b, a] = values;
                return array ? values.filter(value_ => is$1('number', value_)) : "rgb".concat(a ? 'a' : '', "(").concat(r, ", ").concat(g, ", ").concat(b).concat(a ? ", ".concat(a) : '', ")");
              }
            };

            var hexToRgb$1 = hexToRgb;

            const hslToRgb = function (value) {
              let opacity = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : undefined;
              let array = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

              if (isValid('color-hsl', value)) {
                let values = value.replace(/hsl|a|\(|%|\s|\)/g, '').split(',');
                let [h, s, l, a] = [...values.slice(0, 3).map(item => +castParam$1(item).toFixed(0)), values[3]];
                h = parseInt(h, 10);
                s = parseInt(s, 10) / 100;
                l = parseInt(l, 10) / 100;
                a = opacity !== undefined ? opacity > 1 ? (opacity / 100).toFixed(2) : clamp$1(opacity, 0, 1) : parseFloat(a);

                const k = n => (n + h / 30) % 12;

                const u = s * Math.min(l, 1 - l);

                const f = n => l - u * Math.max(-1, Math.min(k(n) - 3, Math.min(9 - k(n), 1)));

                let r = 255 * f(0);
                let g = 255 * f(8);
                let b = 255 * f(4);
                values = [...[r, g, b].map(item => Math.round(castParam$1(item))), a && +a];
                [r, g, b, a] = values;
                return array ? values.filter(value_ => is$1('number', value_)) : "rgb".concat(a ? 'a' : '', "(").concat(r, ", ").concat(g, ", ").concat(b).concat(a ? ", ".concat(a) : '', ")");
              }
            };

            var hslToRgb$1 = hslToRgb;

            const colorToRgb = function (value) {
              let opacity = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : undefined;
              let array = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;
              if (isValid('color-rgb', value)) return rgbToRgba$1(value, opacity, array);
              if (isValid('color-hex', value)) return hexToRgb$1(value, opacity, array);
              if (isValid('color-hsl', value)) return hslToRgb$1(value, opacity, array);
            };

            var colorToRgb$1 = colorToRgb;

            const lighten = (value, coefficient) => {
              const values = colorToRgb$1(value, undefined, true);

              if (is$1('array', values) && values.length >= 3) {
                values.slice(0, 3).forEach((item, index) => values[index] += (255 - item) * clamp$1(coefficient, 0, 1));
                const [r, g, b, a] = [...values.slice(0, 3).map(item => Math.round(Number(item))), values[3]];
                return "rgb".concat(a ? 'a' : '', "(").concat(r, ", ").concat(g, ", ").concat(b).concat(a ? ", ".concat(a) : '', ")");
              }
            };

            var lighten$1 = lighten;

            const darken = (value, coefficient) => {
              const values = colorToRgb$1(value, undefined, true);

              if (is$1('array', values) && values.length >= 3) {
                values.slice(0, 3).forEach((_, index) => values[index] *= 1 - clamp$1(coefficient, 0, 1));
                const [r, g, b, a] = [...values.slice(0, 3).map(item => Math.round(Number(item))), values[3]];
                return "rgb".concat(a ? 'a' : '', "(").concat(r, ", ").concat(g, ", ").concat(b).concat(a ? ", ".concat(a) : '', ")");
              }
            };

            var darken$1 = darken;

            const alpha = (value, opacity) => colorToRgb$1(value, opacity);

            var alpha$1 = alpha;

            const getLuminance = value => {
              let values = colorToRgb$1(value, undefined, true);

              if (is$1('array', values) && values.length >= 3) {
                values = values.slice(0, 3).map(item => {
                  // Normalize
                  item /= 255;
                  return item <= 0.03928 ? item / 12.92 : ((item + 0.055) / 1.055) ** 2.4;
                });
                const [r, g, b] = values;
                return Number((r * 0.2126 + g * 0.7152 + b * 0.0722).toFixed(2));
              }
            };

            var getLuminance$1 = getLuminance;

            const emphasize = function (value) {
              let coefficient = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0.14;
              const luminance = getLuminance$1(value);

              if (luminance !== undefined) {
                return getLuminance$1(value) > 0.5 ? darken$1(value, coefficient) : lighten$1(value, coefficient);
              }
            };

            var emphasize$1 = emphasize;

            const getContrastRatio = (valueA, valueB) => {
              const lumA = getLuminance$1(valueA);
              const lumB = getLuminance$1(valueB);

              if (lumA !== undefined && lumB !== undefined) {
                return +((Math.max(lumA, lumB) + 0.05) / (Math.min(lumA, lumB) + 0.05)).toFixed(2);
              }
            };

            var getContrastRatio$1 = getContrastRatio;

            const colorRange = value => {
              const min = new Array(3).fill(Number.MAX_VALUE);
              const max = new Array(3).fill(Number.MIN_VALUE);
              value.forEach(item => {
                item.forEach((value_, index) => {
                  min[index] = Math.min(min[index], value_);
                  max[index] = Math.max(max[index], value_);
                });
              });
              const ranges = min.map((item, index) => max[index] - item);
              const maxRange = Math.max(...ranges);
              if (maxRange === ranges[0]) return 0;else if (maxRange === ranges[1]) return 1;
              return 2;
            };

            const quantizeMethod = function (value) {
              let depth = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
              const MAX_DEPTH = 7;
              if (!value.length) return [];

              if (MAX_DEPTH === depth) {
                const color = value.reduce((result, item) => {
                  item.forEach((value_, index) => result[index] += value_);
                  return result;
                }, [0, 0, 0]);
                return [color.map(item => Math.round(item / value.length))];
              }

              const sortIndex = colorRange(value);
              value.sort((a, b) => a[sortIndex] - b[sortIndex]);
              const mid = value.length / 2; // Reverse so primary is a first value

              return [...quantizeMethod(value.slice(0, mid), depth + 1), ...quantizeMethod(value.slice(mid + 1), depth + 1)].reverse();
            };

            const quantize = function (value) {
              let amount = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 4;
              const depth = 7 - Math.ceil(Math.log2(amount));
              return quantizeMethod(value, depth).slice(0, amount);
            };

            var quantize$1 = quantize;

            const imageToPalette = function (value) {
              let options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {
                amount: 4,
                size: 400,
                allowCrossOrigin: false
              };
              return new Promise(resolve => {
                if (isEnvironment('browser')) {
                  const img = new Image();
                  const canvas = document.createElement('canvas');
                  const ctx = canvas.getContext('2d');
                  if (options.allowCrossOrigin) img.crossOrigin = 'Anonymous';

                  img.onload = () => {
                    const size = options.size || 140;
                    let width = img.naturalWidth || img.offsetWidth || img.width;
                    let height = img.naturalHeight || img.offsetHeight || img.height; // resize

                    if (width > size) {
                      height /= width / size;
                      width = size;
                    }

                    if (height > size) {
                      width /= height / size;
                      height = size;
                    }

                    canvas.width = width;
                    canvas.height = height;
                    ctx.drawImage(img, 0, 0, canvas.width, canvas.height); // image data

                    const imageData = ctx.getImageData(0, 0, canvas.width, canvas.height);
                    const data = imageData.data;
                    const length = data.length;
                    const values = []; // array of rgb values

                    for (let i = 0; i < length; i += 4) values.push([data[i], data[i + 1], data[i + 2]]); // quantize image values to a palette


                    const result = quantize$1(values, options.amount || 4);
                    return resolve(result.map(item => "rgb(".concat(item[0], ", ").concat(item[1], ", ").concat(item[2], ")")));
                  };

                  img.onerror = () => resolve([]); // src


                  img.src = value;
                } else resolve(undefined);
              });
            };

            var imageToPalette$1 = imageToPalette;

            function _defineProperty(obj, key, value) {
              if (key in obj) {
                Object.defineProperty(obj, key, {
                  value: value,
                  enumerable: true,
                  configurable: true,
                  writable: true
                });
              } else {
                obj[key] = value;
              }

              return obj;
            }

            const optionsDefault$d = {
              emit: {
                priorValue: true,
                copy: false,
                pre: {},
                post: {}
              }
            };

            class AmauiSubscription {
              constructor(value) {
                let options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : optionsDefault$d;

                _defineProperty(this, "methods", []);

                this.value = value;
                this.options = options;
                this.options = { ...this.options,
                  ...optionsDefault$d
                };
              }

              get length() {
                return this.methods.length;
              }

              emit() {
                var _this$options$emit;

                for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
                  args[_key] = arguments[_key];
                }

                const value = args; // Important for use cases,
                // to be available pre emit,
                // Save value as last emitted value as a previous state optionally

                if ((_this$options$emit = this.options.emit) !== null && _this$options$emit !== void 0 && _this$options$emit.priorValue) this.value = value.length === 1 ? value[0] : value; // Pre
                // Value might be of simple type so we have to assign a new value to the value

                if (is$1('function', this.options.emit.pre.method)) this.options.emit.pre.method(...value);
                this.methods.map(method => {
                  // Whether to send a copied value version or not,
                  // it might be useful since if value is of reference type,
                  // methods in the beginning might update the value,
                  // and other following methods wouldn't get the
                  // same value as it was sent to the first method.
                  if (is$1('function', method)) method(...(this.options.emit.copy ? copy$1(value) : value));
                }); // Post
                // Value might be of simple type so we have to assign a new value to the value

                if (is$1('function', this.options.emit.post.method)) this.options.emit.post.method(...value);
              }

              forEach() {
                for (var _len2 = arguments.length, args = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
                  args[_key2] = arguments[_key2];
                }

                this.methods.forEach(method => Try$1(() => method(...args)));
              }

              map(value_) {
                if (!this.methods.length) return;
                let value = value_;

                for (const method of this.methods) value = Try$1(() => method(value));

                return value;
              }

              subscribe(method) {
                if (this.methods.indexOf(method) === -1) this.methods.push(method);
              }

              unsubscribe(method) {
                if (this.methods.indexOf(method) > -1) {
                  const index = this.methods.findIndex(method_ => method_ === method);
                  if (index > -1) this.methods.splice(index, 1);
                }
              }

            }

            var AmauiSubscription$1 = AmauiSubscription;

            const cammelCaseToKebabCase = value => is('string', value) ? value.replace(/[A-Z]/g, v => "-".concat(v[0])).toLowerCase() : value;
            const kebabCasetoCammelCase = value => is('string', value) ? value.replace(/-./g, v => v[1] !== undefined ? v[1].toUpperCase() : '') : value;
            const capitalizedCammelCase = value => capitalize(kebabCasetoCammelCase(value));
            const capitalize = value => is('string', value) ? value.charAt(0).toUpperCase() + value.slice(1) : value;
            const is = (variant, value) => {
              switch (variant) {
                case 'string':
                  return typeof value === 'string';

                case 'number':
                  return typeof value === 'number' && !Number.isNaN(value);

                case 'array':
                  return Array.isArray(value);

                case 'boolean':
                  return typeof value === 'boolean';

                case 'null':
                  return value === null;

                case 'undefined':
                  return value === undefined;

                case 'object':
                  const isObject = typeof value === 'object' && !!value && value.constructor === Object;
                  return isObject;

                case 'function':
                  return !!(value && value instanceof Function);

                case 'simple':
                  return is('string', value) || is('number', value) || is('boolean', value) || is('undefined', value) || is('null', value);

                default:
                  return;
              }
            };
            const isAmauiSubscription = value => value instanceof AmauiSubscription$1 || is('function', value === null || value === void 0 ? void 0 : value.emit);
            const getRefs = value => {
              const items = [];

              if (is('string', value)) {
                const regex = /(?:\$)[^ \$\.#]+/g;
                items.push(...(value.match(regex) || []).map(item => item.replace('$', '')));
              }

              return items;
            };
            const valueResolve = (property, value, amauiStyle) => {
              const response = {
                value: [],
                options: {}
              }; // Mange all the values

              if (is('string', property) && !!property.length && value !== undefined && amauiStyle) {
                // String
                if (is('string', value)) response.value = [value]; // Number
                else if (is('number', value)) {
                  var _amauiStyle$subscript;

                  const unit = (_amauiStyle$subscript = amauiStyle.subscriptions.rule.unit.map({
                    property,
                    value
                  })) === null || _amauiStyle$subscript === void 0 ? void 0 : _amauiStyle$subscript.value;
                  response.value = [(unit === null || unit === void 0 ? void 0 : unit.value) || value];
                } // Array of simple
                else if (is('array', value) && value.every(item => is('simple', item))) {
                  response.value = [value.flatMap(item => valueResolve(property, item, amauiStyle).value).join(' ')];
                } // Array of arrays
                // Array of objects
                else if (is('array', value) && value.every(item => is('array', item) || is('object', item))) {
                  response.value = [value.flatMap(item => valueResolve(property, item, amauiStyle).value).join(', ')];
                } // Object
                else if (is('object', value)) {
                  // Object value
                  if (value.value) {
                    const fallbacks = (value.fallbacks || []).flatMap(item => valueResolve(property, item, amauiStyle).value);
                    response.value = [fallbacks, valueResolve(property, value.value, amauiStyle).value].flat().filter(Boolean);
                    if (value.rule) response.options.rule = value.rule;
                  } else {
                    var _amauiStyle$subscript2;

                    // Value plugins
                    const value_ = (_amauiStyle$subscript2 = amauiStyle.subscriptions.rule.value.map({
                      property,
                      value
                    })) === null || _amauiStyle$subscript2 === void 0 ? void 0 : _amauiStyle$subscript2.value;
                    response.value = value_ || [];
                  }
                } // Method
                // AmauiSubscription
                // For methods and AmauiSubscription leave as is
                // these are only used during add method
                else response.value = [value];
              }

              return response;
            };
            const dynamic = value => is('function', value) || isAmauiSubscription(value) || is('object', value) && Object.keys(value).some(prop => dynamic(value[prop]));
            function* makeName() {
              let length_ = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 2;
              let input_ = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'abcdefghijklmnopqrstuvwxyz';
              const input = is('array', input_) ? input_ : input_.split('');
              let length = length_;
              let value;
              let methodNameGenerator = variationWithRepetition(input, length, {
                response: 'yield'
              })();

              while (true) {
                var _value;

                value = methodNameGenerator.next();

                if ((_value = value) !== null && _value !== void 0 && _value.done) {
                  methodNameGenerator = variationWithRepetition(input, ++length, {
                    response: 'yield'
                  })();
                  value = methodNameGenerator.next();
                }

                yield value.value.join('');
              }
            }
            const pxToRem = function (value) {
              let htmlFontSize = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 16;
              return Number((value / htmlFontSize).toFixed(4));
            };
            const env$2 = getEnvironment$1();
            env$2.amaui_methods = {
              makeName: makeName()
            };
            const names = value => {
              if (is('object', value)) {
                // Update styles, className and class
                if (!value.hasOwnProperty('className')) Object.defineProperty(value, 'className', {
                  get: function () {
                    return Object.keys(value.classNames).map(item => value.classNames[item]).join(' ');
                  }
                });
                if (!value.hasOwnProperty('class')) Object.defineProperty(value, 'class', {
                  get: function () {
                    return Object.keys(value.classes).map(item => value.classes[item]).join(' ');
                  }
                });
                if (!value.hasOwnProperty('styles')) value.styles = function () {
                  const values = [];

                  for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
                    args[_key] = arguments[_key];
                  }

                  args.forEach(arg => {
                    if (value.classes[arg]) values.push(value.classes[arg]);
                  });
                  return values.join(' ');
                };
                return value;
              }

              return value;
            };
            let i = 0;
            const getID = () => "".concat(i++, "-").concat(new Date().getTime());

            const unix = () => Math.floor(new Date().getTime() / 1000);

            const optionsDefault$c = {
              value: {
                copy: false
              },
              add: {
                override: true
              }
            };

            class AmauiMeta {
              static meta = new WeakMap();
              static options_ = optionsDefault$c;

              static get options() {
                return this.options_;
              }

              static set options(value) {
                this.options_ = { ...value,
                  ...this.options
                };
              } // Class decorator


              static class() {
                for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
                  args[_key] = arguments[_key];
                }

                return object => {
                  const [key, value] = args; // Add key and value to object which is a class itself

                  this.add(key, value, object);
                };
              } // Method decorator


              static method() {
                for (var _len2 = arguments.length, args = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
                  args[_key2] = arguments[_key2];
                }

                return (object_, property) => {
                  const object = object_.constructor;
                  const [key, value] = args; // Add key and value to object property which is a class's method

                  this.add(key, value, object, property);
                };
              } // Property decorator


              static property() {
                for (var _len3 = arguments.length, args = new Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
                  args[_key3] = arguments[_key3];
                }

                return (object_, property) => {
                  const object = object_.constructor;
                  const [key, value] = args; // Add key and value to object property which is a class's property

                  this.add(key, value, object, property);
                };
              } // Parameter decorator


              static parameter() {
                for (var _len4 = arguments.length, args = new Array(_len4), _key4 = 0; _key4 < _len4; _key4++) {
                  args[_key4] = arguments[_key4];
                }

                return (object_, property, parameterIndex) => {
                  const object = object_.constructor;
                  const [value] = args; // Add key and value to object property which is a class's method

                  this.add("amaui-meta-param:".concat(parameterIndex), value, object, property);
                };
              }

              static add(key, value, object, property) {
                // A WeakMap's key can only be
                // of a reference value type
                if (!is$1('simple', object)) {
                  const value_ = this.options.value.copy ? copy$1(value) : value;
                  let mapObject = this.meta.get(object);

                  if (!mapObject) {
                    // Add a new map for the object
                    mapObject = new Map();
                    this.meta.set(object, mapObject);
                  }

                  if (property !== undefined) {
                    let mapProperty = mapObject.get(property);

                    if (!mapProperty) {
                      // Add a new map for the object's property
                      mapProperty = new Map();
                      mapObject.set(property, mapProperty);
                    }

                    if (!mapProperty.has(key) || this.options.add.override) {
                      // Set map property's key and value
                      mapProperty.set(key, {
                        value: this.options.value.copy ? copy$1(value) : value,
                        added_at: unix()
                      });
                      return value_;
                    }
                  } else {
                    if (!mapObject.has(key) || this.options.add.override) {
                      // Set map object's key and value
                      mapObject.set(key, {
                        value: this.options.value.copy ? copy$1(value) : value,
                        added_at: unix()
                      });
                      return value_;
                    }
                  }
                }
              }

              static update(key, value, object, property) {
                // A WeakMap's key can only be
                // of a reference value type
                if (!is$1('simple', object)) {
                  const value_ = this.options.value.copy ? copy$1(value) : value;
                  let mapObject = this.meta.get(object);

                  if (!mapObject) {
                    // Add a new map for the object
                    mapObject = new Map();
                    this.meta.set(object, mapObject);
                  }

                  if (property !== undefined) {
                    let mapProperty = mapObject.get(property);

                    if (!mapProperty) {
                      // Add a new map for the object's property
                      mapProperty = new Map();
                      mapObject.set(property, mapProperty);
                    }

                    if (mapProperty.has(key)) {
                      // Set map property's key and value
                      const atmValue = mapProperty.get(key);
                      atmValue.value = value_;
                      atmValue.updated_at = unix();
                      mapProperty.set(key, atmValue);
                      return value_;
                    }
                  } else {
                    if (mapObject.has(key)) {
                      // Set map object's key and value
                      const atmValue = mapObject.get(key);
                      atmValue.value = value_;
                      atmValue.updated_at = unix();
                      mapObject.set(key, atmValue);
                      return value_;
                    }
                  }
                }
              }

              static values(object, property) {
                // A WeakMap's key can only be
                // of a reference value type
                if (!is$1('simple', object)) {
                  const mapObject = this.meta.get(object);
                  if (!mapObject) return;

                  if (property !== undefined) {
                    const mapProperty = mapObject.get(property); // Return object property's values as array

                    return mapProperty && Array.from(mapProperty.values());
                  } // Return object's values as array


                  return mapObject && Array.from(mapObject.values()).map(item => item.value);
                }
              }

              static keys(object, property) {
                // A WeakMap's key can only be
                // of a reference value type
                if (!is$1('simple', object)) {
                  const mapObject = this.meta.get(object);
                  if (!mapObject) return;

                  if (property !== undefined) {
                    const mapProperty = mapObject.get(property); // Return object property's keys as array

                    return mapProperty && Array.from(mapProperty.keys());
                  } // Return object's keys as array


                  return mapObject && Array.from(mapObject.keys());
                }
              }

              static get(key, object, property) {
                // A WeakMap's key can only be
                // of a reference value type
                if (!is$1('simple', object)) {
                  const mapObject = this.meta.get(object);
                  if (!mapObject) return;

                  if (property !== undefined) {
                    const mapProperty = mapObject.get(property);
                    if (!mapProperty) return;
                    if (!mapProperty.has(key)) return; // Get map property key's and value

                    return this.options.value.copy ? copy$1(mapProperty.get(key).value) : mapProperty.get(key).value;
                  }

                  if (!mapObject.has(key)) return; // Get map object key's and value

                  return this.options.value.copy ? copy$1(mapObject.get(key).value) : mapObject.get(key).value;
                }
              }

              static has(key, object, property) {
                // A WeakMap's key can only be
                // of a reference value type
                if (!is$1('simple', object)) {
                  const mapObject = this.meta.get(object);
                  if (!mapObject) return false;

                  if (property !== undefined) {
                    const mapProperty = mapObject.get(property);
                    if (!mapProperty) return false; // Map has property key

                    return mapProperty.has(key);
                  } // Map has object key's and value


                  return mapObject.has(key);
                }
              }

              static remove(key, object, property) {
                // A WeakMap's key can only be
                // of a reference value type
                if (!is$1('simple', object)) {
                  const mapObject = this.meta.get(object);
                  if (!mapObject) return;

                  if (property !== undefined) {
                    const mapProperty = mapObject.get(property);
                    if (!mapProperty) return; // Remove map property's key and value

                    mapProperty.delete(key);
                  } else {
                    // Remove map object's key and value
                    mapObject.delete(key);
                  }
                }
              }

              static reset() {
                this.meta = new WeakMap();
                this.options = optionsDefault$c;
              }

            }

            var AmauiMeta$1 = AmauiMeta;

            class AmauiStyleRenderer {
              make() {
                let attributes = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {
                  element: {},
                  data: {}
                };
                let variant = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'style';

                // Append to the bottom of head element
                if (isEnvironment('browser')) {
                  const element = window.document.createElement(variant);
                  Object.keys((attributes === null || attributes === void 0 ? void 0 : attributes.element) || {}).forEach(attribute => element.setAttribute(attribute, attributes.element[attribute])); // Add attributes

                  Object.keys((attributes === null || attributes === void 0 ? void 0 : attributes.data) || {}).forEach(attribute => {
                    element[attribute] = attributes.data[attribute];
                    element.setAttribute(attribute.indexOf('data-') === 0 ? attribute : "data-".concat(attribute), attributes.data[attribute]);
                  });
                  return element;
                }
              }

              add(value) {
                let priority = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'lower';
                let attributes = arguments.length > 2 ? arguments[2] : undefined;

                // Append to the bottom of head element
                if (isEnvironment('browser')) {
                  const styleSheets = window.document.styleSheets;
                  if (!styleSheets.length || priority === 'upper') window.document.head.append(value);else {
                    var _attributes$data, _styleSheets$;

                    const reset = Array.from(styleSheets).find(item => item.ownerNode.method === 'reset');
                    let pure = Array.from(styleSheets).filter(item => item.ownerNode.method === 'pure');
                    pure = pure[pure.length - 1];
                    if ((attributes === null || attributes === void 0 ? void 0 : (_attributes$data = attributes.data) === null || _attributes$data === void 0 ? void 0 : _attributes$data.method) === 'reset' || priority === 'lower' && !(pure || reset)) window.document.head.insertBefore(value, ((_styleSheets$ = styleSheets[0]) === null || _styleSheets$ === void 0 ? void 0 : _styleSheets$.ownerNode) || null);else if (priority === 'lower') {
                      if (pure) window.document.head.insertBefore(value, pure.ownerNode.nextElementSibling);else if (reset) window.document.head.insertBefore(value, reset.ownerNode.nextElementSibling);
                    } else window.document.head.append(value);
                  }
                  return value;
                }
              }

              remove(value) {
                var _element;

                let element = value;
                if (value !== null && value !== void 0 && value.ownerNode) element = element.ownerNode;
                if ((_element = element) !== null && _element !== void 0 && _element.remove) element.remove();
              }

            }

            var AmauiStyleRenderer$1 = AmauiStyleRenderer;

            const optionsDefault$b = {
              rule: {
                sort: true,
                prefix: false,
                rtl: false
              },
              optimize: true
            };

            class AmauiStyle {
              // Any new property
              constructor(
              /* tslint:disable-next-line */
              element, mode) {
                let renderer = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : new AmauiStyleRenderer$1();
                let options = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : copy$1(optionsDefault$b);

                _defineProperty(this, "subscriptions", {
                  className: {
                    pre: new AmauiSubscription$1(),
                    name: new AmauiSubscription$1(),
                    post: new AmauiSubscription$1()
                  },
                  keyframes: {
                    pre: new AmauiSubscription$1(),
                    name: new AmauiSubscription$1(),
                    post: new AmauiSubscription$1()
                  },
                  rule: {
                    pre: new AmauiSubscription$1(),
                    unit: new AmauiSubscription$1(),
                    value: new AmauiSubscription$1(),
                    prefix: new AmauiSubscription$1(),
                    rtl: new AmauiSubscription$1(),
                    add: new AmauiSubscription$1(),
                    update: new AmauiSubscription$1(),
                    update_props: new AmauiSubscription$1(),
                    remove: new AmauiSubscription$1(),
                    post: new AmauiSubscription$1()
                  },
                  rules: {
                    sort: new AmauiSubscription$1()
                  },
                  sheet: {
                    add: new AmauiSubscription$1(),
                    update: new AmauiSubscription$1(),
                    update_props: new AmauiSubscription$1(),
                    remove: new AmauiSubscription$1()
                  },
                  sheet_manager: {
                    add: new AmauiSubscription$1(),
                    update: new AmauiSubscription$1(),
                    update_props: new AmauiSubscription$1(),
                    remove: new AmauiSubscription$1()
                  }
                });

                _defineProperty(this, "values", {
                  css: ''
                });

                _defineProperty(this, "refs", {});

                _defineProperty(this, "sheets", []);

                _defineProperty(this, "sheet_managers", []);

                this.element = element;
                this.mode = mode;
                this.renderer = renderer;
                this.options = options;
                this.options = merge$1(options, optionsDefault$b, {
                  copy: true
                });
                this.init();
              }

              get response() {
                this.values.css = "";
                this.sheets.forEach(sheet => {
                  const css = sheet.css;

                  if (css) {
                    this.values.css += css;
                  }
                });
                if (this.values.css) this.values.css = "\n".concat(this.values.css, "\n");
                return this.values;
              }

              get css() {
                return this.response.css;
              }

              get plugins() {
                const amauiStyle = this;
                return {
                  // Add plugins
                  set add(value_) {
                    const value = is('array', value_) ? value_ : [value_];
                    value.filter(method => is('function', method) && !AmauiMeta$1.get(method, amauiStyle, 'plugin')).forEach(method => {
                      try {
                        const response = method(amauiStyle);
                        AmauiMeta$1.add(method, response, amauiStyle, 'plugin');
                      } catch (error) {
                        console.error('AmauiStyle use: ', error);
                      }
                    });
                  },

                  // Remove plugins
                  set remove(value_) {
                    const value = is('array', value_) ? value_ : [value_];
                    value.filter(method => is('function', method)).forEach(method => {
                      try {
                        const response = AmauiMeta$1.get(method, amauiStyle, 'plugin');
                        if (is('function', response === null || response === void 0 ? void 0 : response.remove)) response.remove();
                      } catch (error) {
                        console.error('AmauiStyle remove plugin: ', error);
                      }
                    });
                  }

                };
              }

              init() {
                if (this.id === undefined) this.id = getID();

                if (isEnvironment('browser')) {
                  if (this.element) {
                    // AmauiStyle in element
                    this.element.setAttribute('data-amaui-style', 'true');
                    this.element['amaui-style'] = true;
                    this.element.amaui_style = this; // Ltr

                    const style = Try$1(() => window.getComputedStyle(this.element));
                    this.direction = (style === null || style === void 0 ? void 0 : style.direction) || Try$1(() => window.getComputedStyle(document.documentElement).direction) || 'ltr';
                    this.options.rule.rtl = this.direction === 'rtl';
                  }
                }
              }

              static get(value) {
                let index = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
                const themes = this.all(value);
                return themes[index === -1 ? themes.length - 1 : index];
              }

              static first(value) {
                return this.get(value);
              }

              static last(value) {
                return this.get(value, -1);
              }

              static nearest(value) {
                var _element$nearest;

                return (_element$nearest = element(value).nearest(this.attributes.map(item => "[".concat(item, "]")))) === null || _element$nearest === void 0 ? void 0 : _element$nearest.amaui_style;
              }

              static furthest(value) {
                var _element$furthest;

                return (_element$furthest = element(value).furthest(this.attributes.map(item => "[".concat(item, "]")))) === null || _element$furthest === void 0 ? void 0 : _element$furthest.amaui_style;
              }

              static all(value) {
                const elements = [value, ...element(value).parents(this.attributes.map(item => "[".concat(item, "]")))];
                return elements.filter(Boolean).map(item => item.amaui_style).filter(Boolean) || [];
              }

            }

            _defineProperty(AmauiStyle, "attributes", ['data-amaui-style', 'amaui-style']);

            var AmauiStyle$1 = AmauiStyle;

            class AmauiStyleRuleProperty {
              constructor(value, property) {
                let value_variant = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 'value';
                let pure = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : false;
                let owner = arguments.length > 4 ? arguments[4] : undefined;
                let parents = arguments.length > 5 && arguments[5] !== undefined ? arguments[5] : [];
                let amauiStyleRule = arguments.length > 6 ? arguments[6] : undefined;
                let amauiStyleSheet = arguments.length > 7 ? arguments[7] : undefined;
                let amauiStyle = arguments.length > 8 ? arguments[8] : undefined;
                let options = arguments.length > 9 && arguments[9] !== undefined ? arguments[9] : {};

                _defineProperty(this, "values", {
                  property: '',
                  value: '',
                  css: ''
                });

                this.value = value;
                this.property = property;
                this.value_variant = value_variant;
                this.pure = pure;
                this.owner = owner;
                this.parents = parents;
                this.amauiStyleRule = amauiStyleRule;
                this.amauiStyleSheet = amauiStyleSheet;
                this.amauiStyle = amauiStyle;
                this.options = options;
                this.init();
              }

              get parent() {
                return this.parents[this.parents.length - 1];
              }

              get response() {
                return {
                  css: this.values.css
                };
              }

              get css() {
                return this.response.css;
              }

              updateValues() {
                // Response
                this.values.css = "".concat(this.values.property, ": ").concat(this.values.value, ";"); // For undefined animation name value

                if (this.values.css.indexOf('undefined') > -1) this.values.css = '';
              }

              init(value) {
                var _this$amauiStyleSheet, _this$parent, _this$amauiStyleSheet2, _this$parent2;

                // Update values
                this.values.property = cammelCaseToKebabCase(this.property);
                this.values.value = value !== undefined ? value : this.value;
                if (this.id === undefined) this.id = getID();
                if (this.level === undefined) this.level = this.parents.length - 1; // Add to rules_owned to all parents

                this.parents.filter(parent => !(parent instanceof AmauiStyleSheet$1)).forEach(parent => parent.rules_owned.push(this)); // method or AmauiSubscription

                if (value === undefined && ['method', 'amaui_subscription'].indexOf(this.value_variant) > -1) {
                  if (this.value_variant === 'method') this.values.value = Try$1(() => this.value(this.amauiStyleSheet.props));else if (this.value_variant === 'amaui_subscription') {
                    this.values.value = this.value.value;
                    if (!this.value.subscribed) this.value.subscribed = [];

                    if (this.value.subscribed.indexOf(this) === -1) {
                      this.value.subscribe(this.update.bind(this));
                      this.value.subscribed.push(this);
                    }
                  } // Value

                  this.values.value = is('function', this.values.value) ? Try$1(() => this.values.value(this.amauiStyleSheet.props)) : this.values.value;
                } // Value


                this.values.value = valueResolve(this.values.property, this.values.value, this.amauiStyle).value[0]; // Move through plugins
                // rtl

                const useRtl = this.amauiStyle.options.rule.rtl && (this.amauiStyleSheet === undefined || this.amauiStyleSheet.options.rule.rtl !== false) && (((_this$amauiStyleSheet = this.amauiStyleSheet) === null || _this$amauiStyleSheet === void 0 ? void 0 : _this$amauiStyleSheet.amauiTheme) === undefined || this.amauiStyleSheet.amauiTheme.options.rule.rtl) && // by default is true
                ((_this$parent = this.parent) === null || _this$parent === void 0 ? void 0 : _this$parent.options.rtl) !== false;

                if (useRtl) {
                  const rtl = this.amauiStyle.subscriptions.rule.rtl.map(this.values);

                  if (rtl !== null && rtl !== void 0 && rtl.value) {
                    var _rtl$value, _rtl$value2;

                    if (rtl !== null && rtl !== void 0 && (_rtl$value = rtl.value) !== null && _rtl$value !== void 0 && _rtl$value.property) this.values.property = rtl.value.property;
                    if (rtl !== null && rtl !== void 0 && (_rtl$value2 = rtl.value) !== null && _rtl$value2 !== void 0 && _rtl$value2.value) this.values.value = rtl.value.value;
                  }
                } // prefix


                const usePrefix = this.amauiStyle.options.rule.prefix && (this.amauiStyleSheet === undefined || this.amauiStyleSheet.options.rule.prefix !== false) && (((_this$amauiStyleSheet2 = this.amauiStyleSheet) === null || _this$amauiStyleSheet2 === void 0 ? void 0 : _this$amauiStyleSheet2.amauiTheme) === undefined || this.amauiStyleSheet.amauiTheme.options.rule.prefix !== false) && // by default is true
                ((_this$parent2 = this.parent) === null || _this$parent2 === void 0 ? void 0 : _this$parent2.options.prefix) !== false;

                if (usePrefix && this.values.property.indexOf('-') !== 0 && Try$1(() => this.values.value.indexOf('-') !== 0)) {
                  var _this$amauiStyle$subs;

                  const prefixes = ((_this$amauiStyle$subs = this.amauiStyle.subscriptions.rule.prefix.map({
                    value: this.values.value,
                    property: this.values.property
                  })) === null || _this$amauiStyle$subs === void 0 ? void 0 : _this$amauiStyle$subs.value) || [];

                  if (!!prefixes.length) {
                    prefixes.forEach(item => {
                      var _this$parent3;

                      const exists = (((_this$parent3 = this.parent) === null || _this$parent3 === void 0 ? void 0 : _this$parent3.rules) || []).find(rule_ => rule_ instanceof AmauiStyleRuleProperty && rule_.values.property === item.property && rule_.values.value === item.value);

                      if (!exists && this.parent) {
                        AmauiStyleRuleProperty.make(item.value, item.property, 'value', this.pure, this.parent, this.parents, this.amauiStyleRule, this.parent.amauiStyleSheet, this.parent.amauiStyle);
                      }
                    });
                  }
                } // Add itself to owner rules


                if (this.owner) {
                  const exists = this.owner.rules.find(rule => rule.value.id === this.id);
                  if (!exists) this.owner.rules.push({
                    property: this.property,
                    value: this
                  });
                  this.level_actual = this.owner.level_actual + 1;
                } // Update values


                this.updateValues();
              } // Update only if amauiStyleSheet is variant 'dynamic'


              update(value) {
                // Init with value
                if (value !== undefined) this.init(value); // Update the rule
                // method or AmauiSubscription

                if (value === undefined && ['method', 'amaui_subscription'].indexOf(this.value_variant) > -1) {
                  if (this.value_variant === 'method') this.values.value = Try$1(() => this.value(this.amauiStyleSheet.props));else if (this.value_variant === 'amaui_subscription') this.values.value = this.value.value; // Value

                  this.values.value = is('function', this.values.value) ? Try$1(() => this.values.value(this.amauiStyleSheet.props)) : this.values.value;
                  this.values.value = valueResolve(this.values.property, this.values.value, this.amauiStyle).value[0];
                } // Update values


                this.updateValues(); // Only if rule reference exists

                if (this.owner.rule) {
                  // Only update if value is diff from previous update
                  if (this.owner.rule.style[this.values.property] !== this.values.value) {
                    Try$1(() => this.owner.rule.style[this.values.property] = this.values.value); // Update the values css string value

                    this.values.css = "".concat(this.values.property, ": ").concat(this.values.value, ";");
                  }
                }
              }

              remove() {
                this.clear();
              }

              makeSelector() {
                if (['animation', 'animation-name'].some(item => this.values.property.indexOf(item) > -1)) {
                  const refs = getRefs(this.values.value);
                  const refValues = refs.map(item => this.amauiStyleSheet.amauiStyleSheetManager.names.keyframes[item]).filter(Boolean);
                  refs.forEach((ref, i) => this.values.value = this.values.value.replace("$".concat(ref), refValues[i])); // Update values

                  this.updateValues();
                }
              }

              clear() {
                var _this$owner;

                // rule
                if ((_this$owner = this.owner) !== null && _this$owner !== void 0 && _this$owner.rule) this.owner.rule.style[this.values.property] = ''; // rules

                if (this.owner) {
                  const index = this.owner.rules.findIndex(item => item.value === this);
                  if (index > -1) this.owner.rules.splice(index, 1);
                } // rules owned


                this.parents.filter(parent => !(parent instanceof AmauiStyleSheet$1)).forEach(parent => {
                  const index = parent.rules_owned.findIndex(item => item.value === this);
                  if (index > -1) parent.rules_owned.splice(index, 1);
                });
              }

              static make(value, property) {
                let variant = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 'value';
                let pure = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : false;
                let owner = arguments.length > 4 ? arguments[4] : undefined;
                let parents = arguments.length > 5 && arguments[5] !== undefined ? arguments[5] : [this];
                let amauiStyleRule = arguments.length > 6 ? arguments[6] : undefined;
                let amauiStyleSheet = arguments.length > 7 ? arguments[7] : undefined;
                let amauiStyle = arguments.length > 8 ? arguments[8] : undefined;
                let ruleOptions = arguments.length > 9 ? arguments[9] : undefined;
                return new AmauiStyleRuleProperty(value, property, variant, pure, owner, parents, amauiStyleRule, amauiStyleSheet, amauiStyle, ruleOptions);
              }

            }

            var AmauiStyleRuleProperty$1 = AmauiStyleRuleProperty;

            function classNames(value) {
              let prefix = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '';
              let array = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;
              let classNameValues = [];

              const method = (item, prop) => {
                if (is('string', item) && item.length) classNameValues.push(item);else if (is('object', item)) Object.keys(item).forEach(prop_ => method(item[prop_], prop_));else if (is('array', item)) item.forEach(item_ => method(item_));else if (prop && !!item) classNameValues.push(prop);
              }; // Move through the value


              method(value);
              classNameValues = classNameValues.filter(Boolean).map(item_ => {
                let item = item_.trim();
                if (['.', '#'].indexOf(item[0]) > -1) item = item.slice(1);
                return "".concat(prefix || '').concat(item);
              });
              classNameValues = unique$1(classNameValues);
              return array ? classNameValues : classNameValues.join(' ');
            }

            const optionsDefault$a = {
              sort: true,
              prefix: true,
              rtl: true
            };
            const env$1 = getEnvironment$1();
            env$1.amaui_counter = {
              className: 0,
              keyframesName: 0
            };

            const makeRuleClassNameDefault = function () {
              let value = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 'a';
              return "".concat(value, "-").concat(env$1.amaui_counter.className++);
            };

            const makeRuleKeyframesNameDefault = function () {
              let value = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 'a';
              return "".concat(value, "-").concat(env$1.amaui_counter.keyframesName++);
            };

            class AmauiStyleRule {
              constructor(value, property) {
                let mode = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 'regular';
                let variant = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 'property';
                let pure = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : false;
                let index = arguments.length > 5 && arguments[5] !== undefined ? arguments[5] : 0;
                let owner = arguments.length > 6 ? arguments[6] : undefined;
                let parents = arguments.length > 7 && arguments[7] !== undefined ? arguments[7] : [];
                let amauiStyleSheet = arguments.length > 8 ? arguments[8] : undefined;
                let amauiStyle = arguments.length > 9 ? arguments[9] : undefined;
                let options = arguments.length > 10 && arguments[10] !== undefined ? arguments[10] : optionsDefault$a;

                _defineProperty(this, "value_variant", 'value');

                _defineProperty(this, "status", 'idle');

                _defineProperty(this, "isVariable", false);

                _defineProperty(this, "static", true);

                _defineProperty(this, "rules_owned", []);

                _defineProperty(this, "className_", '');

                _defineProperty(this, "selector_", '');

                _defineProperty(this, "classNames_", '');

                _defineProperty(this, "keyframesName_", '');

                _defineProperty(this, "values", {
                  value: undefined,
                  css: ''
                });

                _defineProperty(this, "rules", []);

                this.value = value;
                this.property = property;
                this.mode = mode;
                this.variant = variant;
                this.pure = pure;
                this.index = index;
                this.owner = owner;
                this.parents = parents;
                this.amauiStyleSheet = amauiStyleSheet;
                this.amauiStyle = amauiStyle;
                this.options = options;
                this.options = { ...optionsDefault$a,
                  ...this.options
                };
                this.init();
              }

              get selector() {
                return this.selector_;
              }

              set selector(value) {
                this.selector_ = value;
              }

              get className() {
                return this.className_;
              }

              set className(value) {
                const parentKeyframes = this.parent.variant === 'at-rule';

                if (!parentKeyframes) {
                  this.className_ = value; // Update classNames

                  if (this.classNames.indexOf(this.className) === -1 && this.classNames.indexOf(".".concat(this.className)) === -1) this.classNames = "".concat(this.className, " ").concat(this.classNames).trim();
                  this.amauiStyleSheet.names.classNames[this.property] = this.className; // in amauiStyleSheetManager only for static sheets

                  if (this.amauiStyleSheet.variant === 'static' && this.amauiStyleSheet.amauiStyleSheetManager) {
                    this.amauiStyleSheet.amauiStyleSheetManager.names.classNames[this.property] = this.className;
                  }
                }
              }

              get classNames() {
                return this.classNames_;
              }

              set classNames(value) {
                this.classNames_ = value;
                this.amauiStyleSheet.names.classes[this.property] = this.classNames; // in amauiStyleSheetManager only for static sheets

                if (this.amauiStyleSheet.variant === 'static' && this.amauiStyleSheet.amauiStyleSheetManager) {
                  this.amauiStyleSheet.amauiStyleSheetManager.names.classes[this.property] = this.classNames;
                }
              }

              get keyframesName() {
                return this.keyframesName_;
              }

              set keyframesName(value) {
                this.keyframesName_ = value;
                const property = this.property.indexOf('@') === 0 ? this.property.split(' ')[1] : this.property; // Update amauiStyleSheet keyframes

                this.amauiStyleSheet.names.keyframes[property] = this.keyframesName; // in amauiStyleSheetManager only for static sheets

                if (this.amauiStyleSheet.variant === 'static' && this.amauiStyleSheet.amauiStyleSheetManager) {
                  if (!this.amauiStyleSheet.amauiStyleSheetManager.names.keyframes[property]) {
                    this.amauiStyleSheet.amauiStyleSheetManager.names.keyframes[property] = this.keyframesName;
                  }
                }
              }

              get hash() {
                return this.hash_;
              }

              get parent() {
                return this.parents[this.parents.length - 1];
              }

              get response() {
                return {
                  css: this.values.css
                };
              }

              get css() {
                return this.response.css;
              }

              updateValues() {
                let hash_ = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : true;
                // Response
                const selector = this.selector || this.property;
                this.values.css = "".concat(selector, " {\n");
                let empty = true;
                this.rules.forEach((rule, index) => {
                  const css = rule.value.css;

                  if (css) {
                    empty = false;
                    this.values.css += "".concat('  '.repeat(rule.value.level_actual)).concat(css).concat('\n'.repeat(rule.value instanceof AmauiStyleRule && index !== this.rules.length - 1 ? 2 : 1));
                  }
                });
                this.values.css += "".concat('  '.repeat(this.level_actual), "}"); // Empty
                // Only if it's a variable,
                // in some use cases if there are no props in css,
                // we still have to insertRule for dynamic rules
                // so we have a rule ref for that amauiStyleRuleProperty to
                // update with a new value on update

                if (empty && (!this.className || this.amauiStyleSheet.variant === 'static')) this.values.css = ''; // Hash

                if (hash_) this.makeHash();
              }

              makeHash() {
                if (!this.hash && this.amauiStyleSheet.amauiStyle.options.optimize && this.static && this.amauiStyleSheet.variant === 'static' && this.variant === 'property' && !(this.isVariable && this.amauiStyleSheet.mode === 'atomic')) this.hash_ = hash$1(this.css.replace(/.+\{|\}.*$/g, ''));
              }

              init(value_) {
                var _this$property,
                    _this = this;

                let value = value_ !== undefined ? value_ : this.value;
                if (this.id === undefined) this.id = getID();
                if (this.level === undefined) this.level = this.parents.length - 1;
                if (this.owner) this.level_actual = this.owner.level_actual === undefined ? 0 : this.owner.level_actual + 1; // Add to rules_owned to all parents

                this.parents.filter(parent => !(parent instanceof AmauiStyleSheet$1)).forEach(parent => parent.rules_owned.push(this)); // Make string template value into an object

                const valueString = () => {
                  const rule = {};
                  value.trim().split('\n').filter(Boolean).map(item => item.trim()).forEach(item => {
                    if (item) {
                      const items = item.split(':');
                      let value__ = items[1];
                      const property = items[0];
                      value__ = value__ && value__.trim().replace(';', '');
                      if (property && value__) rule[property] = castParam$1(value__, {
                        decode: false
                      });
                    }
                  });
                  return rule;
                };

                if (is('string', value)) value = valueString();

                if (is('object', value)) {
                  if (value['@pure'] !== undefined) this.pure = !!value['@pure'];
                  if (value['@p'] !== undefined) this.pure = !!value['@p'];
                }

                if (!this.pure && this.level === 0 && this.property.indexOf('@') !== 0) this.isVariable = true; // value method or amauiSubscription

                if (is('function', value)) this.value_variant = 'method';else if (isAmauiSubscription(value)) {
                  this.value_variant = 'amaui_subscription';
                  if (!value.subscribed) value.subscribed = [];

                  if (value.subscribed.indexOf(this) === -1) {
                    value.subscribe(this.update.bind(this));
                    value.subscribed.push(this);
                  }
                } else {
                  this.values.value = value;
                }
                const atRule = ((_this$property = this.property) === null || _this$property === void 0 ? void 0 : _this$property.indexOf('@')) === 0;
                this.variant = atRule ? 'at-rule' : 'property'; // method or AmauiSubscription

                if (['method', 'amaui_subscription'].indexOf(this.value_variant) > -1) {
                  if (this.value_variant === 'method') this.values.value = Try$1(() => value(this.amauiStyleSheet.props));else if (this.value_variant === 'amaui_subscription') this.values.value = this.value.value; // Value

                  this.values.value = is('function', this.values.value) ? Try$1(() => this.values.value(this.amauiStyleSheet.props)) : this.values.value;
                }

                value = this.values.value;

                if (is('object', value)) {
                  // Additional @classNames provided
                  if (value['@classNames'] || value['@cs']) {
                    const classNames$1 = classNames(value['@classNames'] || value['@cs']);
                    if (this.classNames.indexOf(classNames$1) === -1) this.classNames = "".concat(this.classNames || '', " ").concat(classNames$1).trim();
                  } // Options


                  if (value['@options'] || value['@o']) this.options = merge$1(value['@options'] || value['@o'] || {}, this.options);
                  const props = Object.keys(value); // rules owned

                  const rules_owned = this.rules_owned; // Reset rules and rules owned

                  this.rules = [];
                  this.rules_owned = []; // Add all new rules
                  // and it adds new and existing again

                  props.forEach(prop => this.addProperty(prop, value[prop], this.rules.length, false, false)); // Remove all the previous rules

                  rules_owned.forEach(rule => rule.remove()); // Sort and making unique rules

                  this.unique; // Dynamic

                  const dynamic = function () {
                    let rule = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : _this;
                    return rule.rules.some(item => is('function', item.value.value) || isAmauiSubscription(item.value.value) || item.value instanceof AmauiStyleRule && dynamic(item.value));
                  }; // Static


                  this.static = !dynamic();
                } // Add itself to owner rules


                if (this.owner) {
                  const exists = this.owner.rules.find(rule => rule.value.id === this.id);
                  if (!exists) this.owner.rules.push({
                    property: this.property,
                    value: this
                  });
                } // Status


                this.status = 'inited';
              }

              addProperty(prop, value) {
                var _parent$property;

                let index = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : this.rules.length;
                let unique = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : true;
                let add = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : true;
                const atRule_ = prop.indexOf('@') === 0;
                const parent = this;
                const parentAtRule = this.variant === 'at-rule';
                const parentKeyFrames = parentAtRule && ((_parent$property = parent.property) === null || _parent$property === void 0 ? void 0 : _parent$property.indexOf('@keyframes')) > -1;
                const selector = prop.indexOf('&') > -1 || parentAtRule || parentKeyFrames;
                const isProperty = !(atRule_ || selector);
                const toSkip = ['@classNames', '@cs', '@options', '@o', '@pure', '@p'].indexOf(prop) > -1;
                if (toSkip) return; // if it's a css property

                if (isProperty) {
                  const property = cammelCaseToKebabCase(prop);
                  const {
                    value: ruleValues = [],
                    options
                  } = valueResolve(property, value, this.amauiStyle); // Add to rules

                  ruleValues.forEach(item => {
                    if (this.amauiStyleSheet.mode === 'regular' || !parent.isVariable) {
                      if (!!item) {
                        AmauiStyleRuleProperty$1.make(item, property, is('function', item) || isAmauiSubscription(item) ? is('function', item) ? 'method' : 'amaui_subscription' : 'value', this.pure, parent, [...parent.parents, parent], parent, parent.amauiStyleSheet, parent.amauiStyle, options.rule);
                      }
                    } else if (this.amauiStyleSheet.mode === 'atomic' && parent.isVariable) {
                      AmauiStyleRule.make({
                        [property]: item
                      }, env$1.amaui_methods.makeName.next().value, 'atomic', 'property', this.pure, this.index + 1 + index, parent.parent, [...parent.parents, parent], parent.amauiStyleSheet, parent.amauiStyle);
                    }
                  });
                } else {
                  // Pre
                  this.amauiStyle.subscriptions.rule.pre.emit();
                  let rule;
                  const parents = [...parent.parents, parent]; // if its an at-rule

                  const atTopLevel = ['@import', '@charset', '@namespace', '@color-profile', '@property', '@font-feature-values', '@counter-style', '@keyframes', '@font-face', '@page'];
                  const atNested = ['@media', '@supports']; // if parent is keyframes

                  if (parentKeyFrames) {
                    rule = AmauiStyleRule.make(value, prop, 'regular', atRule_ ? 'at-rule' : 'property', false, index, parent, parents, parent.amauiStyleSheet, parent.amauiStyle);
                  } // if it's a top level at-rule
                  else if (atRule_ && atTopLevel.some(item => prop.indexOf(item) === 0)) {
                    rule = AmauiStyleRule.make(value, prop, 'regular', atRule_ ? 'at-rule' : 'property', false, index, this.amauiStyleSheet, parents, parent.amauiStyleSheet, parent.amauiStyle);
                  } // if it's @media or @supports or
                  // it's & or a $ ref value
                  else if (atRule_ && atNested.some(item => prop.indexOf(item) === 0) || selector) {
                    let owner;

                    for (let i = parents.length - 1; i >= 0; i--) {
                      owner = parents[i]; // Move it to nearest @media or @supports or AmauiStyleSheet parent as a rule in rules value
                      // only if the parent is at-rule @media or @supports, or AmauiStyleSheet

                      if (owner.variant === 'at-rule' && atNested.some(item => owner.property.indexOf(item) === 0) || owner instanceof AmauiStyleSheet$1) break;
                    }

                    rule = AmauiStyleRule.make(value, prop, 'regular', atRule_ ? 'at-rule' : 'property', false, index, owner, parents, parent.amauiStyleSheet, parent.amauiStyle);
                  } // Post


                  this.amauiStyle.subscriptions.rule.post.emit(rule);
                } // Adding individual new prop
                // Sort and making unique rules


                if (unique) this.unique;

                if (add) {
                  // Add
                  const added = this.add(); // Update

                  if (!added) this.rules_owned.forEach(rule => rule.update());
                }
              }

              add() {
                let update = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : true;
                // Update values
                // manually adding the rule
                if (!this.css) this.updateValues(); // Make selector

                this.makeSelector(); // add for amauiStyleRule value

                this.rules_owned.filter(rule => rule instanceof AmauiStyleRule).forEach(rule => rule.add()); // Update values

                if (update) this.updateValues(); // Add rule if sheet is active

                if (this.amauiStyleSheet.status === 'active') return this.addRuleToCss();
                this.status = 'active';
              }

              updateProps() {
                if (['method', 'amaui_subscription'].indexOf(this.value_variant) > -1) this.init(); // Add

                this.add(false); // Update

                this.rules_owned.forEach(rule => rule.update()); // Update values

                this.updateValues();
                this.amauiStyle.subscriptions.rule.update_props.emit(this);
              }

              update(value) {
                // Manual update
                if (value !== undefined || ['method', 'amaui_subscription'].indexOf(this.value_variant) > -1) this.init(value); // Add

                this.add(false); // Update

                this.rules_owned.forEach(rule => rule.update()); // Update values

                this.updateValues();
                this.amauiStyle.subscriptions.rule.update.emit(this);
              }

              remove() {
                // Remove all own amauiStyleRules
                this.rules_owned.filter(rule => rule instanceof AmauiStyleRule).forEach(rule => rule.remove()); // Only if rule and amauiStyleSheet.sheet exists
                // find index of the rule in the sheet
                // remove the rule from the sheet

                const ref = this.amauiStyle.refs[this.hash]; // No ref or ref is main and ref.refs are empty

                if (!ref || ref.main.rule === this && !ref.refs.length) {
                  if (ref) delete this.amauiStyle.refs[this.hash];

                  if (this.amauiStyleSheet.sheet) {
                    const index = Array.from(this.amauiStyleSheet.sheet.cssRules).findIndex(item => item === this.rule);
                    if (index > -1) this.amauiStyleSheet.sheet.deleteRule(index);
                  }

                  this.clear();
                } else if (ref && ref.main.rule !== this) {
                  const indexRef = ref.refs.indexOf(this.amauiStyleSheet);
                  if (indexRef > -1) ref.refs.splice(indexRef, 1); // if ref is removed and there are no more refs trigger sheet remove
                  // which if that sheet has no more refs will be removed

                  if (!ref.refs.length && ref.main.sheet.status === 'remove') ref.main.sheet.remove();
                  this.clear();
                }
              }

              addRuleToCss() {
                // if !rule ref &
                // if not a ref rule
                if (!this.rule && !this.ref) {
                  const css = this.css;

                  if (css) {
                    const rule = this.owner.sheet || this.owner.rule;

                    if (rule !== null && rule !== void 0 && rule.cssRules) {
                      let index = rule.cssRules.length;
                      index = Try$1(() => rule.insertRule(css, index));

                      if (index !== undefined) {
                        const ruleCSS = rule.cssRules[index];
                        this.rule = ruleCSS;
                        this.amauiStyle.subscriptions.rule.add.emit(this);
                        return true;
                      }
                    }
                  }
                }
              }

              addRuleRef() {
                if (!this.rule) {
                  const rule = this.owner.sheet || this.owner.rule;

                  if (rule !== null && rule !== void 0 && rule.cssRules) {
                    const ref = Array.from(rule.cssRules).find(item => item.selectorText === this.selector);
                    if (ref !== undefined) this.rule = ref; // Move through rules

                    this.rules_owned.filter(rule_ => rule_ instanceof AmauiStyleRule).forEach(rule_ => rule_.addRuleRef());
                  }
                }
              }

              makeSelector() {
                if (!this.selector) {
                  // Make hash first so we can use refs
                  if (!this.hash) this.makeHash();
                  const parentAtRule = this.parent.variant === 'at-rule';
                  const isKeyframes = this.property.indexOf('@keyframes') === 0; // Variable

                  if (this.isVariable || this.mode === 'atomic' || isKeyframes) {
                    // if it's a variable
                    this.makeRuleClassName(); // if it's a keyframes rule

                    this.makeRuleKeyframesName();
                  } else {
                    // Make property the selector
                    this.selector = this.property; // level 0 property inside an at-rule

                    if (parentAtRule && this.variant === 'property') {
                      // & ref
                      let parent = this.parent;

                      while (parent.variant === 'at-rule') parent = parent.parent;

                      this.selector = this.selector.replace(/&/g, parent.selector); // properties ie. body should remain the same targeting html element
                      // we only replace $ ref values in properties
                      // $ ref

                      const refs = getRefs(this.property);
                      refs.forEach(ref => {
                        const className = this.makeClassName(ref);
                        const regex = new RegExp("\\$".concat(ref), 'g');
                        this.selector = this.property.replace(regex, ".".concat(className));
                      });
                    } // other regular selectors
                    // and & value rules
                    else {
                      // & ref
                      this.selector = this.selector.replace(/&/g, this.parent.selector); // $ ref

                      const refs = getRefs(this.selector);
                      refs.forEach(ref => {
                        const className = this.makeClassName(ref);
                        const regex = new RegExp("\\$".concat(ref), 'g');
                        this.selector = this.selector.replace(regex, ".".concat(className));
                      });
                    }
                  } // Move through the rules


                  this.rules.forEach(rule => rule.value.makeSelector()); // Update values without hash

                  this.updateValues(false);
                }
              }

              makeClassName(property, rule) {
                var _this$amauiStyleSheet, _this$amauiStyle$subs;

                const names = ((_this$amauiStyleSheet = this.amauiStyleSheet.amauiStyleSheetManager) === null || _this$amauiStyleSheet === void 0 ? void 0 : _this$amauiStyleSheet.names) || this.amauiStyleSheet.names;
                const cached = names.classNames[property];
                if (cached) return cached; // amauiStyle ref
                // ref className already exists for the same hash

                const ref = this.amauiStyle.refs[this.hash]; // Only reuse classNames for static amauiStyleSheets and for variables only not & rules

                if (rule instanceof AmauiStyleRule && (rule.isVariable || rule.mode === 'atomic') && this.hash && ref && this.amauiStyleSheet.variant === 'static') {
                  // Push amauiStyleSheet ref if it doesn't already exist in refs
                  if (ref.main.sheet !== this.amauiStyleSheet && ref.refs.indexOf(this.amauiStyleSheet) === -1) ref.refs.push(this.amauiStyleSheet); // Update rule ref

                  rule.ref = ref;
                  return ref.className;
                } // Make a className


                const className = // Make with plugin/s
                ((_this$amauiStyle$subs = this.amauiStyle.subscriptions.className.name.map({
                  property,
                  value: rule === null || rule === void 0 ? void 0 : rule.value
                })) === null || _this$amauiStyle$subs === void 0 ? void 0 : _this$amauiStyle$subs.value) || // Make with a default method
                makeRuleClassNameDefault(property); // Add to amauiStyle ref,
                // only reuse classNames for static amauiStyleSheets

                if (rule instanceof AmauiStyleRule && (rule.isVariable || rule.mode === 'atomic') && this.hash && this.amauiStyleSheet.variant === 'static') {
                  this.amauiStyle.refs[this.hash] = {
                    main: {
                      sheet: this.amauiStyleSheet,
                      rule: this
                    },
                    className,
                    refs: []
                  };
                } // if no rule, means it's a non-existent (or dynamic) variable
                // so cache the className value as this value


                if (!rule) {
                  this.amauiStyleSheet.names.classNames[property] = className;
                  this.amauiStyleSheet.names.classes[property] = className;

                  if (this.amauiStyleSheet.variant === 'static' && this.amauiStyleSheet.amauiStyleSheetManager) {
                    this.amauiStyleSheet.amauiStyleSheetManager.names.classNames[property] = className;
                    this.amauiStyleSheet.amauiStyleSheetManager.names.classes[property] = className;
                  }
                }

                return className;
              }

              makeRuleClassName() {
                let property = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : this.property;
                let rule = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : this;

                if (rule instanceof AmauiStyleRule && (rule.isVariable || rule.mode === 'atomic') && !rule.className) {
                  var _this$amauiStyleSheet2;

                  const names = ((_this$amauiStyleSheet2 = this.amauiStyleSheet.amauiStyleSheetManager) === null || _this$amauiStyleSheet2 === void 0 ? void 0 : _this$amauiStyleSheet2.names) || this.amauiStyleSheet.names;
                  const cached = names.classNames[rule.property];
                  if (cached && rule.className) return cached; // Make a className
                  // Pre

                  this.amauiStyle.subscriptions.className.pre.emit(); // Name

                  const className = this.makeClassName(property, rule); // Post

                  this.amauiStyle.subscriptions.className.post.emit(className);
                  rule.className = className;
                  rule.selector = ".".concat(className);
                  return className;
                }
              }

              makeRuleKeyframesName() {
                let property_ = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : this.property;
                let rule = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : this;

                if (rule instanceof AmauiStyleRule && !rule.keyframesName && rule.property.indexOf('@keyframes') === 0) {
                  var _this$amauiStyleSheet3, _this$amauiStyle$subs2;

                  const keyframe = property_.indexOf('@keyframes') === 0;
                  const property = keyframe ? property_.split(' ')[1] : property_;
                  const names = ((_this$amauiStyleSheet3 = this.amauiStyleSheet.amauiStyleSheetManager) === null || _this$amauiStyleSheet3 === void 0 ? void 0 : _this$amauiStyleSheet3.names) || this.amauiStyleSheet.names;
                  const cached = names.keyframes[property];
                  if (cached) return cached; // Make a keyframes name value
                  // Pre

                  this.amauiStyle.subscriptions.keyframes.pre.emit(); // Name

                  const keyframesName = // Make with plugin/s
                  ((_this$amauiStyle$subs2 = this.amauiStyle.subscriptions.keyframes.name.map({
                    property,
                    value: rule.value
                  })) === null || _this$amauiStyle$subs2 === void 0 ? void 0 : _this$amauiStyle$subs2.value) || // Make with a default method
                  makeRuleKeyframesNameDefault(property); // Post

                  this.amauiStyle.subscriptions.keyframes.post.emit(keyframesName);
                  rule.keyframesName = keyframesName; // Add to selector as well

                  rule.selector = "@keyframes ".concat(keyframesName);
                }
              }

              get rule() {
                return this.rule_;
              }

              set rule(rule) {
                var _this$rule, _this$rule$cssRules;

                // Update active status
                this.status = 'active';
                this.rule_ = rule;

                if (!!((_this$rule = this.rule) !== null && _this$rule !== void 0 && (_this$rule$cssRules = _this$rule.cssRules) !== null && _this$rule$cssRules !== void 0 && _this$rule$cssRules.length)) {
                  Array.from(this.rule.cssRules).forEach(item => {
                    let selector = item.selectorText;
                    if (item instanceof CSSMediaRule) selector = "@media ".concat(item.conditionText);else if (item instanceof CSSSupportsRule) selector = "@supports ".concat(item.conditionText);
                    const rule_ = this.rules.find(rule__ => rule__.value.selector === selector);
                    if (rule_) rule_.value.rule = item;
                  });
                }
              }

              get unique() {
                var _this$amauiStyleSheet4, _this$amauiStyleSheet5;

                // Remove duplicate rules
                const values = [];
                this.rules.forEach((rule, index) => {
                  const exists = rule instanceof AmauiStyleRuleProperty$1 && values.find(item => item instanceof AmauiStyleRuleProperty$1 && item.values.property === rule.values.property && item.values.value === rule.values.value);
                  if (exists) this.rules.splice(index, 1);else values.push(rule);
                }); // Native sort based on levels first
                // for making, updating & refs

                this.rules.sort((a, b) => {
                  if (a.value.level === b.value.level) return 0;
                  return a.value.level < b.value.level ? -1 : 1;
                });
                const atRule = this.variant === 'at-rule'; // Sort

                const useSort = !atRule && this.amauiStyle.options.rule.sort && (this.amauiStyleSheet !== undefined || this.amauiStyleSheet.options.rule.sort !== false) && (((_this$amauiStyleSheet4 = this.amauiStyleSheet) === null || _this$amauiStyleSheet4 === void 0 ? void 0 : _this$amauiStyleSheet4.amauiTheme) !== undefined || ((_this$amauiStyleSheet5 = this.amauiStyleSheet.amauiTheme) === null || _this$amauiStyleSheet5 === void 0 ? void 0 : _this$amauiStyleSheet5.options.rule.sort) !== false) && // by default is true
                this.options.sort !== false;

                if (useSort) {
                  this.amauiStyle.subscriptions.rules.sort.map(this.rules); // Post

                  this.amauiStyle.subscriptions.rules.sort.emit(this);
                }

                return this.rules;
              }

              clear() {
                this.rule = undefined;
                this.ref = undefined;
                this.status = 'idle'; // rules

                if (this.owner) {
                  const index = this.owner.rules.findIndex(item => item.value === this);
                  if (index > -1) this.owner.rules.splice(index, 1);
                } // rules owned


                this.parents.filter(parent => !(parent instanceof AmauiStyleSheet$1)).forEach(parent => {
                  const index = parent.rules_owned.findIndex(item => item.value === this);
                  if (index > -1) parent.rules_owned.splice(index, 1);
                }); // remove it's selector
                // or keyframes name from
                // sheet and sheetManager

                if (this.className) {
                  delete this.amauiStyleSheet.names.classNames[this.property];
                  delete this.amauiStyleSheet.names.classes[this.property];
                  delete this.amauiStyleSheet.amauiStyleSheetManager.names.classNames[this.property];
                  delete this.amauiStyleSheet.amauiStyleSheetManager.names.classes[this.property];
                } else if (this.keyframesName) {
                  const property = this.property.split(' ')[1];
                  delete this.amauiStyleSheet.names.keyframes[property];
                  delete this.amauiStyleSheet.amauiStyleSheetManager.names.keyframes[property];
                }

                this.amauiStyle.subscriptions.rule.remove.emit(this);
              }

              static make(value, property) {
                let mode = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 'regular';
                let variant = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 'property';
                let pure = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : false;
                let index = arguments.length > 5 && arguments[5] !== undefined ? arguments[5] : 0;
                let owner = arguments.length > 6 ? arguments[6] : undefined;
                let parents = arguments.length > 7 && arguments[7] !== undefined ? arguments[7] : [this];
                let amauiStyleSheet = arguments.length > 8 ? arguments[8] : undefined;
                let amauiStyle = arguments.length > 9 ? arguments[9] : undefined;
                return new AmauiStyleRule(value, property, mode, variant, pure, index, owner, parents, amauiStyleSheet, amauiStyle);
              }

            }

            var AmauiStyleRule$1 = AmauiStyleRule;

            const env = getEnvironment$1();
            const optionsDefault$9 = {
              style: {
                attributes: {}
              },
              rule: {
                sort: true,
                prefix: true,
                rtl: true
              }
            };

            class AmauiStyleSheet {
              constructor(value) {
                var _this = this;

                let variant = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'static';
                let mode = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 'regular';
                let pure = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : false;
                let priority = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : 'upper';
                let amauiTheme = arguments.length > 5 ? arguments[5] : undefined;
                let amauiStyleSheetManager = arguments.length > 6 ? arguments[6] : undefined;
                let amauiStyle = arguments.length > 7 ? arguments[7] : undefined;
                let props = arguments.length > 8 && arguments[8] !== undefined ? arguments[8] : {};
                let options = arguments.length > 9 && arguments[9] !== undefined ? arguments[9] : copy$1(optionsDefault$9);

                _defineProperty(this, "status", 'idle');

                _defineProperty(this, "props_", {});

                _defineProperty(this, "values", {
                  css: ''
                });

                _defineProperty(this, "rules", []);

                _defineProperty(this, "names", {
                  classNames: {},
                  classes: {},
                  keyframes: {},
                  styles: function () {
                    const value = [];

                    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
                      args[_key] = arguments[_key];
                    }

                    args.forEach(arg => {
                      if (_this.names.classes[arg]) value.push(_this.names.classes[arg]);
                    });
                    return value.join(' ');
                  }
                });

                this.value = value;
                this.variant = variant;
                this.mode = mode;
                this.pure = pure;
                this.priority = priority;
                this.amauiTheme = amauiTheme;
                this.amauiStyleSheetManager = amauiStyleSheetManager;
                this.amauiStyle = amauiStyle;
                this.options = options;
                this.options = merge$1(options, optionsDefault$9, {
                  copy: true
                });
                this.props = props;
                this.init();
              }

              get props() {
                return this.props_;
              }

              set props(props) {
                let update = false;
                if (!equalDeep$1(props, this.props)) update = true;
                this.props_ = copy$1(props); // Update if new props are set

                if (update) this.updateProps();
              }

              get response() {
                // Response
                this.values.css = "";
                this.rules.filter(rule => !rule.value.ref).forEach(rule => {
                  const css = rule.value.css;

                  if (css) {
                    this.values.css += "\n".concat(css, "\n");
                  }
                });
                return this.values;
              }

              get css() {
                return this.response.css;
              }

              get sort() {
                // Sort
                // Native sort based on levels first
                // for making, updating & refs
                this.rules.sort((a, b) => {
                  if (a.value.level === b.value.level) return 0;
                  return a.value.level < b.value.level ? -1 : 1;
                }); // and then based on pure
                // pure rules have lower priority, are on top
                // more specific rules are on bottom have higher specificity

                this.rules.sort((a, b) => {
                  if (a.value.pure && !b.value.pure) return -1;
                  if (b.value.pure && !a.value.pure) return 1;
                  return 0;
                });
                return this.rules;
              }

              init() {
                var _this$amauiStyleSheet;

                this.id = getID(); // Inherits first from amauiStyle

                this.mode = this.amauiStyle.mode || this.mode; // Reset rules

                this.rules = []; // If value is an object

                if (is('object', this.value)) {
                  // Sort all properties so at-rules are at the start (ie. @keyframes for animation makeSelector value)
                  // but @media rules go at the bottom
                  const props = Object.keys(this.value).sort((a, b) => {
                    if (a.indexOf('@keyframes') > -1) return -1;
                    if (b.indexOf('@') > -1) return 1;
                    return 0;
                  });
                  const ignore = ['@pure', '@p']; // Make an AmauiStyleRule for all lvl 0 props

                  props.filter(prop => ignore.indexOf(prop) === -1).forEach((prop, index) => this.makeRule(prop, this.value[prop], index)); // Pure

                  const pure = { ...(this.value['@p'] || {}),
                    ...(this.value['@pure'] || {})
                  };
                  Object.keys(pure).forEach((prop, index) => this.makeRule(prop, pure[prop], props.length + index, true)); // Sort

                  this.sort; // Make selectors
                  // on init so they are
                  // available on init in node for
                  // critical css extraction

                  this.rules.forEach(rule => {
                    // Update values
                    rule.value.updateValues(false); // Make selectors

                    rule.value.makeSelector();
                  });
                } // Add to amauiStyle and amauiStyleSheetManager


                if (this.amauiStyleSheetManager) (_this$amauiStyleSheet = this.amauiStyleSheetManager.sheets[this.variant]) === null || _this$amauiStyleSheet === void 0 ? void 0 : _this$amauiStyleSheet.push(this);
                if (this.amauiStyleSheetManager && this.amauiStyleSheetManager.options.amaui_style_cache) this.amauiStyle.sheets.push(this); // Update inited status

                this.status = 'inited';
              }

              addRule(value, property_) {
                let add = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : true;
                const isDynamic = dynamic(value);

                if (value !== undefined && (this.variant === 'static' && !isDynamic || this.variant === 'dynamic' && isDynamic)) {
                  let property = property_ !== undefined ? property_ : env.amaui_methods.makeName.next().value;
                  const props = is('object', this.value) && Object.keys(this.value) || [];
                  if (!!props.length) while (props.indexOf(property) > -1) property = env.amaui_methods.makeName.next().value;
                  const isPure = ['@pure', '@p'];

                  if (isPure.indexOf(property) > -1 && is('object', value)) {
                    Object.keys(value).forEach(item => this.makeRule(item, value[item]));
                  } else {
                    const rule = this.makeRule(property, value); // Add

                    if (add) rule.add();

                    if (rule.status === 'active') {
                      const response = {
                        className: rule.className,
                        classNames: rule.classNames,
                        keyframeName: rule.keyframesName
                      };
                      return response;
                    }
                  }
                }
              }

              add(props) {
                // Update props
                if (props !== undefined) this.props = props;

                if (this.status !== 'active' && !!this.rules.length && this.rules.some(rule => rule.value && !!rule.value.rules.length)) {
                  // If in browser only
                  if (isEnvironment('browser')) {
                    var _this$options$style;

                    // Make a style tag
                    const attributes = {
                      element: {
                        type: 'text/css',
                        id: 'a' + this.id
                      },
                      data: { ...(((_this$options$style = this.options.style) === null || _this$options$style === void 0 ? void 0 : _this$options$style.attributes) || {}),
                        amaui: true,
                        mode: this.mode,
                        pure: this.pure,
                        variant: this.variant
                      }
                    };
                    this.element = this.amauiStyle.renderer.make(attributes); // Add to the DOM

                    this.amauiStyle.renderer.add(this.element, this.priority, attributes); // Add

                    this.rules.filter(item => !item.value.ref).forEach(item => {
                      item.value.add();
                    }); // Make css
                    // Only if it's not a ref rule
                    // this.rules.filter(item => !item.value.ref).forEach(item => item.value.addRuleToCss());

                    this.element.innerHTML = this.response.css; // Make a sheet ref

                    this.sheet = this.element.sheet; // Move through rules

                    this.rules.forEach(rule => rule.value.addRuleRef()); // Update active status

                    this.status = 'active';
                    this.amauiStyle.subscriptions.sheet.add.emit(this);
                  } // Node only make names
                  else {
                    this.rules.filter(item => !item.value.ref).forEach(item => {
                      // Add
                      item.value.add();
                    });
                  }
                }
              }

              update(value) {
                if (is('object', value)) {
                  // Update active status
                  if (this.status === 'remove') this.status = 'active'; // Update all the items to pure

                  if (this.pure) Object.keys(value).forEach(item => {
                    if (is('object', value[item])) value[item]['@p'] = true;
                  });
                  const properties = {
                    add: [],
                    update: [],
                    remove: []
                  };
                  const items = {
                    previous: this.rules,
                    new: []
                  };
                  const pure = { ...(value['@pure'] || {}),
                    ...(value['@p'] || {})
                  }; // Props

                  Object.keys(value).filter(item => ['@pure', '@p'].indexOf(item) === -1).forEach(item => {
                    items.new.push({
                      property: item,
                      value: value[item]
                    });
                  }); // Pure

                  Object.keys(pure).forEach(item => {
                    items.new.push({
                      property: item,
                      value: pure[item]
                    });
                  }); // To update, add

                  items.new.forEach(itemNew => {
                    const previous = items.previous.find(itemPrevious => itemPrevious.value.pure === !!(itemNew.value['@pure'] || itemNew.value['@p']) && itemPrevious.property === itemNew.property); // Add or update

                    properties[previous ? 'update' : 'add'].push(itemNew);
                  }); // To remove

                  items.previous.forEach(itemPrevious => {
                    const newItem = items.new.find(itemNew => itemPrevious.value.pure === !!(itemNew.value['@pure'] || itemNew.value['@p']) && itemPrevious.property === itemNew.property); // Remove

                    if (!newItem) properties.remove.push(itemPrevious);
                  }); // Activity

                  Object.keys(properties).forEach(activity => {
                    // Activity items
                    properties[activity].forEach(item => {
                      const rule = this.rules.find(rule_ => rule_.value.pure === !!(item.value['@pure'] || item.value['@p']) && rule_.value.property === item.property);

                      switch (activity) {
                        case 'add':
                          this.addRule(item.value, item.property);
                          break;

                        case 'remove':
                          if (rule) rule.value.remove();
                          break;

                        case 'update':
                          if (rule) rule.value.update(item.value);
                          break;
                      }
                    });
                  });
                  this.amauiStyle.subscriptions.sheet.update.emit(this);
                }
              }

              remove() {
                // Remove all the rules
                const rules = this.rules.map(item => item.value);
                rules.forEach(rule => rule.remove()); // Remove the style tag, only if all the rules are removed

                if (!this.rules.length) {
                  var _this$element;

                  if (is('function', (_this$element = this.element) === null || _this$element === void 0 ? void 0 : _this$element.remove)) this.amauiStyle.renderer.remove(this.element); // Remove from amauistyle

                  let index = this.amauiStyle.sheets.findIndex(sheet => sheet.id === this.id);
                  if (index > -1) this.amauiStyle.sheets.splice(index, 1); // Remove from amauiStyleSheetManager

                  index = this.amauiStyleSheetManager.sheets[this.variant].findIndex(sheet => sheet.id === this.id);
                  if (index > -1) this.amauiStyleSheetManager.sheets[this.variant].splice(index, 1); // Update idle status

                  this.status = 'idle';
                  this.element = undefined;
                  this.sheet = undefined;
                  this.amauiStyle.subscriptions.sheet.remove.emit(this);
                } else {
                  // Update remove status
                  this.status = 'remove';
                }
              }

              updateProps() {
                this.rules.forEach(rule => rule.value.updateProps());
                this.amauiStyle.subscriptions.sheet.update_props.emit(this);
              }

              makeRule(property, value) {
                let index = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : this.rules.length;
                let pure = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : false;
                // Pre
                this.amauiStyle.subscriptions.rule.pre.emit();
                const rule = AmauiStyleRule$1.make(value, property, 'regular', 'property', this.pure || pure, index, this, [this], this, this.amauiStyle); // Post

                this.amauiStyle.subscriptions.rule.post.emit(rule);
                return rule;
              }

            }

            var AmauiStyleSheet$1 = AmauiStyleSheet;

            const optionsDefault$8 = {
              style: {
                attributes: {}
              },
              rule: {
                sort: true,
                prefix: true,
                rtl: true
              },
              amaui_style_cache: true
            };

            class AmauiStyleSheetManager {
              constructor(value) {
                let mode = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'regular';
                let pure = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;
                let priority = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 'upper';
                let amauiTheme = arguments.length > 4 ? arguments[4] : undefined;
                let amauiStyle = arguments.length > 5 ? arguments[5] : undefined;
                let options = arguments.length > 6 && arguments[6] !== undefined ? arguments[6] : optionsDefault$8;

                _defineProperty(this, "status", 'idle');

                _defineProperty(this, "values", {
                  css: ''
                });

                _defineProperty(this, "properties", {
                  static: [],
                  dynamic: []
                });

                _defineProperty(this, "sheets", {
                  static: [],
                  dynamic: []
                });

                _defineProperty(this, "names", {
                  classNames: {},
                  classes: {},
                  keyframes: {}
                });

                this.value = value;
                this.mode = mode;
                this.pure = pure;
                this.priority = priority;
                this.amauiTheme = amauiTheme;
                this.amauiStyle = amauiStyle;
                this.options = options;
                this.options = merge$1(options, optionsDefault$8, {
                  copy: true
                });
                this.init();
              }

              propertiesVariant() {
                let variant = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 'static';
                let properties = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : this.properties;
                const value = {
                  '@pure': {}
                };
                properties[variant].forEach(item => {
                  if (item.value['@pure']) value['@pure'][item.property] = item.value;else value[item.property] = item.value;
                });
                if (!Object.keys(value['@pure']).length) delete value['@pure'];
                return value;
              }

              set props(value) {
                const ids = is('array', value.ids) ? value.ids : [value.ids]; // Update all dynamic sheets from ids value

                this.sheets.dynamic.filter(sheet => ids.some(id => sheet.id === id)).forEach(sheet => {
                  // Update
                  sheet.props = value.props;
                });
                this.amauiStyle.subscriptions.sheet_manager.update_props.emit(this, value);
              }

              get ids() {
                const ids = {
                  static: [],
                  dynamic: []
                };
                Object.keys(this.sheets).forEach(variant => {
                  this.sheets[variant].forEach(sheet => ids[variant].push(sheet.id));
                });
                return ids;
              }

              get response() {
                // Response
                this.values.css = ""; // Static

                this.sheets.static.forEach(sheet => {
                  const {
                    css
                  } = sheet.response;

                  if (css) {
                    this.values.css += "\n".concat(css, "\n");
                  }
                }); // Dynamic

                this.sheets.dynamic.forEach(sheet => {
                  const {
                    css
                  } = sheet.response;

                  if (css) {
                    this.values.css += "\n".concat(css, "\n");
                  }
                });
                return this.values;
              }

              get css() {
                return this.response.css;
              }

              init() {
                this.id = getID(); // Inherits first from amauiStyle

                this.mode = this.amauiStyle.mode || this.mode; // if value is an object

                if (is('object', this.value)) {
                  // Props put into values.static and values.dynamic
                  const variants = this.variants(this.value); // Add values to the properties

                  Object.keys(variants).forEach(variant => {
                    variants[variant].forEach(item => {
                      this.properties[variant].push(item);
                    });
                  }); // Make a static sheet only if it doesn't already exist

                  if (!!this.properties.static.length && !this.sheets.static.length) {
                    new AmauiStyleSheet$1(this.propertiesVariant(), 'static', this.mode, this.pure, this.priority, this.amauiTheme, this, this.amauiStyle, {}, this.options);
                  }
                } // Update names with methods


                names(this.names); // Add to amauiStyle

                this.amauiStyle.sheet_managers.push(this); // Update inited status

                this.status = 'inited';
              }

              add(props) {
                let response = {
                  ids: {
                    static: this.ids.static,
                    dynamic: []
                  }
                };
                response = merge$1(response, this.names, {
                  copy: true
                });
                const sheets = [...this.sheets.static]; // Reviving the static status removed

                if (!!this.sheets.static.length) this.sheets.static.filter(sheet => sheet.status === 'remove').forEach(sheet => sheet.update(this.propertiesVariant())); // Static

                sheets.filter(sheet => sheet.variant === 'static').forEach(sheet => {
                  // Add
                  sheet.add(props);
                }); // if values.dynamic min 1 prop make a dynamic sheet

                if (!!this.properties.dynamic.length) {
                  const sheet = new AmauiStyleSheet$1(this.propertiesVariant('dynamic'), 'dynamic', this.mode, this.pure, this.priority, this.amauiTheme, this, this.amauiStyle, {}, this.options); // Add

                  sheet.add(props); // atm

                  sheets.push(sheet); // Add dynamic names into the response

                  response = merge$1(response, sheet.names, {
                    copy: true
                  }); // Add id to the response

                  response.ids.dynamic.push(sheet.id);
                }

                if (isEnvironment('browser')) {
                  // Status
                  this.status = 'active';
                } // Update object names value


                names(response);
                this.amauiStyle.subscriptions.sheet_manager.add.emit(this);
                return response;
              } // Make sure to also call all the update hooks


              update(value) {
                // Make all props into remove, add, update props
                if (is('object', value)) {
                  const variants = {
                    previous: {
                      static: this.properties.static,
                      dynamic: this.properties.dynamic
                    },
                    new: {
                      static: [],
                      dynamic: []
                    }
                  }; // Props put into values.static and values.dynamic

                  const variants_values = this.variants(value); // Add values to the variants new

                  Object.keys(variants_values).forEach(variant => {
                    variants_values[variant].forEach(item => {
                      variants.new[variant].push(item);
                    });
                  }); // Update
                  // Static

                  if (!!variants.new.static.length) this.sheets.static.forEach(sheet => sheet.update(this.propertiesVariant('static', variants_values))); // Dynamic

                  if (!!variants.new.dynamic.length) this.sheets.dynamic.forEach(sheet => sheet.update(this.propertiesVariant('dynamic', variants_values)));
                }

                const response = {
                  ids: this.ids,
                  ...this.names
                };
                this.amauiStyle.subscriptions.sheet_manager.update.emit(this);
                return response;
              }

              remove() {
                let ids_ = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
                const ids = is('array', ids_) ? ids_ : [ids_]; // Remove all dynamic sheets from ids value

                this.sheets.dynamic.filter(sheet => ids.some(id => sheet.id === id)).forEach(sheet => {
                  // Remove
                  sheet.remove();
                }); // And if !sheets.dynamic.length remove all static sheets as well

                if (!this.sheets.dynamic.length) {
                  this.sheets.static.forEach(sheet => {
                    // Remove
                    sheet.remove();
                  });
                } // If no more !sheets.static.length and !sheets.dynamic.length
                // update status to idle else update status to remove


                this.status = !this.sheets.static.length && !this.sheets.dynamic.length ? 'idle' : 'remove';
                this.amauiStyle.subscriptions.sheet_manager.remove.emit(ids, this);
              }

              variants(value) {
                const response = {
                  static: [],
                  dynamic: []
                };

                if (is('object', value)) {
                  const pureValues = {
                    static: {},
                    dynamic: {}
                  }; // pure props

                  Object.keys(value).filter(prop => value[prop]['@pure'] === true || value[prop]['@p'] === true).forEach(prop => {
                    pureValues[!dynamic(value[prop]) ? 'static' : 'dynamic'][prop] = value[prop];
                  }); // @pure object

                  const pure = merge$1(value['@pure'] || {}, value['@p'] || {});
                  Object.keys(pure).forEach(prop => {
                    const isStatic = !dynamic(pure[prop]);
                    pureValues[isStatic ? 'static' : 'dynamic'][prop] = { ...merge$1(pureValues[isStatic ? 'static' : 'dynamic'][prop] || {}, pure[prop]),
                      '@pure': true
                    };
                  }); // regular props

                  Object.keys(value).filter(prop => ['@pure', '@p'].indexOf(prop) === -1 && !(value[prop]['@pure'] === true || value[prop]['@p'] === true)).forEach(prop => {
                    response[!dynamic(value[prop]) ? 'static' : 'dynamic'].push({
                      property: prop,
                      value: value[prop]
                    });
                  }); // Merge pure and regular props

                  response.static = [...Object.keys(pureValues.static).map(prop => ({
                    property: prop,
                    value: pureValues.static[prop]
                  })), ...response.static];
                  response.dynamic = [...Object.keys(pureValues.dynamic).map(prop => ({
                    property: prop,
                    value: pureValues.dynamic[prop]
                  })), ...response.dynamic];
                }

                return response;
              }

            }

            var AmauiStyleSheetManager$1 = AmauiStyleSheetManager;

            const colors = {
              red: {
                '50': '#FFEBEE',
                '100': '#FFCDD2',
                '200': '#EF9A9A',
                '300': '#E57373',
                '400': '#EF5350',
                '500': '#F44336',
                '600': '#E53935',
                '700': '#D32F2F',
                '800': '#C62828',
                '900': '#B71C1C',
                'A100': '#FF8A80',
                'A200': '#FF5252',
                'A400': '#FF1744',
                'A700': '#D50000'
              },
              pink: {
                '50': '#FCE4EC',
                '100': '#F8BBD0',
                '200': '#F48FB1',
                '300': '#F06292',
                '400': '#EC407A',
                '500': '#E91E63',
                '600': '#D81B60',
                '700': '#C2185B',
                '800': '#AD1457',
                '900': '#880E4F',
                'A100': '#FF80AB',
                'A200': '#FF4081',
                'A400': '#F50057',
                'A700': '#C51162'
              },
              purple: {
                '50': '#F3E5F5',
                '100': '#E1BEE7',
                '200': '#CE93D8',
                '300': '#BA68C8',
                '400': '#AB47BC',
                '500': '#9C27B0',
                '600': '#8E24AA',
                '700': '#7B1FA2',
                '800': '#6A1B9A',
                '900': '#4A148C',
                'A100': '#EA80FC',
                'A200': '#E040FB',
                'A400': '#D500F9',
                'A700': '#AA00FF'
              },
              deeppurple: {
                '50': '#EDE7F6',
                '100': '#D1C4E9',
                '200': '#B39DDB',
                '300': '#9575CD',
                '400': '#7E57C2',
                '500': '#673AB7',
                '600': '#5E35B1',
                '700': '#512DA8',
                '800': '#4527A0',
                '900': '#311B92',
                'A100': '#B388FF',
                'A200': '#7C4DFF',
                'A400': '#651FFF',
                'A700': '#6200EA'
              },
              indigo: {
                '50': '#E8EAF6',
                '100': '#C5CAE9',
                '200': '#9FA8DA',
                '300': '#7986CB',
                '400': '#5C6BC0',
                '500': '#3F51B5',
                '600': '#3949AB',
                '700': '#303F9F',
                '800': '#283593',
                '900': '#1A237E',
                'A100': '#8C9EFF',
                'A200': '#536DFE',
                'A400': '#3D5AFE',
                'A700': '#304FFE'
              },
              blue: {
                '50': '#E3F2FD',
                '100': '#BBDEFB',
                '200': '#90CAF9',
                '300': '#64B5F6',
                '400': '#42A5F5',
                '500': '#2196F3',
                '600': '#1E88E5',
                '700': '#1976D2',
                '800': '#1565C0',
                '900': '#0D47A1',
                'A100': '#82B1FF',
                'A200': '#448AFF',
                'A400': '#2979FF',
                'A700': '#2962FF'
              },
              lightblue: {
                '50': '#E1F5FE',
                '100': '#B3E5FC',
                '200': '#81D4FA',
                '300': '#4FC3F7',
                '400': '#29B6F6',
                '500': '#03A9F4',
                '600': '#039BE5',
                '700': '#0288D1',
                '800': '#0277BD',
                '900': '#01579B',
                'A100': '#80D8FF',
                'A200': '#40C4FF',
                'A400': '#00B0FF',
                'A700': '#0091EA'
              },
              cyan: {
                '50': '#E0F7FA',
                '100': '#B2EBF2',
                '200': '#80DEEA',
                '300': '#4DD0E1',
                '400': '#26C6DA',
                '500': '#00BCD4',
                '600': '#00ACC1',
                '700': '#0097A7',
                '800': '#00838F',
                '900': '#006064',
                'A100': '#84FFFF',
                'A200': '#18FFFF',
                'A400': '#00E5FF',
                'A700': '#00B8D4'
              },
              teal: {
                '50': '#E0F2F1',
                '100': '#B2DFDB',
                '200': '#80CBC4',
                '300': '#4DB6AC',
                '400': '#26A69A',
                '500': '#009688',
                '600': '#00897B',
                '700': '#00796B',
                '800': '#00695C',
                '900': '#004D40',
                'A100': '#A7FFEB',
                'A200': '#64FFDA',
                'A400': '#1DE9B6',
                'A700': '#00BFA5'
              },
              green: {
                '50': '#E8F5E9',
                '100': '#C8E6C9',
                '200': '#A5D6A7',
                '300': '#81C784',
                '400': '#66BB6A',
                '500': '#4CAF50',
                '600': '#43A047',
                '700': '#388E3C',
                '800': '#2E7D32',
                '900': '#1B5E20',
                'A100': '#B9F6CA',
                'A200': '#69F0AE',
                'A400': '#00E676',
                'A700': '#00C853'
              },
              lightgreen: {
                '50': '#F1F8E9',
                '100': '#DCEDC8',
                '200': '#C5E1A5',
                '300': '#AED581',
                '400': '#9CCC65',
                '500': '#8BC34A',
                '600': '#7CB342',
                '700': '#689F38',
                '800': '#558B2F',
                '900': '#33691E',
                'A100': '#CCFF90',
                'A200': '#B2FF59',
                'A400': '#76FF03',
                'A700': '#64DD17'
              },
              lime: {
                '50': '#F9FBE7',
                '100': '#F0F4C3',
                '200': '#E6EE9C',
                '300': '#DCE775',
                '400': '#D4E157',
                '500': '#CDDC39',
                '600': '#C0CA33',
                '700': '#AFB42B',
                '800': '#9E9D24',
                '900': '#827717',
                'A100': '#F4FF81',
                'A200': '#EEFF41',
                'A400': '#C6FF00',
                'A700': '#AEEA00'
              },
              yellow: {
                '50': '#FFFDE7',
                '100': '#FFF9C4',
                '200': '#FFF59D',
                '300': '#FFF176',
                '400': '#FFEE58',
                '500': '#FFEB3B',
                '600': '#FDD835',
                '700': '#FBC02D',
                '800': '#F9A825',
                '900': '#F57F17',
                'A100': '#FFFF8D',
                'A200': '#FFFF00',
                'A400': '#FFEA00',
                'A700': '#FFD600'
              },
              amber: {
                '50': '#FFF8E1',
                '100': '#FFECB3',
                '200': '#FFE082',
                '300': '#FFD54F',
                '400': '#FFCA28',
                '500': '#FFC107',
                '600': '#FFB300',
                '700': '#FFA000',
                '800': '#FF8F00',
                '900': '#FF6F00',
                'A100': '#FFE57F',
                'A200': '#FFD740',
                'A400': '#FFC400',
                'A700': '#FFAB00'
              },
              orange: {
                '50': '#FFF3E0',
                '100': '#FFE0B2',
                '200': '#FFCC80',
                '300': '#FFB74D',
                '400': '#FFA726',
                '500': '#FF9800',
                '600': '#FB8C00',
                '700': '#F57C00',
                '800': '#EF6C00',
                '900': '#E65100',
                'A100': '#FFD180',
                'A200': '#FFAB40',
                'A400': '#FF9100',
                'A700': '#FF6D00'
              },
              deeporange: {
                '50': '#FBE9E7',
                '100': '#FFCCBC',
                '200': '#FFAB91',
                '300': '#FF8A65',
                '400': '#FF7043',
                '500': '#FF5722',
                '600': '#F4511E',
                '700': '#E64A19',
                '800': '#D84315',
                '900': '#BF360C',
                'A100': '#FF9E80',
                'A200': '#FF6E40',
                'A400': '#FF3D00',
                'A700': '#DD2C00'
              },
              brown: {
                '50': '#EFEBE9',
                '100': '#D7CCC8',
                '200': '#BCAAA4',
                '300': '#A1887F',
                '400': '#8D6E63',
                '500': '#795548',
                '600': '#6D4C41',
                '700': '#5D4037',
                '800': '#4E342E',
                '900': '#3E2723'
              },
              gray: {
                '50': '#FAFAFA',
                '100': '#F5F5F5',
                '200': '#EEEEEE',
                '300': '#E0E0E0',
                '400': '#BDBDBD',
                '500': '#9E9E9E',
                '600': '#757575',
                '700': '#616161',
                '800': '#424242',
                '900': '#212121'
              },
              'blue gray': {
                '50': '#ECEFF1',
                '100': '#CFD8DC',
                '200': '#B0BEC5',
                '300': '#90A4AE',
                '400': '#78909C',
                '500': '#607D8B',
                '600': '#546E7A',
                '700': '#455A64',
                '800': '#37474F',
                '900': '#263238'
              },
              'black': '#000000',
              'white': '#FFFFFF'
            };
            var colors$1 = colors;

            const FONT_FAMILY = {
              primary: ['Roboto', 'Helvetica', '"Helvetica Neue"', '-apple-system', 'BlinkMacSystemFont', '"Segoe UI"', 'Arial', '"Apple Color Emoji"', '"Segoe UI Emoji"', '"Segoe UI Symbol"', 'sans-serif'].join(', '),
              secondary: ['Roboto', 'Helvetica', '"Helvetica Neue"', '-apple-system', 'BlinkMacSystemFont', '"Segoe UI"', 'Arial', '"Apple Color Emoji"', '"Segoe UI Emoji"', '"Segoe UI Symbol"', 'sans-serif'].join(', '),
              mono: ['Roboto Mono', 'monospace'].join(', ')
            };
            /*! normalize.css v8.0.1 | MIT License | github.com/necolas/normalize.css */

            const normalize = {
              html: {
                lineHeight: 1.15,
                '-webkit-text-size-adjust': '100%'
              },
              main: {
                display: 'block'
              },
              h1: {
                fontSize: '2em',
                margin: '0.67em 0'
              },
              hr: {
                boxSizing: 'content-box',
                height: 0,
                overflow: 'visible'
              },
              pre: {
                fontFamily: 'monospace, monospace',
                fontSize: '1em'
              },
              a: {
                backgroundColor: 'transparent'
              },
              'abbr[title]': {
                borderBottom: 'none',
                textDecoration: {
                  value: 'underline',
                  fallbacks: ['underline dotted']
                }
              },
              'b, strong': {
                fontWeight: 'bolder'
              },
              'code, kbd, samp': {
                fontFamily: 'monospace, monospace',
                fontSize: '1em'
              },
              small: {
                fontSize: '80%'
              },
              'sub, sup': {
                fontSize: '75%',
                lineHeight: 0,
                position: 'relative',
                verticalAlign: 'baseline'
              },
              sub: {
                bottom: '-0.25em'
              },
              sup: {
                top: '-0.5em'
              },
              img: {
                borderStyle: 'none'
              },
              'button, input, optgroup, select, textarea': {
                fontFamily: 'inherit',
                fontSize: '100%',
                lineHeight: 1.15,
                margin: 0
              },
              'button, input': {
                overflow: 'visible'
              },
              'button, select': {
                textTransform: 'none'
              },
              'button, [type="button"], [type="reset"], [type="submit"]': {
                '-webkit-appearance': 'button'
              },
              'button::-moz-focus-inner, [type="button"]::-moz-focus-inner, [type="reset"]::-moz-focus-inner, [type="submit"]::-moz-focus-inner': {
                borderStyle: 'none',
                padding: 0
              },
              'button:-moz-focusring, [type="button"]:-moz-focusring, [type="reset"]:-moz-focusring, [type="submit"]:-moz-focusring': {
                outline: '1px dotted ButtonText'
              },
              fieldset: {
                padding: '0.35em 0.75em 0.625em'
              },
              legend: {
                boxSizing: 'border-box',
                color: 'inherit',
                display: 'table',
                maxWidth: '100%',
                padding: 0,
                whiteSpace: 'normal'
              },
              progress: {
                verticalAlign: 'baseline'
              },
              textarea: {
                overflow: 'auto'
              },
              '[type="checkbox"], [type="radio"]': {
                boxSizing: 'border-box',
                padding: 0
              },
              '[type="number"]::-webkit-inner-spin-button, [type="number"]::-webkit-outer-spin-button': {
                height: 'auto'
              },
              '[type="search"]': {
                '-webkit-appearance': 'textfield',
                outlineOffset: -2
              },
              '[type="search"]::-webkit-search-decoration': {
                '-webkit-appearance': 'none'
              },
              '::-webkit-file-upload-button': {
                '-webkit-appearance': 'button',
                font: 'inherit'
              },
              details: {
                display: 'block'
              },
              summary: {
                display: 'list-item'
              },
              template: {
                display: 'none'
              },
              '[hidden]': {
                display: 'none'
              }
            };
            const resetDefault = {
              '*': {
                margin: 0,
                padding: 0,
                border: 0,
                outline: 'none',
                fontSize: '100%',
                background: 'transparent',
                boxSizing: 'border-box',
                touchAction: 'manipulation',
                '&[contenteditable]': {
                  userSelect: 'text'
                }
              },
              body: {
                fontSize: 14,
                fontFamily: FONT_FAMILY.secondary,
                fontWeight: 'normal',
                fontStyle: 'normal',
                position: 'relative',
                overflowX: 'hidden',
                backgroundColor: '#fff'
              },
              'img, embed, object, video': {
                maxWidth: '100%',
                height: 'auto'
              },
              a: {
                textDecoration: 'none',
                cursor: 'pointer'
              },
              form: {
                width: '100%'
              },
              span: {
                wordWrap: 'break-word'
              },
              hr: {
                height: '1px',
                background: '#ddd',
                width: '100%',
                margin: '24px 0'
              },
              'pre, code, kbd, samp': {
                fontFamily: FONT_FAMILY.mono
              },
              code: {
                '& span': {
                  whiteSpace: 'pre-wrap'
                }
              },
              ':focus': {
                outline: 'none'
              }
            };

            function reset$1(value_) {
              let options_ = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
              const optionsDefault = {
                amaui_style: {
                  get: AmauiStyle$1.first.bind(AmauiStyle$1)
                },
                amaui_theme: {
                  get: AmauiTheme$1.first.bind(AmauiTheme$1)
                }
              };
              const options = merge$1(options_, optionsDefault, {
                copy: true
              }); // Amaui style

              let amauiStyle = options.amaui_style.value || is('function', options.amaui_style.get) && options.amaui_style.get(options.element);
              if (amauiStyle === undefined) amauiStyle = new AmauiStyle$1(); // Amaui theme

              const amauiTheme = options.amaui_theme.value || is('function', options.amaui_theme.get) && options.amaui_theme.get(options.element); // Make value if it's a function

              let value = is('function', value_) ? Try$1(() => value_(amauiTheme)) : value_;
              if (!is('object', value)) value = {}; // Default

              const valueDefault = merge$1(resetDefault, normalize, {
                copy: true
              }); // Add reset defaults
              // user provided values override reset default values

              if (options.override) value = { ...valueDefault,
                ...value
              };else value = merge$1(value, valueDefault, {
                copy: true
              }); // Make an instance of amauiStyleSheetManager

              const amauiStyleSheetManager = new AmauiStyleSheetManager$1(value, 'regular', true, 'lower', amauiTheme, amauiStyle, {
                style: {
                  attributes: {
                    method: 'reset'
                  }
                }
              });
              const response = {
                ids: amauiStyleSheetManager.ids,
                amaui_style_sheet_manager: amauiStyleSheetManager,
                sheets: amauiStyleSheetManager.sheets,
                add: amauiStyleSheetManager.add.bind(amauiStyleSheetManager),

                set props(value__) {
                  amauiStyleSheetManager.props = value__;
                },

                update: amauiStyleSheetManager.update.bind(amauiStyleSheetManager),
                remove: amauiStyleSheetManager.remove.bind(amauiStyleSheetManager),
                addRule: amauiStyleSheetManager.sheets.static[0] && amauiStyleSheetManager.sheets.static[0].addRule.bind(amauiStyleSheetManager.sheets.static[0])
              }; // Response

              return response;
            }

            const optionsDefault$7 = {
              rule: {
                sort: true,
                prefix: false,
                rtl: false
              }
            };
            const amauiThemeValueDefault = {
              preference: {
                background: {
                  default: 'neutral'
                },
                text: {
                  default: 'neutral'
                },
                visual_contrast: {
                  default: 'regular'
                }
              },
              palette: {
                light: true,
                accessibility: 'regular',
                visual_contrast: {
                  low: {
                    opacity: {
                      primary: .77,
                      secondary: .44,
                      tertiary: .27,
                      quaternary: .14,
                      active: .44,
                      disabled: .27,
                      drag: .14,
                      divider: .13,
                      press: .1,
                      focus: .07,
                      selected: .05,
                      hover: .03
                    },
                    contrast_threshold: 2.4
                  },
                  regular: {
                    opacity: {
                      primary: .87,
                      secondary: .54,
                      tertiary: .37,
                      quaternary: .24,
                      active: .54,
                      disabled: .37,
                      drag: .16,
                      divider: .14,
                      press: .12,
                      focus: .11,
                      selected: .07,
                      hover: .04
                    },
                    contrast_threshold: 3
                  },
                  high: {
                    opacity: {
                      primary: 1,
                      secondary: .74,
                      tertiary: .57,
                      quaternary: .44,
                      active: .74,
                      disabled: .57,
                      drag: .27,
                      divider: .24,
                      press: .22,
                      focus: .21,
                      selected: .17,
                      hover: .14
                    },
                    contrast_threshold: 4
                  }
                },
                color: {
                  primary: {
                    light: colors$1.yellow[300],
                    main: colors$1.yellow[500],
                    dark: colors$1.yellow[700]
                  },
                  secondary: {
                    light: colors$1.lightgreen[300],
                    main: colors$1.lightgreen[500],
                    dark: colors$1.lightgreen[700]
                  },
                  tertiary: {
                    light: colors$1.amber[300],
                    main: colors$1.amber[500],
                    dark: colors$1.amber[700]
                  },
                  quaternary: {
                    light: colors$1.cyan[300],
                    main: colors$1.cyan[500],
                    dark: colors$1.cyan[700]
                  },
                  info: {
                    light: colors$1.lightblue[300],
                    main: colors$1.lightblue[500],
                    dark: colors$1.lightblue[700]
                  },
                  success: {
                    light: colors$1.green[300],
                    main: colors$1.green[500],
                    dark: colors$1.green[700]
                  },
                  warning: {
                    light: colors$1.orange[300],
                    main: colors$1.orange[500],
                    dark: colors$1.orange[700]
                  },
                  error: {
                    light: colors$1.deeporange[300],
                    main: colors$1.deeporange[500],
                    dark: colors$1.deeporange[700]
                  },
                  neutral: {
                    main: colors$1.black
                  }
                },
                text: {},
                background: {}
              },
              shape: {
                radius: {
                  xs: 4,
                  sm: 8,
                  md: 12,
                  lg: 16,
                  xl: 28
                }
              },
              breakpoints: {
                values: {
                  xs: 0,
                  sm: 600,
                  md: 1240,
                  lg: 1440,
                  xl: 1920
                },
                unit: 'px'
              },
              space: {
                values: {
                  xs: 4,
                  sm: 8,
                  md: 16,
                  lg: 32,
                  xl: 64
                },
                unit: 8
              },
              shadows: {
                values: {},
                opacities: [.14, .11, .17]
              },
              typography: {
                unit: 'px',
                font_size: {
                  html: 16
                },
                font_family: {
                  primary: FONT_FAMILY.primary,
                  secondary: FONT_FAMILY.secondary,
                  mono: FONT_FAMILY.mono
                },
                values: {
                  d1: {
                    fontSize: "".concat(pxToRem(57, 16), "rem"),
                    fontFamily: FONT_FAMILY.primary,
                    fontWeight: 400,
                    lineHeight: 64 / 57,
                    letterSpacing: '0em'
                  },
                  d2: {
                    fontSize: "".concat(pxToRem(45, 16), "rem"),
                    fontFamily: FONT_FAMILY.primary,
                    fontWeight: 400,
                    lineHeight: 52 / 45,
                    letterSpacing: '0em'
                  },
                  d3: {
                    fontSize: "".concat(pxToRem(35, 16), "rem"),
                    fontFamily: FONT_FAMILY.primary,
                    fontWeight: 400,
                    lineHeight: 44 / 35,
                    letterSpacing: '0em'
                  },
                  h1: {
                    fontSize: "".concat(pxToRem(32, 16), "rem"),
                    fontFamily: FONT_FAMILY.primary,
                    fontWeight: 400,
                    lineHeight: 40 / 32,
                    letterSpacing: '0em'
                  },
                  h2: {
                    fontSize: "".concat(pxToRem(27, 16), "rem"),
                    fontFamily: FONT_FAMILY.primary,
                    fontWeight: 400,
                    lineHeight: 35 / 27,
                    letterSpacing: '0em'
                  },
                  h3: {
                    fontSize: "".concat(pxToRem(24, 16), "rem"),
                    fontFamily: FONT_FAMILY.primary,
                    fontWeight: 400,
                    lineHeight: 32 / 24,
                    letterSpacing: '0em'
                  },
                  t1: {
                    fontSize: "".concat(pxToRem(21, 16), "rem"),
                    fontFamily: FONT_FAMILY.primary,
                    fontWeight: 400,
                    lineHeight: 64 / 21,
                    letterSpacing: '0em'
                  },
                  t2: {
                    fontSize: "".concat(pxToRem(16, 16), "rem"),
                    fontFamily: FONT_FAMILY.primary,
                    fontWeight: 500,
                    lineHeight: 24 / 16,
                    letterSpacing: '.15em'
                  },
                  t3: {
                    fontSize: "".concat(pxToRem(14, 16), "rem"),
                    fontFamily: FONT_FAMILY.primary,
                    fontWeight: 500,
                    lineHeight: 20 / 14,
                    letterSpacing: '.1em'
                  },
                  l1: {
                    fontSize: "".concat(pxToRem(14, 16), "rem"),
                    fontFamily: FONT_FAMILY.secondary,
                    fontWeight: 500,
                    lineHeight: 20 / 14,
                    letterSpacing: '.1em'
                  },
                  l2: {
                    fontSize: "".concat(pxToRem(12, 16), "rem"),
                    fontFamily: FONT_FAMILY.secondary,
                    fontWeight: 500,
                    lineHeight: 15 / 12,
                    letterSpacing: '.5em'
                  },
                  l3: {
                    fontSize: "".concat(pxToRem(11, 16), "rem"),
                    fontFamily: FONT_FAMILY.secondary,
                    fontWeight: 500,
                    lineHeight: 5 / 11,
                    letterSpacing: '.5em'
                  },
                  b1: {
                    fontSize: "".concat(pxToRem(16, 16), "rem"),
                    fontFamily: FONT_FAMILY.secondary,
                    fontWeight: 400,
                    lineHeight: 24 / 16,
                    letterSpacing: '.5em'
                  },
                  b2: {
                    fontSize: "".concat(pxToRem(14, 16), "rem"),
                    fontFamily: FONT_FAMILY.secondary,
                    fontWeight: 400,
                    lineHeight: 20 / 14,
                    letterSpacing: '.25em'
                  },
                  b3: {
                    fontSize: "".concat(pxToRem(11, 16), "rem"),
                    fontFamily: FONT_FAMILY.secondary,
                    fontWeight: 400,
                    lineHeight: 15 / 11,
                    letterSpacing: '.4em'
                  }
                }
              },
              transitions: {
                timing_function: {
                  standard: 'cubic-bezier(.4, 0, .2, 1)',
                  emphasized: 'cubic-bezier(.4, 0, .6, 1) ',
                  decelerated: 'cubic-bezier(0, 0, .2, 1)',
                  accelerated: 'cubic-bezier(.4, 0, 1, 1)'
                },
                duration: {
                  smallest: 100,
                  smaller: 200,
                  small: 250,
                  regular: 300,
                  enter: 250,
                  leave: 200,
                  complex: 500
                }
              },
              z_index: {
                tooltip: 1700,
                modal: 1500,
                menu_modal: 1400,
                menu: 1300,
                button_float: 1200,
                app_bar: 1100,
                main: 1000,
                text: 0
              }
            };

            class AmauiTheme {
              // Preference
              // Mode
              // Colors
              // Shape
              // Breakpoints
              // Space
              // Shadows
              // Typography
              // Transitions
              // zIndex
              // Methods
              // Any new property
              constructor() {
                var _this = this;

                let _value = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : amauiThemeValueDefault;

                let element = arguments.length > 1 ? arguments[1] : undefined;
                let options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : copy$1(optionsDefault$7);

                _defineProperty(this, "subscriptions", {
                  update: new AmauiSubscription$1()
                });

                _defineProperty(this, "direction", 'ltr');

                _defineProperty(this, "preference", copy$1(amauiThemeValueDefault.preference));

                _defineProperty(this, "mode", 'regular');

                _defineProperty(this, "palette", copy$1(amauiThemeValueDefault.palette));

                _defineProperty(this, "shape", copy$1(amauiThemeValueDefault.shape));

                _defineProperty(this, "breakpoints", copy$1(amauiThemeValueDefault.breakpoints));

                _defineProperty(this, "space", copy$1(amauiThemeValueDefault.space));

                _defineProperty(this, "shadows", copy$1(amauiThemeValueDefault.shadows));

                _defineProperty(this, "typography", copy$1(amauiThemeValueDefault.typography));

                _defineProperty(this, "transitions", copy$1(amauiThemeValueDefault.transitions));

                _defineProperty(this, "z_index", copy$1(amauiThemeValueDefault.z_index));

                _defineProperty(this, "methods", {
                  palette: {
                    image: async function (image) {
                      let options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
                      const values = await imageToPalette$1(image, {
                        amount: 4,
                        size: 140,
                        allowCrossOrigin: false,
                        ...options
                      });
                      return values || [];
                    },
                    color: {
                      value: function (variant, tone) {
                        let light = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : true;
                        const color = _this.palette.color[variant];
                        if (color) return _this.palette.light === light ? color[tone] : color[Math.abs(100 - tone)];
                      },
                      text: background => {
                        const preferenceText = this.preference.text.default || 'neutral';
                        const luminances = {
                          background: getLuminance$1(background),
                          text: getLuminance$1(this.palette.text.default.primary)
                        };
                        const valueLighten = luminances.text >= luminances.background;
                        let tone = preferenceText === 'neutral' ? this.palette.light ? 0 : 100 : 50;
                        let contrastRatio = getContrastRatio$1(background, this.palette.text.default.primary);
                        let color = this.palette.text.default.primary;

                        while (contrastRatio < this.palette.visual_contrast.default.contrast_threshold && !(tone <= 0 || tone >= 100)) {
                          // Update tone
                          valueLighten ? tone += 10 : tone -= 10;
                          tone = clamp$1(tone, 0, 100);
                          color = colorToRgb$1(this.palette.color[preferenceText][tone], this.palette.visual_contrast.default.opacity.primary);
                          contrastRatio = getContrastRatio$1(background, color);
                        }

                        return color;
                      },
                      alpha: alpha$1,
                      emphasize: emphasize$1,
                      lighten: lighten$1,
                      darken: darken$1,
                      getLuminance: getLuminance$1,
                      getContrastRatio: getContrastRatio$1,
                      colorToRgb: colorToRgb$1,
                      rgbToRgba: rgbToRgba$1,
                      rgbToHsl: rgbToHsl$1,
                      rgbToHex: rgbToHex$1,
                      hslToRgb: hslToRgb$1,
                      hexToRgb: hexToRgb$1
                    }
                  },
                  space: {
                    value: value => this.space.unit * value
                  },
                  breakpoints: {
                    up: function (value) {
                      let media = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'only screen';
                      return "@media ".concat(media, " and (min-width: ").concat(value).concat(_this.breakpoints.unit, ")");
                    },
                    down: function (value) {
                      let media = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'only screen';
                      return "@media ".concat(media, " and (max-width: ").concat(value).concat(_this.breakpoints.unit, ")");
                    },
                    between: function (value1, value2) {
                      let media = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 'only screen';
                      return "@media ".concat(media, " and (min-width: ").concat(value1).concat(_this.breakpoints.unit, ") and (max-width: ").concat(value2).concat(_this.breakpoints.unit, ")");
                    },
                    only: value => this.methods.breakpoints.between(value, value),
                    not: function (value) {
                      let media = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'only screen';
                      if (value === 'xs') return _this.methods.breakpoints.up(_this.breakpoints.values.xs);else if (value === 'xl') return _this.methods.breakpoints.down(_this.breakpoints.values.xl);
                      const keys = _this.breakpoints.keys;
                      const index = keys.findIndex(item => item === value);
                      return "@media ".concat(media, " and (max-width: ").concat(_this.breakpoints.values[value]).concat(_this.breakpoints.unit, ") and (min-width: ").concat(_this.breakpoints.values[keys[index + 1]]).concat(_this.breakpoints.unit, ")");
                    }
                  },
                  transitions: {
                    make: (properties, options) => {
                      const props = is('array', properties) ? properties : [properties];
                      const duration = this.transitions.duration[options === null || options === void 0 ? void 0 : options.duration] || is('number', options === null || options === void 0 ? void 0 : options.duration) && (options === null || options === void 0 ? void 0 : options.duration) || this.transitions.duration.regular;
                      const timing_function = this.transitions.timing_function[options === null || options === void 0 ? void 0 : options.timing_function] || is('string', options === null || options === void 0 ? void 0 : options.timing_function) && (options === null || options === void 0 ? void 0 : options.timing_function) || this.transitions.timing_function.standard;
                      const delay = (options === null || options === void 0 ? void 0 : options.delay) !== undefined ? options.delay : 0;
                      return props.map(prop => "".concat(prop, " ").concat(duration, "ms ").concat(timing_function, " ").concat(delay, "ms")).join(', ');
                    }
                  }
                });

                this.element = element;
                this.options = options;
                this.options = merge$1(options, optionsDefault$7, {
                  copy: true
                });
                this.init(_value);
              }

              init() {
                var _this$preference$visu, _this$palette$visual_5, _this$palette$visual_6, _this$palette$visual_7, _this$palette$visual_8;

                let value_ = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : this;
                const {
                  mode,
                  preference,
                  palette = {},
                  shape = {},
                  breakpoints = {},
                  space = {},
                  shadows = {},
                  typography = {},
                  transitions,
                  z_index = {},
                  id,
                  subscriptions,
                  methods,
                  element,
                  options,
                  direction,
                  ...other
                } = copy$1(value_ || {});
                const {
                  light,
                  color = {},
                  background = {},
                  text = {},
                  visual_contrast = {},
                  accessibility
                } = palette || {};
                if (this.id === undefined) this.id = getID(); // Direction

                if (isEnvironment('browser')) {
                  if (this.element) {
                    // AmauiStyle in element
                    this.element.setAttribute('data-amaui-theme', 'true');
                    this.element['amaui-theme'] = true;
                    this.element.amaui_theme = this;
                    const style = Try$1(() => window.getComputedStyle(this.element));
                    this.direction = (style === null || style === void 0 ? void 0 : style.direction) || Try$1(() => window.getComputedStyle(document.documentElement).direction) || 'ltr';
                    this.options.rule.rtl = this.direction === 'rtl';
                  }
                } // Light


                if (light !== undefined) this.palette.light = light; // Mode

                if (mode !== undefined) this.mode = mode; // Preference

                if (is('object', preference)) this.preference = merge$1(preference, this.preference); // Visual contrast

                if (is('object', visual_contrast)) this.palette.visual_contrast = merge$1(visual_contrast, this.palette.visual_contrast);
                this.palette.visual_contrast.default = this.palette.visual_contrast[((_this$preference$visu = this.preference.visual_contrast) === null || _this$preference$visu === void 0 ? void 0 : _this$preference$visu.default) || 'regular']; // Colors

                const defaults = ['primary', 'secondary', 'tertiary', 'quaternary', 'info', 'success', 'warning', 'error', 'neutral']; // Normalize

                Object.keys(color).forEach(prop => {
                  const item = color[prop];
                  if (is('string', item)) color[prop] = {
                    main: item
                  };
                }); // Defaults if not provided add 'em

                defaults.forEach(item => {
                  if (!is('object', color[item])) color[item] = this.palette.color[item];

                  if (!(is('string', color[item].light) || is('string', color[item].main) || is('string', color[item].dark))) {
                    color[item].main = this.palette.color[item].main;
                  }
                }); // Accessibility

                if (accessibility !== undefined) this.palette.accessibility = accessibility;

                if (this.palette.accessibility !== 'regular') {
                  switch (this.palette.accessibility) {
                    case 'colorblind':
                      // Primary
                      color.primary.light = colors$1.blue[300];
                      color.primary.main = colors$1.blue[500];
                      color.primary.dark = colors$1.blue[700]; // Secondary

                      color.secondary.light = colors$1.orange[300];
                      color.secondary.main = colors$1.orange[500];
                      color.secondary.dark = colors$1.orange[700]; // Tertiary

                      color.tertiary.light = colors$1.yellow[300];
                      color.tertiary.main = colors$1.yellow[500];
                      color.tertiary.dark = colors$1.yellow[700]; // Quaternary

                      color.quaternary.light = colors$1.gray[300];
                      color.quaternary.main = colors$1.gray[500];
                      color.quaternary.dark = colors$1.gray[700];
                      break;

                    case 'tritanopia':
                      // Primary
                      color.primary.light = colors$1.blue[300];
                      color.primary.main = colors$1.blue[500];
                      color.primary.dark = colors$1.blue[700]; // Secondary

                      color.secondary.light = colors$1.red[300];
                      color.secondary.main = colors$1.red[500];
                      color.secondary.dark = colors$1.red[700]; // Tertiary

                      color.tertiary.light = colors$1.cyan[300];
                      color.tertiary.main = colors$1.cyan[500];
                      color.tertiary.dark = colors$1.cyan[700]; // Quaternary

                      color.quaternary.light = colors$1.gray[300];
                      color.quaternary.main = colors$1.gray[500];
                      color.quaternary.dark = colors$1.gray[700];
                      break;
                  }
                }

                Object.keys(color).forEach(prop => {
                  const item = color[prop];
                  const value = AmauiTheme.make.color(color[prop].main || color[prop].light || color[prop].dark);

                  if (value) {
                    this.palette.color[prop] = value;
                    const variants = ['light', 'main', 'dark']; // Override the main variants with user provided values

                    variants.forEach(variant => {
                      if (is('string', item[variant]) && !!item[variant].length) this.palette.color[prop][variant] = item[variant];
                    });
                  }
                }); // text
                // add to text

                Object.keys(text).forEach(prop => {
                  this.palette.text[prop] = text[prop];
                });
                Object.keys(this.palette.color).forEach(item => {
                  var _text$item, _this$palette$visual_, _text$item2, _this$palette$visual_2, _text$item3, _this$palette$visual_3, _text$item4, _this$palette$visual_4;

                  const variant = this.palette.color[item];
                  if (!this.palette.text[item]) this.palette.text[item] = {};
                  const colorValue = this.palette.light ? variant[item === 'neutral' ? 0 : 30] : variant[item === 'neutral' ? 100 : 70];
                  this.palette.text[item].primary = ((_text$item = text[item]) === null || _text$item === void 0 ? void 0 : _text$item.primary) || colorToRgb$1(colorValue, (_this$palette$visual_ = this.palette.visual_contrast.default) === null || _this$palette$visual_ === void 0 ? void 0 : _this$palette$visual_.opacity.primary);
                  this.palette.text[item].secondary = ((_text$item2 = text[item]) === null || _text$item2 === void 0 ? void 0 : _text$item2.secondary) || colorToRgb$1(colorValue, (_this$palette$visual_2 = this.palette.visual_contrast.default) === null || _this$palette$visual_2 === void 0 ? void 0 : _this$palette$visual_2.opacity.secondary);
                  this.palette.text[item].tertiary = ((_text$item3 = text[item]) === null || _text$item3 === void 0 ? void 0 : _text$item3.tertiary) || colorToRgb$1(colorValue, (_this$palette$visual_3 = this.palette.visual_contrast.default) === null || _this$palette$visual_3 === void 0 ? void 0 : _this$palette$visual_3.opacity.tertiary);
                  this.palette.text[item].quaternary = ((_text$item4 = text[item]) === null || _text$item4 === void 0 ? void 0 : _text$item4.quaternary) || colorToRgb$1(colorValue, (_this$palette$visual_4 = this.palette.visual_contrast.default) === null || _this$palette$visual_4 === void 0 ? void 0 : _this$palette$visual_4.opacity.quaternary);
                }); // background
                // add to text

                Object.keys(background).forEach(prop => {
                  this.palette.background[prop] = background[prop];
                });
                Object.keys(this.palette.color).forEach(item => {
                  var _background$item, _background$item2, _background$item3, _background$item4;

                  const variant = this.palette.color[item];
                  if (!this.palette.background[item]) this.palette.background[item] = {};
                  this.palette.background[item].primary = ((_background$item = background[item]) === null || _background$item === void 0 ? void 0 : _background$item.primary) || variant[!this.palette.light ? 0 : 100];
                  this.palette.background[item].secondary = ((_background$item2 = background[item]) === null || _background$item2 === void 0 ? void 0 : _background$item2.secondary) || variant[!this.palette.light ? 1 : 99];
                  this.palette.background[item].tertiary = ((_background$item3 = background[item]) === null || _background$item3 === void 0 ? void 0 : _background$item3.tertiary) || variant[!this.palette.light ? 5 : 95];
                  this.palette.background[item].quaternary = ((_background$item4 = background[item]) === null || _background$item4 === void 0 ? void 0 : _background$item4.quaternary) || variant[!this.palette.light ? 10 : 90];
                }); // default

                this.palette.background.default = this.palette.background[this.preference.background.default || 'white'];
                this.palette.text.default = this.palette.text[this.preference.text.default || 'neutral']; // other

                this.palette.text.divider = colorToRgb$1(this.palette.text.default.primary, (_this$palette$visual_5 = this.palette.visual_contrast.default) === null || _this$palette$visual_5 === void 0 ? void 0 : _this$palette$visual_5.opacity.divider);
                this.palette.text.active = this.palette.text.default.secondary;
                this.palette.text.hover = colorToRgb$1(this.palette.text.default.primary, (_this$palette$visual_6 = this.palette.visual_contrast.default) === null || _this$palette$visual_6 === void 0 ? void 0 : _this$palette$visual_6.opacity.hover);
                this.palette.text.selected = colorToRgb$1(this.palette.text.default.primary, (_this$palette$visual_7 = this.palette.visual_contrast.default) === null || _this$palette$visual_7 === void 0 ? void 0 : _this$palette$visual_7.opacity.selected);
                this.palette.text.focus = colorToRgb$1(this.palette.text.default.primary, (_this$palette$visual_8 = this.palette.visual_contrast.default) === null || _this$palette$visual_8 === void 0 ? void 0 : _this$palette$visual_8.opacity.focus);
                this.palette.text.disabled = this.palette.text.default.tertiary; // Shape

                if (is('object', shape)) this.shape = merge$1(shape, this.shape); // Breakpoints

                if (is('object', breakpoints)) this.breakpoints = merge$1(breakpoints, this.breakpoints);
                const instance = this;
                if (!this.breakpoints.keys) Object.defineProperty(this.breakpoints, 'keys', {
                  get() {
                    return Object.keys(instance.breakpoints.values);
                  }

                }); // Space

                if (is('object', space)) {
                  this.space = merge$1(space, this.space);
                  this.space.unit = space.unit !== undefined ? space.unit : this.space.unit;
                  this.space.values.xs = Math.round(this.space.unit / 2);
                  this.space.values.sm = Math.round(this.space.unit);
                  this.space.values.md = Math.round(this.space.unit * 2);
                  this.space.values.lg = Math.round(this.space.unit * 4);
                  this.space.values.xl = Math.round(this.space.unit * 8);
                }

                if (!this.space.keys) Object.defineProperty(this.space, 'keys', {
                  get() {
                    return Object.keys(instance.space.values);
                  }

                }); // Shadows

                if (is('object', shadows)) this.shadows = merge$1(shadows, this.shadows);
                Object.keys(this.palette.color).forEach(item => {
                  const variant = this.palette.color[item];
                  this.shadows.values[item] = AmauiTheme.make.shadow(variant.main, this.shadows.opacities);
                }); // Typography

                if (is('object', typography)) this.typography = merge$1(typography, this.typography); // Transitions

                if (is('object', this.transitions)) this.transitions = merge$1(transitions, this.transitions); // zIndex

                if (is('object', z_index)) this.z_index = merge$1(z_index, this.z_index); // Other

                Object.keys(other).forEach(prop => this[prop] = other[prop]);
              }

              async image(value_) {
                let other = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

                // Image
                if (value_) {
                  const values = await this.methods.palette.image(value_);

                  if (!!values.length) {
                    const palette = {
                      color: {
                        primary: {},
                        secondary: {},
                        tertiary: {},
                        quaternary: {}
                      }
                    };
                    palette.color.primary.main = values[0];
                    palette.color.secondary.main = values[1];
                    palette.color.tertiary.main = values[2];
                    palette.color.quaternary.main = values[3];
                    const value = merge$1({
                      palette
                    }, other, {
                      copy: true
                    });
                    this.init(value); // Add image to the palette

                    this.palette.image = value_;
                  }
                }
              }

              update(value) {
                if (value !== undefined) {
                  this.init(copy$1(value));
                  this.subscriptions.update.emit(value, this);
                }
              }

              static get amaui_theme() {
                return new AmauiTheme();
              }

              static get make() {
                return {
                  color: value => {
                    const rgb = colorToRgb$1(value);

                    if (rgb) {
                      const values = {};
                      const [hue, saturation, light] = rgbToHsl$1(rgb, 1, true);
                      const tones = [0, 1, 5, 10, 20, 30, 40, 50, 60, 70, 80, 90, 95, 99, 100]; // Tones

                      tones.forEach(tone => values[tone] = hslToRgb$1("hsl(".concat(hue, ", ").concat(saturation, "%, ").concat(tone, "%)"))); // Main

                      values.main = rgb;
                      const mainTone = Math.round(castParam$1(light) / 10) * 10;
                      const mainIndex = tones.findIndex(item => item === mainTone); // Light
                      // max light 90 value

                      if (mainIndex >= 10) values.light = values[90]; // min light 10 value
                      else if (mainIndex === 0) values.light = values[10];else values.light = values[tones[mainIndex + 2]]; // Dark
                      // min dark 10 value

                      if (mainIndex < 5) values.dark = values[10]; // max dark 90 value
                      else if (mainIndex === 14) values.dark = values[90];else values.dark = values[tones[mainIndex - 2]];
                      return values;
                    }
                  },
                  shadow: function (value) {
                    let opacities = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
                    const shadow = {
                      '0': 'none'
                    };
                    const values = [['1', [0, 1, 1, 0, 0, 2, 1, -1, 0, 1, 3, 0]], ['2', [0, 2, 2, 0, 0, 3, 3, -2, 0, 1, 8, 0]], ['3', [0, 3, 4, 0, 0, 3, 3, -2, 0, 1, 8, 0]], ['4', [0, 4, 5, 0, 0, 1, 10, 0, 0, 2, 4, -1]], ['6', [0, 6, 10, 0, 0, 1, 18, 0, 0, 3, 5, -1]], ['8', [0, 8, 10, 1, 0, 3, 14, 2, 0, 5, 5, -3]], ['9', [0, 9, 12, 1, 0, 3, 16, 2, 0, 5, 6, -3]], ['12', [0, 12, 17, 2, 0, 5, 22, 4, 0, 7, 7, -4]], ['16', [0, 16, 24, 2, 0, 6, 30, 5, 0, 8, 10, -5]], ['24', [0, 24, 37, 3, 0, 9, 46, 8, 0, 11, 15, -7]]];
                    values.forEach(_ref => {
                      let [item, v] = _ref;
                      return shadow[item] = ["".concat(v[0], "px ").concat(v[1], "px ").concat(v[2], "px ").concat(v[3], "px ").concat(colorToRgb$1(value, opacities[0])), "".concat(v[4], "px ").concat(v[5], "px ").concat(v[6], "px ").concat(v[7], "px ").concat(colorToRgb$1(value, opacities[1])), "".concat(v[8], "px ").concat(v[9], "px ").concat(v[10], "px ").concat(v[11], "px ").concat(colorToRgb$1(value, opacities[2]))].join(', ');
                    });
                    return shadow;
                  }
                };
              }

              static get(value) {
                let index = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
                const themes = this.all(value);
                return themes[index === -1 ? themes.length - 1 : index];
              }

              static first(value) {
                return this.get(value);
              }

              static last(value) {
                return this.get(value, -1);
              }

              static nearest(value) {
                var _elementMethod$neares;

                return (_elementMethod$neares = element(value).nearest(this.attributes.map(item => "[".concat(item, "]")))) === null || _elementMethod$neares === void 0 ? void 0 : _elementMethod$neares.amaui_theme;
              }

              static furthest(value) {
                var _elementMethod$furthe;

                return (_elementMethod$furthe = element(value).furthest(this.attributes.map(item => "[".concat(item, "]")))) === null || _elementMethod$furthe === void 0 ? void 0 : _elementMethod$furthe.amaui_theme;
              }

              static all(value) {
                const elements = [value, ...element(value).parents(this.attributes.map(item => "[".concat(item, "]")))];
                return elements.filter(Boolean).map(item => item.amaui_theme).filter(Boolean) || [];
              }

            }

            _defineProperty(AmauiTheme, "attributes", ['data-amaui-theme', 'amaui-theme']);

            var AmauiTheme$1 = AmauiTheme;

            const optionsDefault$6 = {
              mode: 'regular',
              amaui_style: {
                get: AmauiStyle$1.first.bind(AmauiStyle$1)
              },
              amaui_theme: {
                get: AmauiTheme$1.first.bind(AmauiTheme$1)
              }
            };

            function style$1(value_) {
              let options_ = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
              const options = merge$1(options_, optionsDefault$6, {
                copy: true
              }); // Amaui style

              let amauiStyle = options.amaui_style.value || is('function', options.amaui_style.get) && options.amaui_style.get(options.element);
              if (amauiStyle === undefined) amauiStyle = new AmauiStyle$1(); // Amaui theme

              const amauiTheme = options.amaui_theme.value || is('function', options.amaui_theme.get) && options.amaui_theme.get(options.element); // Make value if it's a function

              const value = is('function', value_) ? Try$1(() => value_(amauiTheme)) : value_; // Make an instance of amauiStyleSheetManager

              const amauiStyleSheetManager = new AmauiStyleSheetManager$1(value, options.mode, false, 'upper', amauiTheme, amauiStyle, {
                style: {
                  attributes: {
                    method: 'style'
                  }
                }
              });
              const response = {
                ids: amauiStyleSheetManager.ids,
                amaui_style_sheet_manager: amauiStyleSheetManager,
                sheets: amauiStyleSheetManager.sheets,
                add: amauiStyleSheetManager.add.bind(amauiStyleSheetManager),

                set props(value__) {
                  amauiStyleSheetManager.props = value__;
                },

                update: amauiStyleSheetManager.update.bind(amauiStyleSheetManager),
                remove: amauiStyleSheetManager.remove.bind(amauiStyleSheetManager),
                addRule: amauiStyleSheetManager.sheets.static[0] && amauiStyleSheetManager.sheets.static[0].addRule.bind(amauiStyleSheetManager.sheets.static[0])
              }; // if add

              if (options.add) {
                const addResponse = response.add(); // return

                return options.return ? addResponse[options.return] || addResponse : addResponse;
              } // Response


              return response;
            }

            const optionsDefault$5 = {
              amaui_style: {
                get: AmauiStyle$1.first.bind(AmauiStyle$1)
              },
              amaui_theme: {
                get: AmauiTheme$1.first.bind(AmauiTheme$1)
              }
            };

            function pure$1(value_) {
              let options_ = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
              const options = merge$1(options_, optionsDefault$5, {
                copy: true
              }); // Amaui style

              let amauiStyle = options.amaui_style.value || is('function', options.amaui_style.get) && options.amaui_style.get(options.element);
              if (amauiStyle === undefined) amauiStyle = new AmauiStyle$1(); // Amaui theme

              const amauiTheme = options.amaui_theme.value || is('function', options.amaui_theme.get) && options.amaui_theme.get(options.element); // Make value if it's a function

              const value = is('function', value_) ? Try$1(() => value_(amauiTheme)) : value_; // Make an instance of amauiStyleSheetManager

              const amauiStyleSheetManager = new AmauiStyleSheetManager$1(value, 'regular', true, 'lower', amauiTheme, amauiStyle, {
                style: {
                  attributes: {
                    method: 'pure'
                  }
                }
              });
              const response = {
                ids: amauiStyleSheetManager.ids,
                amaui_style_sheet_manager: amauiStyleSheetManager,
                sheets: amauiStyleSheetManager.sheets,
                add: amauiStyleSheetManager.add.bind(amauiStyleSheetManager),

                set props(value__) {
                  amauiStyleSheetManager.props = value__;
                },

                update: amauiStyleSheetManager.update.bind(amauiStyleSheetManager),
                remove: amauiStyleSheetManager.remove.bind(amauiStyleSheetManager),
                addRule: amauiStyleSheetManager.sheets.static[0] && amauiStyleSheetManager.sheets.static[0].addRule.bind(amauiStyleSheetManager.sheets.static[0])
              }; // Response

              return response;
            }

            const optionsDefault$4 = {
              amaui_style: {
                get: AmauiStyle$1.first.bind(AmauiStyle$1)
              },
              amaui_theme: {
                get: AmauiTheme$1.first.bind(AmauiTheme$1)
              },
              response: 'css',
              response_json_property_variant: 'cammel'
            };

            function inline$1(value_, props) {
              let options_ = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
              const options = { ...optionsDefault$4,
                ...options_
              }; // Amaui style

              let amauiStyle = options.amaui_style.value || is('function', options.amaui_style.get) && options.amaui_style.get(options.element);
              if (amauiStyle === undefined) amauiStyle = new AmauiStyle$1(); // Amaui theme

              const amauiTheme = options.amaui_theme.value || is('function', options.amaui_theme.get) && options.amaui_theme.get(options.element); // Make value if it's a function

              const value = is('function', value_) ? Try$1(() => value_(amauiTheme)) : value_; // Go through all properties
              // make an AmauiStyleRuleProperty for each prop
              // and then make css string from each one

              let response = '';

              if (is('object', value)) {
                const properties = Object.keys(value);
                const valueNew = {}; // Filter out at-rule and dynamic properies

                properties.filter(prop => prop.indexOf('@') !== 0 && !(is('function', value[prop]) || isAmauiSubscription(value[prop]))).forEach(prop => valueNew[prop] = value[prop]); // Parse dynamic properties into static

                const propertiesDynamic = properties.filter(prop => prop.indexOf('@') !== 0 && (is('function', value[prop]) || isAmauiSubscription(value[prop])));
                propertiesDynamic.forEach(prop => {
                  const valueProp = value[prop];
                  if (is('function', valueProp)) valueNew[prop] = Try$1(() => valueProp(props));else if (isAmauiSubscription(valueProp)) valueNew[prop] = Try$1(() => valueProp.value);
                }); // Make an instance of amauiStyleSheetManager

                const amauiStyleSheetManager = new AmauiStyleSheetManager$1({
                  a: valueNew
                }, 'regular', false, 'upper', amauiTheme, amauiStyle, {
                  style: {
                    attributes: {
                      method: 'inline'
                    }
                  },
                  amaui_style_cache: false
                });
                const rules = amauiStyleSheetManager.sheets.static[0].rules[0].value.rules;
                rules.map(rule => rule.value).forEach(rule => {
                  response += " ".concat(rule.css);
                }); // Make into json

                if (options.response === 'json') {
                  const values = response.split(';').filter(Boolean);
                  response = {};
                  values.forEach(item => {
                    var _property, _value__;

                    let [property, value__] = item.split(':').filter(Boolean);
                    property = (_property = property) === null || _property === void 0 ? void 0 : _property.trim();
                    value__ = (_value__ = value__) === null || _value__ === void 0 ? void 0 : _value__.trim();

                    if (property && value__) {
                      response[options.response_json_property_variant === 'cammel' ? kebabCasetoCammelCase(property) : cammelCaseToKebabCase(property)] = value__;
                    }
                  });
                }
              }

              if (options.response === 'css') response = response.trim(); // Response

              return response;
            }

            if (typeof global$1.setTimeout === 'function') ;
            if (typeof global$1.clearTimeout === 'function') ;

            // from https://github.com/kumavis/browser-process-hrtime/blob/master/index.js
            var performance = global$1.performance || {};
            performance.now        ||
              performance.mozNow     ||
              performance.msNow      ||
              performance.oNow       ||
              performance.webkitNow  ||
              function(){ return (new Date()).getTime() };

            const optionsDefault$3 = {
              production: isEnvironment('browser') || isEnvironment('nodejs') && ['prod', 'production'].indexOf("development") > -1
            };

            function makeClassName(amauiStyle) {
              let options_ = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
              const options = { ...optionsDefault$3,
                ...options_
              }; // If both dev and prod are false, then dev is true

              const production = options.production !== undefined ? options.production : optionsDefault$3.production;
              const makeNameMethodClassName = makeName();
              const makeNameMethodKeyframesName = makeName();

              const domUnique = value => {
                const allClassNames = [...new Set(Array.from(window.document.querySelectorAll('[class]')).flatMap(item => [...item.classList]))];
                return allClassNames.indexOf(value) === -1;
              };

              const method = function () {
                let method_ = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : makeNameMethodClassName;
                let inc = 0;
                return value_ => {
                  const value = {
                    arguments: {
                      value: value_
                    }
                  }; // Make a class name
                  // Production

                  if (production) {
                    value.value = method_.next().value;

                    while (true) {
                      var _options$dom;

                      if ((_options$dom = options.dom) !== null && _options$dom !== void 0 && _options$dom.unique && !domUnique(value.value)) {
                        value.value = method_.next().value;
                      } else break;
                    }
                  } // Development
                  else {
                    value.value = "".concat(value_.property, "-").concat(inc++);

                    while (true) {
                      var _options$dom2;

                      if ((_options$dom2 = options.dom) !== null && _options$dom2 !== void 0 && _options$dom2.unique && !domUnique(value.value)) {
                        value.value = "".concat(value_ === null || value_ === void 0 ? void 0 : value_.property, "-").concat(++inc);
                      } else break;
                    }
                  }

                  return value;
                };
              };

              const methodClassName = method();
              const methodKeyframesName = method(makeNameMethodKeyframesName); // Add methods to subscriptions

              if (amauiStyle) {
                amauiStyle.subscriptions.className.name.subscribe(methodClassName);
                amauiStyle.subscriptions.keyframes.name.subscribe(methodKeyframesName);
              }

              const remove = () => {
                // Remove methods from subscriptions
                if (amauiStyle) {
                  amauiStyle.subscriptions.className.name.unsubscribe(methodClassName);
                  amauiStyle.subscriptions.keyframes.name.unsubscribe(methodKeyframesName);
                }
              };

              const response = {
                methods: {
                  method: methodClassName
                },
                remove
              };
              return response;
            }

            const optionsDefault$2 = {
              ssr: {
                all: true
              }
            }; // As of April, 01, 2022 all prefixed required for some of the browser versions

            const mapAllPropertyPrefixes = {
              'appearance': ['-webkit-', '-moz-'],
              'backdrop-filter': ['-webkit'],
              'box-decoration-break': ['-webkit-'],
              'clip-path': ['-webkit-'],
              'color-adjust': ['-webkit-'],
              'font-smooth': ['-webkit-', '-moz-'],
              'hyphens': ['-webkit-', '-ms-', '-moz-'],
              'initial-letter': ['-webkit-'],
              'line-clamp': ['-webkit-'],
              'writing-mode': ['-webkit-'],
              'text-decoration': ['-webkit-', '-moz-'],
              'text-emphasis': ['-webkit-'],
              'text-orientation': ['-webkit-'],
              'text-size-adjust': ['-webkit-', '-ms-', '-moz-'],
              'user-select': ['-webkit-', '-ms-', '-moz-'],
              'font-kerning': ['-webkit-'],
              'tab-size': ['-o-', '-moz-'],
              'wrap-flow': ['-ms-'],
              'wrap-through': ['-ms-'],
              'grid': ['-ms-'],
              'mask': ['-webkit-'],
              'reflect': ['-webkit-'],
              'flow-into': ['-webkit-', '-ms-'],
              'flow-from': ['-webkit-', '-ms-'],
              'region-fragment': ['-webkit-', '-ms-'],
              'scroll-snap': ['-webkit-', '-ms-'],
              'text-stroke': ['-webkit-'],
              'text-fill': ['-webkit-'],
              'max-content': ['-webkit-', '-moz-'],
              'min-content': ['-webkit-', '-moz-'],
              'fit-content': ['-webkit-', '-moz-'],
              'stretch': ['-webkit-', '-moz-'],
              'available': ['-webkit-', '-moz-'],
              'resolution': ['-webkit-', '-o-'],
              'min-resolution': ['-webkit-', '-o-'],
              'max-resolution': ['-webkit-', '-o-'],
              'keyframe': ['-webkit-', '-o-', '-moz-'],
              'animation': ['-webkit-', '-moz-'],
              'transform': ['-webkit-', '-o-', '-ms-', '-moz-'],
              'transition': ['-webkit-', '-o-', '-moz-']
            }; // Where property index is > -1 and value index is > -1 and replace it with

            const mapAllValuePrefixes = {
              'position': {
                'sticky': ['-webkit-']
              },
              'background-clip': {
                'text': ['-webkit-', '-ms-']
              },
              'background-image': {
                'crossfade': ['-webkit-'],
                'image-set': ['-webkit-', '-o-', '-ms-', '-moz-'],
                'element': ['-moz-'],
                'canvas': ['-webkit-']
              },
              'background': {
                'crossfade': ['-webkit-'],
                'image-set': ['-webkit-', '-o-', '-ms-', '-moz-'],
                'element': ['-moz-'],
                'canvas': ['-webkit-']
              }
            };

            function prefix(amauiStyle) {
              let options_ = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
              const options = { ...optionsDefault$2,
                ...options_
              };

              const valid = (value, property) => {
                if (isEnvironment('browser')) {
                  const element = document.createElement('a');
                  const props = [property];
                  if (property.indexOf('-webkit-') === 0) props.push("webkit".concat(capitalizedCammelCase(property.slice(8))));
                  if (property.indexOf('-o-') === 0) props.push("o".concat(capitalizedCammelCase(property.slice(3))));
                  if (property.indexOf('-ms-') === 0) props.push("ms".concat(capitalizedCammelCase(property.slice(4))));
                  if (property.indexOf('-moz-') === 0) props.push("moz".concat(capitalizedCammelCase(property.slice(5))));

                  for (const prop of props) {
                    if (prop in element.style) {
                      try {
                        element.style[prop] = value;
                        if (!!element.style[prop].length) return true;
                      } catch (error) {}
                    }
                  }

                  return false;
                }

                return options.ssr.all;
              };

              const method = value_ => {
                const value = {
                  value: [],
                  arguments: {
                    value: value_
                  }
                }; // if initial value property are valid return

                if (isEnvironment('browser') && valid(value_.value, value_.property)) return value; // make mix of all prefix variants for value and property and all mixes other than the original

                const propertyPrefixesKey = Object.keys(mapAllPropertyPrefixes).find(item => value_.property.indexOf(item) > -1);
                const propertyPrefixes = mapAllPropertyPrefixes[propertyPrefixesKey] || [];
                const properties = [value_.property, ...propertyPrefixes.map(prefix_ => "".concat(prefix_).concat(value_.property))];
                const valuePrefixesProps = mapAllValuePrefixes[value_.property];
                const valuePrefixesKey = valuePrefixesProps && Object.keys(valuePrefixesProps).find(item => value_.value.indexOf(item) > -1);
                const valuePrefixes = valuePrefixesProps && valuePrefixesProps[valuePrefixesKey] || [];
                const values = [value_.value, ...valuePrefixes.map(prefix_ => "".concat(prefix_).concat(value_.value))];
                const items = [];
                properties.forEach(property__ => {
                  values.forEach(value__ => {
                    if (!(property__ === value_.property && value__ === value_.value)) {
                      items.push({
                        property: property__,
                        value: value__
                      });
                    }
                  });
                }); // for each one that works push it to value.value

                items.forEach(item => {
                  if (valid(item.value, item.property)) value.value.push(item);
                });
                return value;
              }; // Add methods to subscriptions


              if (amauiStyle) {
                amauiStyle.subscriptions.rule.prefix.subscribe(method);
              }

              const remove = () => {
                // Remove methods from subscriptions
                if (amauiStyle) {
                  amauiStyle.subscriptions.rule.prefix.unsubscribe(method);
                }
              };

              const response = {
                methods: {
                  method
                },
                remove
              };
              return response;
            }

            function rtl(amauiStyle) {
              const method = value_ => {
                const value = {
                  value: {
                    value: '',
                    property: ''
                  },
                  arguments: {
                    value: value_
                  }
                };

                if (is('string', value_.value)) {
                  if (value_.value.indexOf('left') > -1) value.value.value = value_.value.replace(/left/ig, 'right');else if (value_.value.indexOf('right') > -1) value.value.value = value_.value.replace(/right/ig, 'left');else value.value.value = value_.value;
                }

                if (is('string', value_.property)) {
                  if (value_.property.indexOf('left') > -1) value.value.property = value_.property.replace(/left/ig, 'right');else if (value_.property.indexOf('right') > -1) value.value.property = value_.property.replace(/right/ig, 'left');else value.value.property = value_.property;
                }

                return value;
              }; // Add method to subscriptions


              if (amauiStyle) {
                amauiStyle.subscriptions.rule.rtl.subscribe(method);
              }

              const remove = () => {
                // Remove method from subscriptions
                if (amauiStyle) {
                  amauiStyle.subscriptions.rule.rtl.unsubscribe(method);
                }
              };

              const response = {
                methods: {
                  method
                },
                remove
              };
              return response;
            }

            const optionsDefault$1 = {
              priority: 'individual'
            };

            function sort(amauiStyle) {
              let options_ = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
              const options = { ...optionsDefault$1,
                ...options_
              };

              const method = values => {
                const value = {
                  arguments: {
                    values
                  }
                };

                if (is('array', values)) {
                  const priority = options.priority; // Sort by grouping all rules

                  values.sort((a, b) => {
                    if (a.value instanceof AmauiStyleRuleProperty$1 && !(b.value instanceof AmauiStyleRuleProperty$1)) return -1;
                    if (a.constructor === b.constructor || is('simple', a.value) && is('simple', b.value)) return 0;
                    return 1;
                  }); // Order by priority

                  if (priority !== 'original') values.sort((a, b) => {
                    if (!(a.value instanceof AmauiStyleRuleProperty$1 && b.value instanceof AmauiStyleRuleProperty$1 || is('simple', a.value) && is('simple', b.value))) return 0;
                    if ((a === null || a === void 0 ? void 0 : a.property) < (b === null || b === void 0 ? void 0 : b.property)) return priority === 'individual' ? -1 : 1;
                    if ((a === null || a === void 0 ? void 0 : a.property) > (b === null || b === void 0 ? void 0 : b.property)) return priority === 'individual' ? 1 : -1;
                  }); // Add sorted array to value

                  value.value = values;
                }

                return value;
              }; // Add method to subscriptions


              if (amauiStyle) {
                amauiStyle.subscriptions.rules.sort.subscribe(method);
              }

              const remove = () => {
                // Remove method from subscriptions
                if (amauiStyle) {
                  amauiStyle.subscriptions.rules.sort.unsubscribe(method);
                }
              };

              const response = {
                methods: {
                  method
                },
                remove
              };
              return response;
            }

            const optionsDefault = {
              units: {}
            };
            const unitsDefault = {
              'animation': 's',
              'animation-delay': 's',
              'animation-duration': 's',
              'ascent-override': '%',
              'background': '%',
              'background-position': '%',
              'background-size': '%',
              'background-position-x': '%',
              'background-position-y': '%',
              'border': 'px',
              'border-top': 'px',
              'border-right': 'px',
              'border-bottom': 'px',
              'border-left': 'px',
              'border-radius': 'px',
              'border-top-left-radius': 'px',
              'border-top-right-radius': 'px',
              'border-bottom-left-radius': 'px',
              'border-bottom-right-radius': 'px',
              'border-width': 'px',
              'border-top-width': 'px',
              'border-right-width': 'px',
              'border-bottom-width': 'px',
              'border-left-width': 'px',
              'border-image': '%',
              'border-image-outset': 'px',
              'border-image-slice': '%',
              'border-image-width': 'px',
              'border-spacing': 'px',
              'block-size': 'px',
              'border-block': 'px',
              'border-block-end': 'px',
              'border-block-end-width': 'px',
              'border-block-start': 'px',
              'border-block-start-width': 'px',
              'border-block-width': 'px',
              'border-end-end-radius': 'px',
              'border-end-start-radius': 'px',
              'border-inline': 'px',
              'border-inline-end': 'px',
              'border-inline-end-width': 'px',
              'border-inline-start': 'px',
              'border-inline-start-width': 'px',
              'border-inline-width': 'px',
              'border-style': 'px',
              'border-start-end-radius': 'px',
              'border-start-start-radius': 'px',
              'box-shadow': 'px',
              'bottom': '%',
              'columns': 'px',
              'column-width': 'px',
              'column-gap': 'px',
              'column-rule': 'px',
              'column-rule-width': 'px',
              'contain-intrinsic-block-size': 'px',
              'contain-intrinsic-height': 'px',
              'contain-intrinsic-inline-size': 'px',
              'contain-intrinsic-size': 'px',
              'contain-intrinsic-width': 'px',
              'cx': '%',
              'cy': '%',
              'descent-override': '%',
              'flex': '%',
              'flex-basis': '%',
              'font': 'px',
              'font-size': 'px',
              'font-synthesis': 'px',
              'font-synthesis-weight': 'px',
              'gap': 'px',
              'grid': 'px',
              'grid-auto-columns': 'px',
              'grid-auto-rows': 'px',
              'grid-column-gap': 'px',
              'grid-gap': 'px',
              'grid-row-gap': 'px',
              'grid-template': 'px',
              'grid-template-rows': 'px',
              'grid-template-columns': 'px',
              'height': 'px',
              'inline-size': 'px',
              'inset': 'px',
              'inset-block': 'px',
              'inset-block-end': 'px',
              'inset-block-start': 'px',
              'inset-inline': 'px',
              'inset-inline-end': 'px',
              'inset-inline-start': 'px',
              'left': '%',
              'letter-spacing': 'px',
              'line-gap-override': '%',
              'margin': 'px',
              'margin-top': 'px',
              'margin-right': 'px',
              'margin-bottom': 'px',
              'margin-left': 'px',
              'margin-block': 'px',
              'margin-block-end': 'px',
              'margin-block-start': 'px',
              'margin-inline': 'px',
              'margin-inline-end': 'px',
              'margin-inline-start': 'px',
              'mask': 'px',
              'mask-position': '%',
              'mask-size': '%',
              'max-block-size': 'px',
              'max-inline-size': 'px',
              'min-block-size': 'px',
              'min-inline-size': 'px',
              'max-height': 'px',
              'max-width': 'px',
              'min-height': 'px',
              'min-width': 'px',
              'object-position': '%',
              'outline': 'px',
              'outline-offset': 'px',
              'outline-width': 'px',
              'offset': 'px',
              'offset-anchor': '%',
              'offset-distance': '%',
              'offset-position': '%',
              'offset-rotate': 'deg',
              'padding': 'px',
              'padding-top': 'px',
              'padding-right': 'px',
              'padding-bottom': 'px',
              'padding-left': 'px',
              'padding-block': 'px',
              'padding-block-end': 'px',
              'padding-block-start': 'px',
              'padding-inline': 'px',
              'padding-inline-end': 'px',
              'padding-inline-start': 'px',
              'perspective': 'px',
              'right': '%',
              'row-gap': 'px',
              'scroll-margin': 'px',
              'scroll-margin-block': 'px',
              'scroll-margin-block-end': 'px',
              'scroll-margin-block-start': 'px',
              'scroll-margin-bottom': 'px',
              'scroll-margin-inline': 'px',
              'scroll-margin-inline-end': 'px',
              'scroll-margin-inline-start': 'px',
              'scroll-margin-left': 'px',
              'scroll-margin-right': 'px',
              'scroll-margin-top': 'px',
              'scroll-padding': 'px',
              'scroll-padding-block': 'px',
              'scroll-padding-block-end': 'px',
              'scroll-padding-block-start': 'px',
              'scroll-padding-bottom': 'px',
              'scroll-padding-inline': 'px',
              'scroll-padding-inline-end': 'px',
              'scroll-padding-inline-start': 'px',
              'scroll-padding-left': 'px',
              'scroll-padding-right': 'px',
              'scroll-padding-top': 'px',
              'shape-margin': 'px',
              'size': 'in',
              'size-adjust': '%',
              'text-decoration': 'px',
              'text-decoration-thickness': 'px',
              'text-indent': 'px',
              'text-size-adjust': '%',
              'text-shadow': 'px',
              'text-underline-offset': 'px',
              'top': '%',
              'transform-origin': '%',
              'transition': 's',
              'transition-delay': 's',
              'transition-duration': 's',
              'width': 'px',
              'word-spacing': 'px',
              'zoom': '%'
            };

            function unit(amauiStyle) {
              let options_ = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
              const options = { ...optionsDefault,
                ...options_
              };
              const units = { ...unitsDefault,
                ...(options.units || {})
              };

              const method = value_ => {
                // Normalize property
                const property = {
                  cammel: cammelCaseToKebabCase(value_.property),
                  kebab: kebabCasetoCammelCase(value_.property)
                };

                const method_ = item => is('function', item) ? item(value_.value) : {
                  value: "".concat(value_.value).concat(item),
                  unit: item
                };

                const value = {
                  value: method_(units[property.cammel]) || method_(units[property.kebab]) || '',
                  arguments: {
                    value: copy$1(value_)
                  }
                };
                return value;
              }; // Add method to subscriptions


              if (amauiStyle) {
                amauiStyle.subscriptions.rule.unit.subscribe(method);
              }

              const remove = () => {
                // Remove method from subscriptions
                if (amauiStyle) {
                  amauiStyle.subscriptions.rule.unit.unsubscribe(method);
                }
              };

              const response = {
                methods: {
                  method
                },
                remove
              };
              return response;
            }

            function valueObject(amauiStyle) {
              const method = function () {
                let value_ = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
                const value = {
                  arguments: {
                    value: value_
                  }
                };
                const {
                  property
                } = value_;

                if (is('object', value_.value)) {
                  // Object extendable
                  let top;
                  let right;
                  let bottom;
                  let left;
                  let width;
                  let height;
                  let horizontal;
                  let vertical;
                  let templateRows;
                  let templateColumns;
                  let position;
                  let duration;
                  let delay;
                  let easingFunction;

                  switch (property) {
                    case 'animation':
                      duration = valueResolve('animation-duration', value_.value['duration'], amauiStyle).value[0];
                      duration = duration !== undefined ? duration : value_.value['duration'];
                      delay = valueResolve('animation-delay', value_.value['delay'], amauiStyle).value[0];
                      delay = delay !== undefined ? delay : value_.value['delay'];
                      easingFunction = value_.value['easing-function'] || value_.value['easingFunction'];
                      const iterationCount = value_.value['iterationCount'] !== undefined ? value_.value['iterationCount'] : value_.value['iteration-count'];
                      const fillMode = value_.value['fillMode'] !== undefined ? value_.value['fillMode'] : value_.value['fill-mode'];
                      const playState = value_.value['playState'] !== undefined ? value_.value['playState'] : value_.value['play-state'];
                      value.value = [[value_.value['name'], duration, easingFunction, delay, iterationCount, value_.value['direction'], fillMode, playState].filter(item => item !== undefined).join(' ')];
                      break;

                    case 'background':
                      position = valueResolve('background-position', value_.value['position'], amauiStyle).value[0];
                      position = position !== undefined ? position : value_.value['position'];
                      value.value = [[value_.value['color'], value_.value['image'], value_.value['repeat'], position].filter(item => item !== undefined).join(' ')];
                      break;

                    case 'border':
                    case 'border-top':
                    case 'border-right':
                    case 'border-bottom':
                    case 'border-left':
                    case 'border-block':
                    case 'border-block-end':
                    case 'border-block-start':
                    case 'border-inline-end':
                    case 'border-inline-start':
                    case 'column-rule':
                      width = valueResolve(property, value_.value['width'], amauiStyle).value[0];
                      width = width !== undefined ? width : value_.value['width'];
                      value.value = [[width, value_.value['style'], value_.value['color']].filter(item => item !== undefined).join(' ')];
                      break;

                    case 'outline':
                      width = valueResolve('outline-width', value_.value['width'], amauiStyle).value[0] || value_.value['width'];
                      width = width !== undefined ? width : value_.value['width'];
                      value.value = [[value_.value['color'], value_.value['style'], width].filter(item => item !== undefined).join(' ')];
                      break;

                    case 'border-color':
                    case 'border-top-color':
                    case 'border-right-color':
                    case 'border-bottom-color':
                    case 'border-left-color':
                      top = value_.value['top'] !== undefined ? value_.value['top'] : 'transparent';
                      right = value_.value['right'] !== undefined ? value_.value['right'] : 'transparent';
                      bottom = value_.value['bottom'] !== undefined ? value_.value['bottom'] : 'transparent';
                      left = value_.value['left'] !== undefined ? value_.value['left'] : 'transparent';
                      value.value = [[top, right, bottom, left].filter(item => item !== undefined).join(' ')];
                      break;

                    case 'background-position':
                      const keys = Object.keys(value_.value);
                      value.value = [[]];
                      keys.forEach(item => {
                        let value__ = valueResolve('background-position', value_.value[item], amauiStyle).value[0];
                        value__ = value__ !== undefined ? value__ : value_.value[item];
                        if (value__ !== undefined) value.value[0].push(value__);
                      });
                      value.value[0] = value.value[0].join(' ');
                      break;

                    case 'font':
                      const lineHeight = value_.value['line-height'] || value_.value['lineHeight'] || value_.value['height'];
                      let fontSize = valueResolve('font-size', value_.value['size'], amauiStyle).value[0];
                      fontSize = fontSize !== undefined ? fontSize : value_.value['size'];
                      let fontFamily = valueResolve('font-family', value_.value['family'], amauiStyle).value[0];
                      fontFamily = fontFamily !== undefined ? fontFamily : value_.value['family'];
                      const other = fontSize && lineHeight ? ["".concat(fontSize, "/").concat(lineHeight)] : [fontSize, lineHeight];
                      value.value = [[value_.value['style'], value_.value['weight'], ...other, fontFamily].filter(item => item !== undefined).join(' ')];
                      break;

                    case 'margin':
                    case 'padding':
                    case 'border-width':
                    case 'border-image-outset':
                      top = valueResolve(property, value_.value['top'], amauiStyle).value[0];
                      top = top !== undefined ? top : value_.value['top'] !== undefined ? value_.value['top'] : 0;
                      right = valueResolve(property, value_.value['right'], amauiStyle).value[0];
                      right = right !== undefined ? right : value_.value['right'] !== undefined ? value_.value['right'] : 0;
                      bottom = valueResolve(property, value_.value['bottom'], amauiStyle).value[0];
                      bottom = bottom !== undefined ? bottom : value_.value['bottom'] !== undefined ? value_.value['bottom'] : 0;
                      left = valueResolve(property, value_.value['left'], amauiStyle).value[0];
                      left = left !== undefined ? left : value_.value['left'] !== undefined ? value_.value['left'] : 0;
                      value.value = [[top, right, bottom, left].filter(item => item !== undefined).join(' ')];
                      break;

                    case 'scroll-margin':
                    case 'scroll-padding':
                      top = valueResolve(property, value_.value['top'], amauiStyle).value[0];
                      top = top !== undefined ? top : value_.value['top'] !== undefined ? value_.value['top'] : 0;
                      right = valueResolve(property, value_.value['right'], amauiStyle).value[0];
                      right = right !== undefined ? right : value_.value['right'] !== undefined ? value_.value['right'] : 0;
                      bottom = valueResolve(property, value_.value['bottom'], amauiStyle).value[0];
                      bottom = bottom !== undefined ? bottom : value_.value['bottom'] !== undefined ? value_.value['bottom'] : 0;
                      left = valueResolve(property, value_.value['left'], amauiStyle).value[0];
                      left = left !== undefined ? left : value_.value['left'] !== undefined ? value_.value['left'] : 0;
                      value.value = [[bottom, left, right, top].filter(item => item !== undefined).join(' ')];
                      break;

                    case 'overflow':
                    case 'background-repeat':
                      value.value = [[value_.value['x'], value_.value['y']].filter(item => item !== undefined).join(' ')];
                      break;

                    case 'background-size':
                      width = valueResolve(property, value_.value['width'], amauiStyle).value[0];
                      width = width !== undefined ? width : value_.value['width'] !== undefined ? value_.value['width'] : 0;
                      height = valueResolve(property, value_.value['height'], amauiStyle).value[0];
                      height = height !== undefined ? height : value_.value['height'] !== undefined ? value_.value['height'] : 0;
                      value.value = [[width, height].filter(item => item !== undefined).join(' ')];
                      break;

                    case 'border-bottom-left-radius':
                    case 'border-bottom-right-radius':
                    case 'border-top-left-radius':
                    case 'border-top-right-radius':
                    case 'border-end-end-radius':
                    case 'border-end-start-radius':
                    case 'border-start-end-radius':
                    case 'border-start-start-radius':
                      horizontal = valueResolve('border-radius', value_.value['horizontal'], amauiStyle).value[0];
                      horizontal = horizontal !== undefined ? horizontal : value_.value['horizontal'] !== undefined ? value_.value['horizontal'] : 0;
                      vertical = valueResolve('border-radius', value_.value['vertical'], amauiStyle).value[0];
                      vertical = vertical !== undefined ? vertical : value_.value['vertical'] !== undefined ? value_.value['vertical'] : 0;
                      value.value = [[horizontal, vertical].filter(item => item !== undefined).join(' ')];
                      break;

                    case 'border-image':
                      width = valueResolve('border-image-width', value_.value['width'], amauiStyle).value[0];
                      width = width !== undefined ? width : value_.value['width'];
                      let outset = valueResolve('border-image-outset', value_.value['outset'], amauiStyle).value[0];
                      outset = outset !== undefined ? outset : value_.value['outset'];
                      const slice = value_.value['slice'];
                      const widthAndSlice = width && slice ? "".concat(slice, " / ").concat(width) : width ? width : slice;
                      value.value = [[value_.value['source'], widthAndSlice, outset, value_.value['repeat']].filter(item => item !== undefined).join(' ')];
                      break;

                    case 'border-radius':
                      let topLeft = valueResolve('border-top-left-radius', value_.value['top-left'] !== undefined ? value_.value['top-left'] : value_.value['topLeft'], amauiStyle).value[0];
                      topLeft = topLeft !== undefined ? topLeft : value_.value['top-left'] !== undefined ? value_.value['top-left'] : value_.value['topLeft'] || 0;
                      let topRight = valueResolve('border-top-right-radius', value_.value['top-right'] !== undefined ? value_.value['top-right'] : value_.value['topRight'], amauiStyle).value[0];
                      topRight = topRight !== undefined ? topRight : value_.value['top-right'] !== undefined ? value_.value['top-right'] : value_.value['topRight'] || 0;
                      let bottomRight = valueResolve('border-bottom-right-radius', value_.value['bottom-right'] !== undefined ? value_.value['bottom-right'] : value_.value['bottomRight'], amauiStyle).value[0];
                      bottomRight = bottomRight !== undefined ? bottomRight : value_.value['bottom-right'] !== undefined ? value_.value['bottom-right'] : value_.value['bottomRight'] || 0;
                      let bottomLeft = valueResolve('border-bottom-left-radius', value_.value['bottom-left'] !== undefined ? value_.value['bottom-left'] : value_.value['bottomLeft'], amauiStyle).value[0];
                      bottomLeft = bottomLeft !== undefined ? bottomLeft : value_.value['bottom-left'] !== undefined ? value_.value['bottom-left'] : value_.value['bottomLeft'] || 0;
                      value.value = [[topLeft, topRight, bottomRight, bottomLeft].filter(item => item !== undefined).join(' ')];
                      break;

                    case 'border-style':
                      top = value_.value['top'] !== undefined ? value_.value['top'] : 'transparent';
                      right = value_.value['right'] !== undefined ? value_.value['right'] : 'transparent';
                      bottom = value_.value['bottom'] !== undefined ? value_.value['bottom'] : 'transparent';
                      left = value_.value['left'] !== undefined ? value_.value['left'] : 'transparent';
                      value.value = [[top, right, bottom, left].filter(item => item !== undefined).join(' ')];
                      break;

                    case 'columns':
                      width = valueResolve('column-width', value_.value['width'], amauiStyle).value[0];
                      width = width !== undefined ? width : value_.value['width'];
                      value.value = [[width, value_.value['count']].filter(item => item !== undefined).join(' ')];
                      break;

                    case 'flex':
                      let basis = valueResolve('flex-basis', value_.value['basis'], amauiStyle).value[0];
                      basis = basis !== undefined ? basis : value_.value['basis'];
                      value.value = [[value_.value['grow'], value_.value['shrink'], basis].filter(item => item !== undefined).join(' ')];
                      break;

                    case 'flex-flow':
                      value.value = [[value_.value['direction'], value_.value['wrap']].filter(item => item !== undefined).join(' ')];
                      break;

                    case 'gap':
                      let row = valueResolve('gap', value_.value['row'], amauiStyle).value[0];
                      row = row !== undefined ? row : value_.value['row'];
                      let column = valueResolve('gap', value_.value['column'], amauiStyle).value[0];
                      column = column !== undefined ? column : value_.value['column'];
                      value.value = [[row, column].filter(item => item !== undefined).join(' ')];
                      break;

                    case 'grid':
                      let autoRows = valueResolve('grid-auto-rows', value_.value['autoRows'] !== undefined ? value_.value['autoRows'] : value_.value['auto-rows'], amauiStyle).value[0];
                      autoRows = autoRows !== undefined ? autoRows : value_.value['autoRows'] !== undefined ? value_.value['autoRows'] : value_.value['auto-rows'];
                      let autoColumns = valueResolve('grid-auto-columns', value_.value['autoColumns'] !== undefined ? value_.value['autoColumns'] : value_.value['auto-columns'], amauiStyle).value[0];
                      autoColumns = autoColumns !== undefined ? autoColumns : value_.value['autoColumns'] !== undefined ? value_.value['autoColumns'] : value_.value['auto-columns'];
                      templateRows = valueResolve('grid-template-rows', value_.value['templateRows'] !== undefined ? value_.value['templateRows'] : value_.value['template-rows'], amauiStyle).value[0];
                      templateRows = templateRows !== undefined ? templateRows : value_.value['templateRows'] !== undefined ? value_.value['templateRows'] : value_.value['template-rows'];
                      templateColumns = valueResolve('grid-template-rows', value_.value['templateColumns'] !== undefined ? value_.value['templateColumns'] : value_.value['template-columns'], amauiStyle).value[0];
                      templateColumns = templateColumns !== undefined ? templateColumns : value_.value['templateColumns'] !== undefined ? value_.value['templateColumns'] : value_.value['template-columns'];

                      if (templateRows) {
                        value.value = [[templateRows, '/', value_.value['auto-flow'] !== undefined ? value_.value['auto-flow'] : value_.value['autoFlow'], autoColumns].filter(item => item !== undefined).join(' ')];
                      } else {
                        value.value = [[value_.value['auto-flow'] !== undefined ? value_.value['auto-flow'] : value_.value['autoFlow'], autoRows, '/', templateColumns].filter(item => item !== undefined).join(' ')];
                      }

                      break;

                    case 'grid-area':
                      value.value = [[(value_.value['row-start'] !== undefined ? value_.value['row-start'] : value_.value['rowStart']) || 0, '/', (value_.value['column-start'] !== undefined ? value_.value['column-start'] : value_.value['columnStart']) || 0, '/', (value_.value['row-end'] !== undefined ? value_.value['row-end'] : value_.value['rowEnd']) || 0, '/', (value_.value['column-end'] !== undefined ? value_.value['column-end'] : value_.value['columnEnd']) || 0].filter(item => item !== undefined).join(' ')];
                      break;

                    case 'grid-column':
                    case 'grid-row':
                      value.value = [[value_.value['end'] || 0, '/', value_.value['start'] || 0].filter(item => item !== undefined).join(' ')];
                      break;

                    case 'grid-template':
                      templateRows = valueResolve('grid-template-rows', value_.value['rows'], amauiStyle).value[0];
                      templateRows = templateRows !== undefined ? templateRows : value_.value['rows'];
                      templateColumns = valueResolve('grid-template-columns', value_.value['columns'], amauiStyle).value[0];
                      templateColumns = templateColumns !== undefined ? templateColumns : value_.value['columns'];
                      value.value = [[value_.value['areas'], templateRows, '/', templateColumns].filter(item => item !== undefined).join(' ')];
                      break;

                    case 'list-style':
                      value.value = [[value_.value['type'], value_.value['image'], value_.value['position']].filter(item => item !== undefined).join(' ')];
                      break;

                    case 'mask':
                      value.value = [[value_.value['image'], value_.value['mode'], value_.value['repeat'], value_.value['position'], value_.value['clip'], value_.value['origin'], value_.value['size'], value_.value['composite']].filter(item => item !== undefined).join(' ')];
                      break;

                    case 'offset':
                      let anchor = valueResolve('offset-anchor', value_.value['anchor'], amauiStyle).value[0];
                      anchor = anchor !== undefined ? anchor : value_.value['anchor'];
                      let distance = valueResolve('offset-distance', value_.value['distance'], amauiStyle).value[0];
                      distance = distance !== undefined ? distance : value_.value['distance'];
                      let rotate = valueResolve('offset-rotate', value_.value['rotate'], amauiStyle).value[0];
                      rotate = rotate !== undefined ? rotate : value_.value['rotate'];
                      position = valueResolve('offset-position', value_.value['position'], amauiStyle).value[0];
                      position = position !== undefined ? position : value_.value['position'];
                      value.value = [[value_.value['path'], distance, rotate, '/', position, anchor].filter(item => item !== undefined).join(' ')];
                      break;

                    case 'place-items':
                    case 'place-self':
                    case 'place-content':
                      value.value = [[value_.value['align'], value_.value['justify']].filter(item => item !== undefined).join(' ')];
                      break;

                    case 'text-decoration':
                      let thickness = valueResolve('text-decoration-thickness', value_.value['thickness'], amauiStyle).value[0];
                      thickness = thickness !== undefined ? thickness : value_.value['thickness'];
                      value.value = [[value_.value['line'], value_.value['style'], value_.value['color'], thickness].filter(item => item !== undefined).join(' ')];
                      break;

                    case 'text-emphasis':
                      value.value = [[value_.value['style'], value_.value['color']].filter(item => item !== undefined).join(' ')];
                      break;

                    case 'transition':
                      duration = valueResolve('transition-duration', value_.value['duration'], amauiStyle).value[0];
                      duration = duration !== undefined ? duration : value_.value['duration'];
                      delay = valueResolve('transition-delay', value_.value['delay'], amauiStyle).value[0] || value_.value['delay'];
                      delay = delay !== undefined ? delay : value_.value['delay'];
                      easingFunction = value_.value['easing-function'] || value_.value['easingFunction'];
                      easingFunction = easingFunction !== undefined ? easingFunction : value_.value['easingFunction'];
                      value.value = [[value_.value['name'], duration, easingFunction, delay].filter(item => item !== undefined).join(' ')];
                      break;
                  }
                }

                return value;
              }; // Add method to subscriptions


              if (amauiStyle) {
                amauiStyle.subscriptions.rule.value.subscribe(method);
              }

              const remove = () => {
                // Remove method from subscriptions
                if (amauiStyle) {
                  amauiStyle.subscriptions.rule.value.unsubscribe(method);
                }
              };

              const response = {
                methods: {
                  method
                },
                remove
              };
              return response;
            }

            const AmauiStyleContext = /*#__PURE__*/React__default["default"].createContext(new AmauiStyle$1());
            var AmauiStyleContext$1 = AmauiStyleContext;

            function _extends() {
              _extends = Object.assign ? Object.assign.bind() : function (target) {
                for (var i = 1; i < arguments.length; i++) {
                  var source = arguments[i];

                  for (var key in source) {
                    if (Object.prototype.hasOwnProperty.call(source, key)) {
                      target[key] = source[key];
                    }
                  }
                }

                return target;
              };
              return _extends.apply(this, arguments);
            }

            function _objectWithoutPropertiesLoose(source, excluded) {
              if (source == null) return {};
              var target = {};
              var sourceKeys = Object.keys(source);
              var key, i;

              for (i = 0; i < sourceKeys.length; i++) {
                key = sourceKeys[i];
                if (excluded.indexOf(key) >= 0) continue;
                target[key] = source[key];
              }

              return target;
            }

            function _objectWithoutProperties(source, excluded) {
              if (source == null) return {};
              var target = _objectWithoutPropertiesLoose(source, excluded);
              var key, i;

              if (Object.getOwnPropertySymbols) {
                var sourceSymbolKeys = Object.getOwnPropertySymbols(source);

                for (i = 0; i < sourceSymbolKeys.length; i++) {
                  key = sourceSymbolKeys[i];
                  if (excluded.indexOf(key) >= 0) continue;
                  if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue;
                  target[key] = source[key];
                }
              }

              return target;
            }

            const _excluded$2 = ["children", "value"];

            function makeAmauiStyle(element) {
              const amauiStyle = new AmauiStyle$1(element); // Add all the plugins

              amauiStyle.plugins.add = [makeClassName, unit, rtl, sort, valueObject];
              return amauiStyle;
            }

            function AmauiStyleProvider(props) {
              const {
                children,
                value: value_
              } = props,
                    other = _objectWithoutProperties(props, _excluded$2);

              const ref = React__default["default"].useRef();
              const [value, setValue] = React__default["default"].useState(() => {
                if (value_ === undefined || !(value_ instanceof AmauiStyle$1)) return makeAmauiStyle();
                return value_;
              });
              React__default["default"].useEffect(() => {
                if (ref.current) {
                  value.element = ref.current; // Init

                  value.init();
                  const valueNew = new AmauiStyle$1(); // Copy over from value

                  Object.keys(value).forEach(prop => valueNew[prop] = value[prop]);
                  setValue(valueNew);
                }
              }, []);

              const update = updateValue => {
                if (updateValue !== undefined) {
                  const valueNew = new AmauiStyle$1();
                  Object.keys(value).forEach(prop => valueNew[prop] = value[prop]);
                  is$1('object', updateValue) && Object.keys(updateValue).forEach(prop => valueNew[prop] = updateValue[prop]);
                  setValue(valueNew);
                  return valueNew;
                }
              }; // Update method


              value.updateWithRerender = update;
              return /*#__PURE__*/React__default["default"].createElement(AmauiStyleContext$1.Provider, {
                value: value
              }, /*#__PURE__*/React__default["default"].createElement("div", _extends({
                ref: ref
              }, other), children));
            }

            function useAmauiStyle() {
              const value = React__default["default"].useContext(AmauiStyleContext$1);
              return value;
            }

            const AmauiThemeContext = /*#__PURE__*/React__default["default"].createContext(new AmauiTheme$1());
            var AmauiThemeContext$1 = AmauiThemeContext;

            function useAmauiTheme() {
              const value = React__default["default"].useContext(AmauiThemeContext$1);
              return value;
            }

            const _excluded$1 = ["children", "value"];

            function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }

            function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }

            const resolveValue = value => {
              const toFilterOut = ['id', 'element'];
              const valueNew = {};
              Object.keys(value).filter(item => toFilterOut.indexOf(item) === -1).forEach(item => valueNew[item] = value[item]);
              return valueNew;
            };

            function AmauiThemeProvider(props) {
              const {
                children,
                value: valueLocal = {}
              } = props,
                    other = _objectWithoutProperties(props, _excluded$1);

              const ref = React__default["default"].useRef();
              const valueParent = useAmauiTheme() || {};
              const [value, setValue] = React__default["default"].useState(() => new AmauiTheme$1(merge$1(copy$1(resolveValue(_objectSpread({}, valueLocal))), copy$1(resolveValue(_objectSpread({}, valueParent))), {
                copy: true
              })));
              React__default["default"].useEffect(() => {
                if (ref.current) {
                  const amauiTheme = new AmauiTheme$1(value, ref.current);
                  amauiTheme.id = value.id; // Init

                  setValue(amauiTheme);
                }
              }, []);
              React__default["default"].useEffect(() => {
                value.update(merge$1(copy$1(resolveValue(_objectSpread({}, valueLocal))), copy$1(resolveValue(_objectSpread({}, valueParent))), {
                  copy: true
                }));
                const amauiTheme = new AmauiTheme$1(value, ref.current);
                amauiTheme.id = value.id; // Init

                setValue(amauiTheme);
              }, [hash$1(resolveValue(valueLocal)), hash$1(resolveValue(valueParent))]);

              const update = updateValue => {
                if (updateValue !== undefined) {
                  // Update
                  value.update(updateValue);
                  const amauiTheme = new AmauiTheme$1(value, ref.current);
                  amauiTheme.id = value.id; // Init

                  setValue(amauiTheme);
                  return value;
                }
              }; // Update method


              value.updateWithRerender = update;
              return /*#__PURE__*/React__default["default"].createElement(AmauiThemeContext$1.Provider, {
                value: value
              }, /*#__PURE__*/React__default["default"].createElement("div", _extends({
                ref: ref
              }, other), children));
            }

            function reset() {
              let value = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
              let options_ = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
              let response;

              function useReset(props) {
                const [values, setValues] = React__default["default"].useState({
                  classes: {},
                  classNames: {},
                  keyframes: {},
                  styles: () => {}
                });
                const amauiStyle = useAmauiStyle();
                const amauiTheme = useAmauiTheme(); // Init only once
                // it has to be in body of method
                // as for ssr it actually calls the method
                // and it doesn't use hooks on ssr

                if (response === undefined) {
                  const options = {
                    amaui_style: {
                      value: undefined
                    },
                    amaui_theme: {
                      value: undefined
                    }
                  }; // AmauiStyle

                  if (amauiStyle !== undefined) options.amaui_style.value = amauiStyle; // AmauiTheme

                  if (amauiTheme !== undefined) options.amaui_theme.value = amauiTheme;
                  if (response === undefined) response = reset$1(value, merge$1(options, options_, {
                    copy: true
                  })); // Update values for ssr as a priorty

                  setValues(names(response.amaui_style_sheet_manager.names));
                } // Add


                React__default["default"].useEffect(() => {
                  const addValues = response.add(props);
                  setValues(addValues); // Updates for amauiTheme

                  const method = () => {
                    if (is$1('function', value)) {
                      var _response;

                      const valueNew = value(amauiTheme); // Update

                      if (((_response = response) === null || _response === void 0 ? void 0 : _response.update) !== undefined) response.update(valueNew);
                    }
                  };

                  if (amauiTheme) amauiTheme.subscriptions.update.subscribe(method); // Clean up

                  return () => {
                    var _addValues$ids;

                    // Unsubscribe
                    if (amauiTheme) amauiTheme.subscriptions.update.unsubscribe(method); // Remove

                    response.remove((_addValues$ids = addValues.ids) === null || _addValues$ids === void 0 ? void 0 : _addValues$ids.dynamic);
                  };
                }, []); // Update props

                React__default["default"].useEffect(() => {
                  if (response !== undefined && values.ids) response.props = {
                    ids: values.ids.dynamic,
                    props
                  };
                }, [hash$1(props)]);
                return values;
              }

              return useReset;
            }

            function pure(value) {
              let options_ = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
              let response;

              function usePure(props) {
                const [values, setValues] = React__default["default"].useState({
                  classes: {},
                  classNames: {},
                  keyframes: {},
                  styles: () => {}
                });
                const amauiStyle = useAmauiStyle();
                const amauiTheme = useAmauiTheme(); // Init only once
                // it has to be in body of method
                // as for ssr it actually calls the method
                // and it doesn't use hooks on ssr

                if (response === undefined) {
                  const options = {
                    amaui_style: {
                      value: undefined
                    },
                    amaui_theme: {
                      value: undefined
                    }
                  }; // AmauiStyle

                  if (amauiStyle !== undefined) options.amaui_style.value = amauiStyle; // AmauiTheme

                  if (amauiTheme !== undefined) options.amaui_theme.value = amauiTheme;
                  if (response === undefined) response = pure$1(value, merge$1(options, options_, {
                    copy: true
                  })); // Update values for ssr as a priorty

                  setValues(names(response.amaui_style_sheet_manager.names));
                } // Add


                React__default["default"].useEffect(() => {
                  const addValues = response.add(props);
                  setValues(addValues); // Updates for amauiTheme

                  const method = () => {
                    if (is$1('function', value)) {
                      var _response;

                      const valueNew = value(amauiTheme); // Update

                      if (((_response = response) === null || _response === void 0 ? void 0 : _response.update) !== undefined) response.update(valueNew);
                    }
                  };

                  if (amauiTheme) amauiTheme.subscriptions.update.subscribe(method); // Clean up

                  return () => {
                    var _addValues$ids;

                    // Unsubscribe
                    if (amauiTheme) amauiTheme.subscriptions.update.unsubscribe(method); // Remove

                    response.remove((_addValues$ids = addValues.ids) === null || _addValues$ids === void 0 ? void 0 : _addValues$ids.dynamic);
                  };
                }, []); // Update props

                React__default["default"].useEffect(() => {
                  if (response !== undefined && values.ids) response.props = {
                    ids: values.ids.dynamic,
                    props
                  };
                }, [hash$1(props)]);
                return values;
              }

              return usePure;
            }

            function style(value) {
              let options_ = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
              let response;
              let values_ = {};

              function useStyle(props) {
                const [values, setValues] = React__default["default"].useState(values_);
                const amauiStyle = useAmauiStyle();
                const amauiTheme = useAmauiTheme(); // Init only once
                // it has to be in body of method
                // as for ssr it actually calls the method
                // and it doesn't use hooks on ssr

                if (response === undefined) {
                  const options = {
                    amaui_style: {
                      value: undefined
                    },
                    amaui_theme: {
                      value: undefined
                    }
                  }; // AmauiStyle

                  if (amauiStyle !== undefined) options.amaui_style.value = amauiStyle; // AmauiTheme

                  if (amauiTheme !== undefined) options.amaui_theme.value = amauiTheme;
                  response = style$1(value, merge$1(options, options_, {
                    copy: true
                  })); // Update values for ssr as a priorty

                  values_ = names(response.amaui_style_sheet_manager.names);
                  setValues(values_);
                } // Add


                React__default["default"].useEffect(() => {
                  const addValues = response.add(props);
                  setValues(addValues); // Updates for amauiTheme

                  const method = () => {
                    if (is$1('function', value)) {
                      var _response;

                      const valueNew = value(amauiTheme); // Update

                      if (((_response = response) === null || _response === void 0 ? void 0 : _response.update) !== undefined) response.update(valueNew);
                    }
                  };

                  if (amauiTheme) amauiTheme.subscriptions.update.subscribe(method); // Clean up

                  return () => {
                    var _addValues$ids;

                    // Unsubscribe
                    if (amauiTheme) amauiTheme.subscriptions.update.unsubscribe(method); // Remove

                    response.remove((_addValues$ids = addValues.ids) === null || _addValues$ids === void 0 ? void 0 : _addValues$ids.dynamic);
                  };
                }, []); // Update props

                React__default["default"].useEffect(() => {
                  if (response !== undefined && values !== null && values !== void 0 && values.ids) response.props = {
                    ids: values.ids.dynamic,
                    props
                  };
                }, [hash$1(props)]);
                return values;
              }

              return useStyle;
            }

            function inline(value_, props) {
              let options_ = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {
                response: 'json'
              };
              const [value, setValue] = React__default["default"].useState(undefined);
              const amauiStyle = useAmauiStyle();
              const amauiTheme = useAmauiTheme();

              const update = function () {
                let update_ = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : true;
                const options = merge$1(options_, {
                  amaui_style: {
                    value: amauiStyle
                  },
                  amaui_theme: {
                    value: amauiTheme
                  }
                }, {
                  copy: true
                }); // Options response value

                options.response = 'json';
                const valueNew = inline$1(value_, props, options);
                if (update_) setValue(valueNew);
                return valueNew;
              }; // Update


              React__default["default"].useEffect(() => {
                update();
                if (amauiTheme) amauiTheme.subscriptions.update.subscribe(update); // Clean up

                return () => {
                  // Unsubscribe
                  if (amauiTheme) amauiTheme.subscriptions.update.unsubscribe(update);
                };
              }, []); // Update props

              React__default["default"].useEffect(() => {
                update();
              }, [hash$1(props)]); // Important for ssr value

              const value__ = value || update(false);
              return value__;
            }

            function string(value_) {
              for (var _len = arguments.length, args = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
                args[_key - 1] = arguments[_key];
              }

              const [response, setResponse] = React__default["default"].useState();
              const [values, setValues] = React__default["default"].useState({
                classes: {},
                classNames: {},
                keyframes: {},
                styles: () => {}
              });
              const amauiStyle = useAmauiStyle();
              const amauiTheme = useAmauiTheme();

              const method = () => value_.reduce((result, item, index) => result += "".concat(item).concat(args[index] || ''), '');

              const value = {
                a: method()
              };
              let values_ = values; // Init only once
              // it has to be in body of method
              // as for ssr it actually calls the method
              // and it doesn't use hooks on ssr

              if (response === undefined) {
                const options = {
                  amaui_style: {
                    value: undefined
                  },
                  amaui_theme: {
                    value: undefined
                  }
                }; // AmauiStyle

                if (amauiStyle !== undefined) options.amaui_style.value = amauiStyle; // AmauiTheme

                if (amauiTheme !== undefined) options.amaui_theme.value = amauiTheme;
                const response_ = style$1(value, options);
                setResponse(response_); // Update values for ssr as a priorty

                values_ = names(response_.amaui_style_sheet_manager.names);
                setValues(values_);
              }

              React__default["default"].useEffect(() => {
                // Add
                const addValues = response.add();
                setValues(addValues); // Clean up

                return () => {
                  var _addValues$ids;

                  // Remove
                  response.remove((_addValues$ids = addValues.ids) === null || _addValues$ids === void 0 ? void 0 : _addValues$ids.dynamic);
                };
              }, []); // Update
              // if values update based on
              // ui props or useAmauiTheme update

              React__default["default"].useEffect(() => {
                if ((response === null || response === void 0 ? void 0 : response.update) !== undefined) response.update(value);
              }, [value.a]);
              return values_.class || '';
            }

            function className(value_) {
              let props = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
              let className_ = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : '';
              let options_ = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {};
              const [response, setResponse] = React__default["default"].useState();
              const [values, setValues] = React__default["default"].useState({
                classes: {},
                classNames: {},
                keyframes: {},
                styles: () => {}
              });
              const amauiStyle = useAmauiStyle();
              const amauiTheme = useAmauiTheme();
              let value = value_;
              if (is$1('string', value_)) value = {
                a: value_
              };
              let values_ = values; // Init only once
              // it has to be in body of method
              // as for ssr it actually calls the method
              // and it doesn't use hooks on ssr

              if (response === undefined) {
                const options = {
                  amaui_style: {
                    value: undefined
                  },
                  amaui_theme: {
                    value: undefined
                  }
                }; // AmauiStyle

                if (amauiStyle !== undefined) options.amaui_style.value = amauiStyle; // AmauiTheme

                if (amauiTheme !== undefined) options.amaui_theme.value = amauiTheme;
                const response_ = style$1(value, merge$1(options, options_, {
                  copy: true
                }));
                setResponse(response_); // Update values for ssr as a priorty

                values_ = names(response_.amaui_style_sheet_manager.names);
                setValues(values_);
              }

              React__default["default"].useEffect(() => {
                // Add
                const addValues = response.add(props);
                setValues(addValues); // Updates for amauiTheme

                const method = () => {
                  if (is$1('function', value)) {
                    const valueNew = value(amauiTheme); // Update

                    if ((response === null || response === void 0 ? void 0 : response.update) !== undefined) response.update(valueNew);
                  }
                };

                if (amauiTheme) amauiTheme.subscriptions.update.subscribe(method); // Clean up

                return () => {
                  var _addValues$ids;

                  // Unsubscribe
                  if (amauiTheme) amauiTheme.subscriptions.update.unsubscribe(method); // Remove

                  response.remove((_addValues$ids = addValues.ids) === null || _addValues$ids === void 0 ? void 0 : _addValues$ids.dynamic);
                };
              }, []); // Update props

              React__default["default"].useEffect(() => {
                if (response !== undefined && values.ids) response.props = {
                  ids: values.ids.dynamic,
                  props
                };
              }, [hash$1(props)]);
              return values_.class && classNames([className_, values_.class]) || '';
            }

            const withStyle = Element => function (value) {
              let options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
              // Use styles
              const useStyle = style(value, options); // Element

              const element = /*#__PURE__*/React__default["default"].forwardRef((props, ref) => {
                const styles = useStyle(props);
                return /*#__PURE__*/React__default["default"].createElement(Element, _extends({
                  ref: ref,
                  styles: styles
                }, props));
              });
              return element;
            };

            var withStyle$1 = withStyle;

            const _excluded = ["children", "className"];

            const styled = Element => function (value) {
              let options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
              // Use styles
              const useStyle = style(value, options); // Element

              const element = /*#__PURE__*/React__default["default"].forwardRef((props, ref) => {
                const {
                  children,
                  className: classNameProp
                } = props,
                      other = _objectWithoutProperties(props, _excluded);

                const styles = useStyle(props);
                return /*#__PURE__*/React__default["default"].createElement(Element, _extends({
                  ref: ref,
                  className: classNames([classNameProp, styles.class])
                }, other), children);
              });
              return element;
            };

            var styled$1 = styled;

            exports.AmauiStyle = AmauiStyle$1;
            exports.AmauiStyleContext = AmauiStyleContext$1;
            exports.AmauiStyleProvider = AmauiStyleProvider;
            exports.AmauiTheme = AmauiTheme$1;
            exports.AmauiThemeContext = AmauiThemeContext$1;
            exports.AmauiThemeProvider = AmauiThemeProvider;
            exports.c = className;
            exports.className = className;
            exports.classNames = classNames;
            exports.colors = colors$1;
            exports.cs = className;
            exports.i = inline;
            exports.inline = inline;
            exports.makeClassName = makeClassName;
            exports.p = pure;
            exports.prefix = prefix;
            exports.pure = pure;
            exports.r = reset;
            exports.reset = reset;
            exports.rtl = rtl;
            exports.s = style;
            exports.sort = sort;
            exports.sr = string;
            exports.string = string;
            exports.style = style;
            exports.styled = styled$1;
            exports.sy = styled$1;
            exports.unit = unit;
            exports.useAmauiStyle = useAmauiStyle;
            exports.useAmauiTheme = useAmauiTheme;
            exports.valueObject = valueObject;
            exports.w = withStyle$1;
            exports.withStyle = withStyle$1;

            Object.defineProperty(exports, '__esModule', { value: true });

}));
